/********************************************************************
  APG - an ABNF Parser Generator
  Copyright  (C) 2007 Coast to Coast Research, Inc.
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program.  If not, see
  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
  or write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

    author: Lowell Thomas
            lowell@coasttocoastresearch.com
            http://www.coasttocoastresearch.com

   created: 8/25/2007 10:27
  filename: D:\Projects\Apgv5_0\ApgLib\Private.h
   purpose: Private APG library header file. Required for building
            APG static library only. Not require for building applications.
  
*********************************************************************/
#ifndef _Private_h_14_12_2006_10_25_
#define _Private_h_14_12_2006_10_25_

// component-dependent assertion macros
#undef VASSERT
#undef PASSERT
#undef CASSERT
#define VASSERT(cond) if(!(int)(cond)) {ulErrReport(spCtx->vpMemCtx, APG_VEC_ID, __LINE__, __FILE__); goto APG_EXIT;}
#define PASSERT(cond) if(!(int)(cond)) {ulErrReport(spCtx->vpMemCtx, APG_PAR_ID, __LINE__, __FILE__); goto APG_EXIT;}
#define CASSERT(cond) if(!(int)(cond)) {ulErrReport(spCtx->vpMemCtx, APG_CAT_ID, __LINE__, __FILE__); goto APG_EXIT;}

// operator function prototype
// void* vpCtx         - pointer to a parser context
// void* vpOp          - pointer to the opcode to be executed
// ulong ulChar        - offset of the first character of the sub-string to be matched
// ulong* ulpState     - returns operation state (EMPTY, MATCH or NOMATCH)
// ulong* ulpMatched   - returns the number of sub-string characters matched
typedef void (*PFN_OP)(void* vpCtx, void* vpOp, ulong ulChar, ulong* ulpState, ulong* ulpMatched);

//////////////////////////////////////////////////////////////////////////
// BIT MANIPULATION MACROS
//////////////////////////////////////////////////////////////////////////
// returns ulValue/4 rounded up to the nearest integer
#define ulCeiling4(ulValue) ((ulong)(ulValue))&3UL ? ((((ulong)(ulValue))>>2UL) + 1UL) : ((ulong)((ulValue))>>2UL)

// returns ulValue/8 rounded up to the nearest integer
#define ulCeiling8(ulValue) ((ulong)(ulValue))&7UL ? ((((ulong)(ulValue))>>3UL) + 1UL) : ((ulong)((ulValue))>>3UL)

// void vSetBit(uchar* ucpA, ulong ulIndex);
// uchar* ucpA   - pointer to an unsigned character array
// ulong ulIndex - index of a bit in the array
// set/clear the bit
static uchar _s_ucaBitMask_[8] = {0xFE, 0xFD, 0xFB, 0xF7, 0xEF, 0xDF, 0xBF, 0x7F};
#define vSetBit(ucpA, ulIndex) ((uchar*)(ucpA))[((ulong)(ulIndex))>>3UL] |= (1UL << (((ulong)(ulIndex))&7UL))
#define vClearBit(ucpA, ulIndex) ((uchar*)(ucpA))[((ulong)(ulIndex))>>3UL] &= _s_ucaBitMask_[(((ulong)(ulIndex))&7UL)]

// uchar* ucpA   - pointer to an unsigned character array
// ulong ulIndex - index of a bit in the array
// returns the value of the indexed bit
#define ulGetBit(ucpA, ulIndex) (ulong)(((((uchar*)(ucpA))[((ulong)(ulIndex))>>3UL] >> (((ulong)(ulIndex))&7UL)) & 1UL))

// uchar* ucpA   - pointer to an unsigned character array
// ulong ulIndex - index of a 2-bit in the array
// ulong ulValue - value to set the 2-bits to
//                 value must be <= 3 or result is undefined
// sets the two-bit value at index in a byte array
static uchar _s_ucaSetMask_[4] = {252,243,207,63};
#define vSetTwoBitValue(ucpA, ulIndex, ulValue)\
  ((uchar*)(ucpA))[((ulong)(ulIndex))>>2UL] =\
  (((uchar*)(ucpA))[((ulong)(ulIndex))>>2UL]&_s_ucaSetMask_[((ulong)(ulIndex))&3UL])|(uchar)(ulValue<<((((ulong)(ulIndex))&3UL)<<1UL))

// uchar* ucpA   - pointer to an unsigned character array
// ulong ulIndex - index of a 2-bit in the array
// returns the indexed two-bit value
static uchar _s_ucaGetMask_[4] = {3,12,48,192};
#define ulGetTwoBitValue(ucpA, ulIndex)\
  (ulong)(((((uchar*)(ucpA))[((ulong)(ulIndex))>>2UL] & _s_ucaGetMask_[((ulong)(ulIndex))&3UL]) >> ((((ulong)(ulIndex))&3UL)<<1UL)))

//////////////////////////////////////////////////////////////////////////
// PRIVATE ERROR COMPONENT
//////////////////////////////////////////////////////////////////////////
typedef struct
{
  ulong       ulAvailable;
  ulong       ulUsed;
  ulong       ulTotal;
  void*       vpValidate;
  void*       vpMemCtx;
  APG_ERR*  spErrors;
} APG_ERR_CTX;

//////////////////////////////////////////////////////////////////////////
// PRIVATE MEMORY COMPONENT
//////////////////////////////////////////////////////////////////////////
typedef struct
{
  ulong ulAllocations;
  ulong ulReAllocations;
  ulong ulFrees;
  ulong ulCells;
  ulong ulMaxCells;
  ulong ulHeapBytes;
  ulong ulMaxHeapBytes;
} APG_MEM_STATS;

// a memory cell
struct APG_MEM_CELL_STRUCT
{
  struct APG_MEM_CELL_STRUCT* spPrev;
  struct APG_MEM_CELL_STRUCT* spNext;
  ulong ulSize; // size (bytes) of memory allocation
  ulong ulSeq;  // sequence number of this cell
};

typedef struct APG_MEM_CELL_STRUCT APG_MEM_CELL;
typedef struct  
{
  ulong           ulActiveCellCount;  // number of cells on the active list
  PFN_ALLOCATOR   pfnAllocator;       // pointer to the allocator function
  PFN_DEALLOCATOR pfnDeAllocator;     // pointer to the de-allocator function
  APG_MEM_STATS   sStats;             // memory statistics
  APG_ERR_CTX*    spErrCtx;           // pointer to the err component
  void*           vpValidate;         // validation handle
  APG_MEM_CELL*   spActiveList;       // pointer to the first (and last) cell
                                      // in a circularly, doubly linked list
} APG_MEM_CTX;

//////////////////////////////////////////////////////////////////////////
// PRIVATE PARSER COMPONENT
//////////////////////////////////////////////////////////////////////////
// executable data header
// NOTE: there is a circular dependence between these structs and the generator (Apg.exe)
//       modification will break the generator
typedef struct 
{
  ulong ulLength;           // length of executable data (including this header)
  ulong ulCrc32;            // CRC32 hash to validate the data transfer
  ulong ulStartRule;        // the start rule - defines the grammar
  ulong ulRuleCount;        // number of rule names
  ulong ulOpcodeCount;      // total number of opcodes
  ulong ulCharSetSize;      // number of characters (alphabet symbol indexes) not including EOF character
  ulong ulEOFSetSize;       // number of characters (alphabet symbol indexes) including EOF character
  ulong ulEOFChar;          // EOF character value (ulEOFSetSize - 1)
  ulong ulMaxChar;          // maximum non-EOF character value (ulCharSetSize - 1)
  ulong ulPPT;              // true if PPTs are used
  
  // section location identifiers
  // all sections are aligned on 4-byte word boundaries
  // all section lengths are in bytes
  APG_BLOC sHeader;
  APG_BLOC sStrTbl;
  APG_BLOC sOpcodes;
  APG_BLOC sRuleList;
  APG_BLOC sPPT;
} APG_EXEHDR;

// parser context
typedef struct 
{
  // contexts
  void*       vpMemCtx;     // context of the inherited Memory component
  void*       vpPtNodes;    // vector of AST nodes
  void*       vpPtValues;   // vector of AST node values
  void*       vpValidate;   // used to validate the context

  // character set
  ulong       ulCharSetSize;// number of characters in the character set
  ulong       ulEOFChar;    // the actual character of the EOF character (ulCharSetSize)
  ulong       ulMaxChar;    // the largest non-EOF character value in the character set

  // rule list
  void*       vpRules;      // pointer to the first rule of the rule list
  ulong       ulRuleCount;  // number of rules in the grammar

  // parsing data
  APG_BSTR    sSrc;         // input string as a binary string
  void*       vpData;       // user-defined data passed to static call back functions
  ulong       ulPrevNode;   // place keeper for parent/child AST nodes
  ulong       ulSeq;        // unique identifier for AST values, incremented for each new value

  // trace & PPT options
  ulong       ulTrace;      // true turns tracing on (when _APG_CFG_DEBUG defined only)
  ulong       ulPPT;        // true if PPTs are used
  void*       vpPackrat;    // packrat context if _APG_CFG_PACKRAT is defined

  // state and stat collection
  APG_PARSER_STATE sState;  // final parser state
  APG_PARSER_STATS sStats;  // parser statistics, if needed
} APG_PARSER_CTX;

// rule list
typedef struct 
{
  char*         cpRuleName;     // pointer to the (null-terminated) ASCII rule name
  uchar*        ucpPPT;         // pointer to the PPT of the first opcode of the rule
  void*         vpOp;           // pointer to the first opcode of the rule
  ushort        usSyntax;       // true(>0) if a call back function is to be called for this rule during syntax analysis
  ushort        usSemantics;    // true(>0) if a call back function is to be called for this rule during semantic analysis
  PFN_CALLBACK  pfnCallBack;    // pointer to the call back function for this rule
} APG_RULES;

// for indexed identification of SABNF operators
#define OP_ID_RNM 0
#define OP_ID_ALT 1
#define OP_ID_CAT 2
#define OP_ID_REP 3
#define OP_ID_PRD 4
#define OP_ID_TRG 5
#define OP_ID_TBS 6
#define OP_ID_TLS 7
#define OP_ID_MAX 8

// NOTE: defined in Operators.c or OperatorsPPT.c, depending on whether using PPTs or not
ulong ulRuleListInit(APG_PARSER_CTX* spCtx, APG_EXEHDR* spExe, void* vpCallbacks);
ulong ulOpcodeInit(APG_PARSER_CTX* spCtx, APG_EXEHDR* spExe);
ulong ulPPTRuleListInit(APG_PARSER_CTX* spCtx, APG_EXEHDR* spExe, void* vpCallbacks);
ulong ulPPTOpcodeInit(APG_PARSER_CTX* spCtx, APG_EXEHDR* spExe);
void  vRunStartRule(APG_PARSER_CTX* spCtx, ulong ulStartRule, ulong* ulpState, ulong* ulpMatched);
void  vPPTRunStartRule(APG_PARSER_CTX* spCtx, ulong ulStartRule, ulong* ulpState, ulong* ulpMatched);

// operator tracing and statistics
// NOTE: defined in Nodes.c only if _APG_CFG_TRACE is defined
void vNodeTracePrint(APG_PARSER_CTX* spCtx, ulong ulChar,
                     void* vpOpcode, ulong ulPPTState, ulong ulState, ulong ulMatched);
void vNodeTraceBegin(APG_PARSER_CTX* spCtx, ulong ulChar, void* vpOpcode);
void vNodeTraceEnd(APG_PARSER_CTX* spCtx, ulong ulChar,
                   void* vpOpcode, ulong ulState, ulong ulMatched);
void vNodeStats(APG_PARSER_CTX* spCtx, ulong ulType, ulong ulID, ulong ulChar, ulong ulState, ulong ulMatched);
void vNodeRnmVisit(APG_PARSER_CTX* spCtx, ulong ulRuleIndex, ulong ulSuffix, ulong ulState); 
void vNodeRnmSetup(APG_PARSER_CTX* spCtx, ulong ulFunc); 

// Packrat table entry
typedef struct
{
  ulong ulState;
  ulong ulMatched;
  ulong ulCount;
} APG_PACKRAT;

typedef struct
{
  void* vpMemCtx;
  void* vpValidate;     // validation handle
  APG_PACKRAT* spTable;
  ulong ulRuleCount;
  ulong ulCharCount;
} APG_PACKRAT_CTX;

// packrat statistics collection
void* vpPackratCtor(void* vpCtx);
void  vPackratDtor(void* vpCtx);
void  vPackratCollect(void* vpCtx, ulong ulRuleIndex, ulong ulChar, ulong ulState, ulong ulMatched);
ulong ulPackratTableCheck(void* vpCtx, ulong ulRuleIndex, ulong ulChar, ulong* ulpState, ulong* ulpMatched);

//////////////////////////////////////////////////////////////////////////
// PRIVATE CATALOG COMPONENT
//////////////////////////////////////////////////////////////////////////
// AST nodes
// there are two APG_PT_NODE structs for each node in the AST
// one for the down branch direction and one for up
#define APG_PT_DOWN       0
#define APG_PT_UP         1
#define APG_PT_UNDEFINED  (-1)
typedef struct 
{
  ulong   ulDirection;  // APG_PT_UP or APG_PT_DOWN
  ulong   ulValue;      // index to the APG_PT_VALUE value
  ulong   ulPairedNode; // index to matching APG_PT_NODE
} APG_PT_NODE;

typedef struct 
{
  ulong    ulRuleIndex;   // index of this node's rule name
  ulong    ulParentNode;  // index of the parent's APG_PT_NODE
  ulong    ulIdentifier;  // unique identifier within this AST
  APG_BLOC sValue;        // location of the node's value (offset into input string and length) 
} APG_PT_VALUE;

typedef struct
{
  ulong     ulID;         // call back function ID of this phrase (NOT the grammar rule ID)
  ulong     ulIdentifier; // unique identifier, matches AST node value identifier
  char*     cpRuleName;   // null-termed rule name
  ulong     ulTreeDepth;  // AST depth
  ulong     ulChildCount; // number of child phrases in this phrase
  void*     vpChildList;  // pointer to the list of child phrases (type APG_PHRASE**)
  void*     vpParent;     // pointer to the parent phrase (type APG_PHRASE*)
  APG_BSTR  sPhrase;      // the phrase (character sub-string)
  ulong     ulIndex;      // used only during construction of the phrase lists
} APG_PHRASE;

typedef struct
{
  APG_PHRASE**  sppPhrase;  // pointer to the first phrase pointer in the list
  ulong         ulCount;    // number of occurrences in the list
  ulong         ulIndex;    // used only during construction of the phrase lists
} APG_RULE_ARRAY;

typedef struct
{
  APG_PARSER_CTX* spParser;       // parser component context
  void*           vpMemCtx;       // memory component context handle
//  void*           vpErrorCtx;     // error component context handle
  void*           vpRuleArrayCtx; // vector context handle of phrase lists for each rule ID
  void*           vpPhraseCtx;    // vector context handle of all phrases
  void*           vpListCtx;      // vector context handle of all lists of phrases
  ulong           ulCallBacks;    // number of call back functions defined
  ulong*          ulpGrammarToCatalog;   // map of grammar rule IDs to catalog phrase IDs
  ulong*          ulpCatalogToGrammar;   // map of catalog phrase IDs to grammar rule IDs
  APG_RULE_ARRAY* spRuleArray;    // pointer to the finished array of phrase info for each rule
  APG_PHRASE*     spPhrases;      // pointer to the list of all phrases
  APG_PHRASE**    sppLists;       // pointer to the list of all phrase lists
  void*           vpValidate;     // validation handle
} APG_CATALOG_CTX;

#endif // _Private_h_14_12_2006_10_25_
