/********************************************************************
  APG - an ABNF Parser Generator
  Copyright  (C) 2007 Coast to Coast Research, Inc.
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program.  If not, see
  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
  or write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  
	 created: Sat Nov 24 14:09:33 2007
	filename: Lexer.c
	  author: Lowell Thomas
	          lowell@coasttocoastresearch.com
	          http://www.coasttocoastresearch.com

	 purpose: Implementation of parser component Lexer.

*********************************************************************/
#include "Apg.h"

//{{Include.pre
#include "Generator.h"
//}}Include.pre

#include "Lexer.h"

//{{Include.post
//}}Include.post

//  GENERAL INFORMATION:
//       static data = 4720 (bytes)
//                   = 4.61E+00 (KB)
//                   = 4.50E-03 (MB)
//   character count = 128 (grammar alphabet character set)
// unused characters = 30 (grammar alphabet character set)
//        rule count = 52
//      opcode count = 232
//         ALT count = 19(67) (alternations)
//         CAT count = 29(76) (concatenations)
//         RNM count = 77
//         REP count = 37
//         PRD count = 0
//         TRG count = 66
//         TBS count = 4
//         TLS count = 0
//      string table = 372 (bytes)
//           opcodes = 3020 (bytes)
//         rule list = 1040 (bytes)
//              PPTs = <not used>

// CALL BACK FUNCTION DECLARATIONS
static ulong pfn_File(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_Line(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_Blank(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_LineError(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_Rule(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_NameDef(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_DefAlt(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_Def(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_NameDefError(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_DefError(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_OpAlt(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_OpNot(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_OpAnd(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_OpRparen(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_OpRoption(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_OpCat(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_OpLparen(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_OpLoption(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_OpRep(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_OpNumber(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_dRange(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_dString(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_dChar(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_xRange(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_xString(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_xChar(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_bRange(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_bString(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_bChar(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_Name(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_OpTls(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_ProsVal(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_TokenError(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_Token(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_OpDec(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_OpHex(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_OpBin(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_dmin(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_dmax(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_dnum(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_bmin(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_bmax(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_bnum(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_xmin(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_xmax(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_xnum(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_alpha(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_wsp(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_comment(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_LineContin(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_LineEnd(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_BlankLineEnd(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);

// CALL BACK FUNCTION POINTERS
static ulong g_ulCallbacks = 52;
static PFN_CALLBACK g_pfnCallbacks[52] = 
{
  pfn_File,
  pfn_Line,
  pfn_Blank,
  pfn_LineError,
  pfn_Rule,
  pfn_NameDef,
  pfn_DefAlt,
  pfn_Def,
  pfn_NameDefError,
  pfn_DefError,
  pfn_OpAlt,
  pfn_OpNot,
  pfn_OpAnd,
  pfn_OpRparen,
  pfn_OpRoption,
  pfn_OpCat,
  pfn_OpLparen,
  pfn_OpLoption,
  pfn_OpRep,
  pfn_OpNumber,
  pfn_dRange,
  pfn_dString,
  pfn_dChar,
  pfn_xRange,
  pfn_xString,
  pfn_xChar,
  pfn_bRange,
  pfn_bString,
  pfn_bChar,
  pfn_Name,
  pfn_OpTls,
  pfn_ProsVal,
  pfn_TokenError,
  pfn_Token,
  pfn_OpDec,
  pfn_OpHex,
  pfn_OpBin,
  pfn_dmin,
  pfn_dmax,
  pfn_dnum,
  pfn_bmin,
  pfn_bmax,
  pfn_bnum,
  pfn_xmin,
  pfn_xmax,
  pfn_xnum,
  pfn_alpha,
  pfn_wsp,
  pfn_comment,
  pfn_LineContin,
  pfn_LineEnd,
  pfn_BlankLineEnd,
};

// EXECUTABLE DATA
static ulong g_ulaExeData[1128] = 
{
  0x000011A0,0xE87200F4,0x00000000,0x00000034,0x000000E8,0x00000080,0x00000000,0x00000080,
  0x0000007F,0x00000000,0x00000000,0x00000050,0x00000050,0x00000174,0x000001C4,0x00000BCC,
  0x00000D90,0x00000410,0x00000000,0x00000000,0x656C6946,0x6E694C00,0x6C420065,0x006B6E61,
  0x656E694C,0x6F727245,0x75520072,0x4E00656C,0x44656D61,0x44006665,0x6C416665,0x2F3D0074,
  0x00666544,0x656D614E,0x45666544,0x726F7272,0x66654400,0x6F727245,0x704F0072,0x00746C41,
  0x6F4E704F,0x704F0074,0x00646E41,0x7052704F,0x6E657261,0x52704F00,0x6974706F,0x4F006E6F,
  0x74614370,0x4C704F00,0x65726170,0x704F006E,0x74706F4C,0x006E6F69,0x6552704F,0x704F0070,
  0x626D754E,0x64007265,0x676E6152,0x53640065,0x6E697274,0x43640067,0x00726168,0x6E615278,
  0x78006567,0x69727453,0x7800676E,0x72616843,0x61526200,0x0065676E,0x72745362,0x00676E69,
  0x61684362,0x614E0072,0x4F00656D,0x736C5470,0x6F725000,0x6C615673,0x6B6F5400,0x72456E65,
  0x00726F72,0x656B6F54,0x704F006E,0x00636544,0x6548704F,0x704F0078,0x006E6942,0x6E696D64,
  0x616D6400,0x6E640078,0x62006D75,0x006E696D,0x78616D62,0x756E6200,0x6D78006D,0x78006E69,
  0x0078616D,0x6D756E78,0x706C6100,0x77006168,0x63007073,0x656D6D6F,0x4C00746E,0x43656E69,
  0x69746E6F,0x0A0D006E,0x656E694C,0x00646E45,0x6C420A0D,0x4C6B6E61,0x45656E69,0x0D00646E,
  0x0000000A,0x00000003,0x0000001C,0x00000000,0xFFFFFFFF,0x00000000,0x0000001C,0x00000001,
  0x00000001,0x00000048,0x00000000,0x00000030,0x00000002,0x00000000,0x0000003C,0x00000004,
  0x00000000,0x00000048,0x00000003,0x00000002,0x00000098,0x00000003,0x00000070,0x00000000,
  0xFFFFFFFF,0x00000005,0x00000070,0x00000020,0x00000020,0x00000003,0x0000008C,0x00000000,
  0x00000001,0x00000000,0x0000008C,0x00000030,0x00000000,0x00000098,0x00000033,0x00000002,
  0x000000CC,0x00000003,0x000000C0,0x00000000,0xFFFFFFFF,0x00000005,0x000000C0,0x00000020,
  0x0000007F,0x00000000,0x000000CC,0x00000032,0x00000002,0x0000019C,0x00000001,0x000000F4,
  0x00000000,0x000000E8,0x00000005,0x00000000,0x000000F4,0x00000008,0x00000003,0x00000110,
  0x00000000,0xFFFFFFFF,0x00000000,0x00000110,0x0000002F,0x00000001,0x0000013C,0x00000000,
  0x00000124,0x00000006,0x00000000,0x00000130,0x00000007,0x00000000,0x0000013C,0x00000009,
  0x00000003,0x00000158,0x00000000,0xFFFFFFFF,0x00000000,0x00000158,0x0000002F,0x00000003,
  0x00000174,0x00000000,0xFFFFFFFF,0x00000000,0x00000174,0x00000021,0x00000003,0x00000190,
  0x00000000,0xFFFFFFFF,0x00000000,0x00000190,0x0000002F,0x00000000,0x0000019C,0x00000032,
  0x00000002,0x000001F4,0x00000000,0x000001B0,0x0000002E,0x00000003,0x000001F4,0x00000000,
  0xFFFFFFFF,0x00000001,0x000001F4,0x00000000,0x000001D4,0x0000002E,0x00000005,0x000001E4,
  0x00000030,0x00000039,0x00000005,0x000001F4,0x0000002D,0x0000002D,0x00000006,0x00000204,
  0x0000002E,0x00000002,0x00000005,0x00000214,0x0000003D,0x0000003D,0x00000003,0x00000234,
  0x00000001,0xFFFFFFFF,0x00000005,0x00000234,0x00000021,0x0000007F,0x00000003,0x00000254,
  0x00000001,0xFFFFFFFF,0x00000005,0x00000254,0x00000021,0x0000007F,0x00000002,0x000002A4,
  0x00000003,0x00000278,0x00000000,0xFFFFFFFF,0x00000000,0x00000278,0x0000002F,0x00000005,
  0x00000288,0x0000002F,0x0000002F,0x00000003,0x000002A4,0x00000000,0xFFFFFFFF,0x00000000,
  0x000002A4,0x0000002F,0x00000005,0x000002B4,0x00000021,0x00000021,0x00000005,0x000002C4,
  0x00000026,0x00000026,0x00000002,0x000002F8,0x00000003,0x000002E8,0x00000000,0xFFFFFFFF,
  0x00000000,0x000002E8,0x0000002F,0x00000005,0x000002F8,0x00000029,0x00000029,0x00000002,
  0x0000032C,0x00000003,0x0000031C,0x00000000,0xFFFFFFFF,0x00000000,0x0000031C,0x0000002F,
  0x00000005,0x0000032C,0x0000005D,0x0000005D,0x00000003,0x00000348,0x00000001,0xFFFFFFFF,
  0x00000000,0x00000348,0x0000002F,0x00000002,0x0000037C,0x00000005,0x00000360,0x00000028,
  0x00000028,0x00000003,0x0000037C,0x00000000,0xFFFFFFFF,0x00000000,0x0000037C,0x0000002F,
  0x00000002,0x000003B0,0x00000005,0x00000394,0x0000005B,0x0000005B,0x00000003,0x000003B0,
  0x00000000,0xFFFFFFFF,0x00000000,0x000003B0,0x0000002F,0x00000005,0x000003C0,0x0000002A,
  0x0000002A,0x00000003,0x000003E0,0x00000001,0xFFFFFFFF,0x00000005,0x000003E0,0x00000030,
  0x00000039,0x00000002,0x0000041C,0x00000000,0x000003F4,0x00000022,0x00000000,0x00000400,
  0x00000025,0x00000005,0x00000410,0x0000002D,0x0000002D,0x00000000,0x0000041C,0x00000026,
  0x00000002,0x00000470,0x00000000,0x00000430,0x00000022,0x00000000,0x0000043C,0x00000027,
  0x00000003,0x00000470,0x00000001,0xFFFFFFFF,0x00000002,0x00000470,0x00000005,0x00000464,
  0x0000002E,0x0000002E,0x00000000,0x00000470,0x00000027,0x00000002,0x00000490,0x00000000,
  0x00000484,0x00000022,0x00000000,0x00000490,0x00000025,0x00000002,0x000004CC,0x00000000,
  0x000004A4,0x00000023,0x00000000,0x000004B0,0x0000002B,0x00000005,0x000004C0,0x0000002D,
  0x0000002D,0x00000000,0x000004CC,0x0000002C,0x00000002,0x00000520,0x00000000,0x000004E0,
  0x00000023,0x00000000,0x000004EC,0x0000002D,0x00000003,0x00000520,0x00000001,0xFFFFFFFF,
  0x00000002,0x00000520,0x00000005,0x00000514,0x0000002E,0x0000002E,0x00000000,0x00000520,
  0x0000002D,0x00000002,0x00000540,0x00000000,0x00000534,0x00000023,0x00000000,0x00000540,
  0x0000002B,0x00000002,0x0000057C,0x00000000,0x00000554,0x00000024,0x00000000,0x00000560,
  0x00000028,0x00000005,0x00000570,0x0000002D,0x0000002D,0x00000000,0x0000057C,0x00000029,
  0x00000002,0x000005D0,0x00000000,0x00000590,0x00000024,0x00000000,0x0000059C,0x0000002A,
  0x00000003,0x000005D0,0x00000001,0xFFFFFFFF,0x00000002,0x000005D0,0x00000005,0x000005C4,
  0x0000002E,0x0000002E,0x00000000,0x000005D0,0x0000002A,0x00000002,0x000005F0,0x00000000,
  0x000005E4,0x00000024,0x00000000,0x000005F0,0x00000028,0x00000002,0x00000648,0x00000000,
  0x00000604,0x0000002E,0x00000003,0x00000648,0x00000000,0xFFFFFFFF,0x00000001,0x00000648,
  0x00000000,0x00000628,0x0000002E,0x00000005,0x00000638,0x00000030,0x00000039,0x00000005,
  0x00000648,0x0000002D,0x0000002D,0x00000002,0x000006A8,0x00000005,0x00000660,0x00000022,
  0x00000022,0x00000003,0x00000698,0x00000000,0xFFFFFFFF,0x00000001,0x00000698,0x00000005,
  0x00000688,0x00000020,0x00000021,0x00000005,0x00000698,0x00000023,0x0000007F,0x00000005,
  0x000006A8,0x00000022,0x00000022,0x00000002,0x00000708,0x00000005,0x000006C0,0x0000003C,
  0x0000003C,0x00000003,0x000006F8,0x00000000,0xFFFFFFFF,0x00000001,0x000006F8,0x00000005,
  0x000006E8,0x00000020,0x0000003D,0x00000005,0x000006F8,0x0000003F,0x0000007F,0x00000005,
  0x00000708,0x0000003E,0x0000003E,0x00000003,0x00000728,0x00000001,0xFFFFFFFF,0x00000005,
  0x00000728,0x00000021,0x0000007F,0x00000001,0x00000844,0x00000000,0x0000073C,0x0000000A,
  0x00000000,0x00000748,0x0000000C,0x00000000,0x00000754,0x0000000B,0x00000000,0x00000760,
  0x0000000D,0x00000000,0x0000076C,0x0000000E,0x00000000,0x00000778,0x0000000F,0x00000000,
  0x00000784,0x00000010,0x00000000,0x00000790,0x00000011,0x00000000,0x0000079C,0x00000012,
  0x00000000,0x000007A8,0x00000013,0x00000000,0x000007B4,0x00000014,0x00000000,0x000007C0,
  0x00000015,0x00000000,0x000007CC,0x00000016,0x00000000,0x000007D8,0x00000017,0x00000000,
  0x000007E4,0x00000018,0x00000000,0x000007F0,0x00000019,0x00000000,0x000007FC,0x0000001A,
  0x00000000,0x00000808,0x0000001B,0x00000000,0x00000814,0x0000001C,0x00000000,0x00000820,
  0x0000001E,0x00000000,0x0000082C,0x0000001D,0x00000000,0x00000838,0x0000001F,0x00000000,
  0x00000844,0x00000020,0x00000002,0x00000884,0x00000005,0x0000085C,0x00000025,0x00000025,
  0x00000001,0x00000884,0x00000005,0x00000874,0x00000044,0x00000044,0x00000005,0x00000884,
  0x00000064,0x00000064,0x00000002,0x000008C4,0x00000005,0x0000089C,0x00000025,0x00000025,
  0x00000001,0x000008C4,0x00000005,0x000008B4,0x00000058,0x00000058,0x00000005,0x000008C4,
  0x00000078,0x00000078,0x00000002,0x00000904,0x00000005,0x000008DC,0x00000025,0x00000025,
  0x00000001,0x00000904,0x00000005,0x000008F4,0x00000042,0x00000042,0x00000005,0x00000904,
  0x00000062,0x00000062,0x00000003,0x00000924,0x00000001,0xFFFFFFFF,0x00000005,0x00000924,
  0x00000030,0x00000039,0x00000003,0x00000944,0x00000001,0xFFFFFFFF,0x00000005,0x00000944,
  0x00000030,0x00000039,0x00000003,0x00000964,0x00000001,0xFFFFFFFF,0x00000005,0x00000964,
  0x00000030,0x00000039,0x00000003,0x00000984,0x00000001,0xFFFFFFFF,0x00000005,0x00000984,
  0x00000030,0x00000031,0x00000003,0x000009A4,0x00000001,0xFFFFFFFF,0x00000005,0x000009A4,
  0x00000030,0x00000031,0x00000003,0x000009C4,0x00000001,0xFFFFFFFF,0x00000005,0x000009C4,
  0x00000030,0x00000031,0x00000003,0x00000A0C,0x00000001,0xFFFFFFFF,0x00000001,0x00000A0C,
  0x00000005,0x000009EC,0x00000030,0x00000039,0x00000005,0x000009FC,0x00000041,0x00000046,
  0x00000005,0x00000A0C,0x00000061,0x00000066,0x00000003,0x00000A54,0x00000001,0xFFFFFFFF,
  0x00000001,0x00000A54,0x00000005,0x00000A34,0x00000030,0x00000039,0x00000005,0x00000A44,
  0x00000041,0x00000046,0x00000005,0x00000A54,0x00000061,0x00000066,0x00000003,0x00000A9C,
  0x00000001,0xFFFFFFFF,0x00000001,0x00000A9C,0x00000005,0x00000A7C,0x00000030,0x00000039,
  0x00000005,0x00000A8C,0x00000041,0x00000046,0x00000005,0x00000A9C,0x00000061,0x00000066,
  0x00000001,0x00000AC4,0x00000005,0x00000AB4,0x00000061,0x0000007A,0x00000005,0x00000AC4,
  0x00000041,0x0000005A,0x00000001,0x00000AF4,0x00000005,0x00000ADC,0x00000020,0x00000020,
  0x00000000,0x00000AE8,0x00000030,0x00000000,0x00000AF4,0x00000031,0x00000002,0x00000B2C,
  0x00000005,0x00000B0C,0x0000003B,0x0000003B,0x00000003,0x00000B2C,0x00000000,0xFFFFFFFF,
  0x00000005,0x00000B2C,0x00000020,0x0000007F,0x00000002,0x00000B7C,0x00000001,0x00000B5C,
  0x00000006,0x00000B4C,0x00000156,0x00000002,0x00000005,0x00000B5C,0x0000000D,0x0000000D,
  0x00000003,0x00000B7C,0x00000001,0xFFFFFFFF,0x00000005,0x00000B7C,0x00000020,0x00000020,
  0x00000001,0x00000BA4,0x00000006,0x00000B94,0x00000160,0x00000002,0x00000005,0x00000BA4,
  0x0000000D,0x0000000D,0x00000001,0x00000BCC,0x00000006,0x00000BBC,0x0000016F,0x00000002,
  0x00000005,0x00000BCC,0x0000000D,0x0000000D,0x00000000,0x00000000,0x00000000,0x00010001,
  0x00000000,0x00000005,0x00000000,0x0000001C,0x00010001,0x00000001,0x0000000A,0x00000000,
  0x00000048,0x00010001,0x00000002,0x00000010,0x00000000,0x00000098,0x00010001,0x00000003,
  0x0000001A,0x00000000,0x000000CC,0x00010001,0x00000004,0x0000001F,0x00000000,0x0000019C,
  0x00010001,0x00000005,0x00000027,0x00000000,0x000001F4,0x00010001,0x00000006,0x00000030,
  0x00000000,0x00000204,0x00010001,0x00000007,0x00000034,0x00000000,0x00000214,0x00010001,
  0x00000008,0x00000041,0x00000000,0x00000234,0x00010001,0x00000009,0x0000004A,0x00000000,
  0x00000254,0x00010001,0x0000000A,0x00000050,0x00000000,0x000002A4,0x00010001,0x0000000B,
  0x00000056,0x00000000,0x000002B4,0x00010001,0x0000000C,0x0000005C,0x00000000,0x000002C4,
  0x00010001,0x0000000D,0x00000065,0x00000000,0x000002F8,0x00010001,0x0000000E,0x0000006F,
  0x00000000,0x0000032C,0x00010001,0x0000000F,0x00000075,0x00000000,0x00000348,0x00010001,
  0x00000010,0x0000007E,0x00000000,0x0000037C,0x00010001,0x00000011,0x00000088,0x00000000,
  0x000003B0,0x00010001,0x00000012,0x0000008E,0x00000000,0x000003C0,0x00010001,0x00000013,
  0x00000097,0x00000000,0x000003E0,0x00010001,0x00000014,0x0000009E,0x00000000,0x0000041C,
  0x00010001,0x00000015,0x000000A6,0x00000000,0x00000470,0x00010001,0x00000016,0x000000AC,
  0x00000000,0x00000490,0x00010001,0x00000017,0x000000B3,0x00000000,0x000004CC,0x00010001,
  0x00000018,0x000000BB,0x00000000,0x00000520,0x00010001,0x00000019,0x000000C1,0x00000000,
  0x00000540,0x00010001,0x0000001A,0x000000C8,0x00000000,0x0000057C,0x00010001,0x0000001B,
  0x000000D0,0x00000000,0x000005D0,0x00010001,0x0000001C,0x000000D6,0x00000000,0x000005F0,
  0x00010001,0x0000001D,0x000000DB,0x00000000,0x00000648,0x00010001,0x0000001E,0x000000E1,
  0x00000000,0x000006A8,0x00010001,0x0000001F,0x000000E9,0x00000000,0x00000708,0x00010001,
  0x00000020,0x000000F4,0x00000000,0x00000728,0x00010001,0x00000021,0x000000FA,0x00000000,
  0x00000844,0x00010001,0x00000022,0x00000100,0x00000000,0x00000884,0x00010001,0x00000023,
  0x00000106,0x00000000,0x000008C4,0x00010001,0x00000024,0x0000010C,0x00000000,0x00000904,
  0x00010001,0x00000025,0x00000111,0x00000000,0x00000924,0x00010001,0x00000026,0x00000116,
  0x00000000,0x00000944,0x00010001,0x00000027,0x0000011B,0x00000000,0x00000964,0x00010001,
  0x00000028,0x00000120,0x00000000,0x00000984,0x00010001,0x00000029,0x00000125,0x00000000,
  0x000009A4,0x00010001,0x0000002A,0x0000012A,0x00000000,0x000009C4,0x00010001,0x0000002B,
  0x0000012F,0x00000000,0x00000A0C,0x00010001,0x0000002C,0x00000134,0x00000000,0x00000A54,
  0x00010001,0x0000002D,0x00000139,0x00000000,0x00000A9C,0x00010001,0x0000002E,0x0000013F,
  0x00000000,0x00000AC4,0x00010001,0x0000002F,0x00000143,0x00000000,0x00000AF4,0x00010001,
  0x00000030,0x0000014B,0x00000000,0x00000B2C,0x00010001,0x00000031,0x00000158,0x00000000,
  0x00000B7C,0x00010001,0x00000032,0x00000162,0x00000000,0x00000BA4,0x00010001,0x00000033,
};

// UNIQUELY NAMED PARSER INITIALIZATION DATA
static APG_PARSER_INIT g_sParserInit = 
{
  0,
  (void*)g_pfnCallbacks,
  (void*)g_ulaExeData
};

void* vpLexerInit = (void*)&g_sParserInit;

// switch template
/*
  switch(ulState)
  {
  case SYN_PRE:
    break;
  case SYN_NOMATCH:
    break;
  case SYN_EMPTY:
    break;
  case SYN_MATCH:
    break;
  case SEM_PRE:
    break;
  case SEM_POST:
    break;
  }
*/

//{{CallBack.body
static ulong ulDec(char* cpSrc, ulong ulLen)
{
  char* cpBeg = cpSrc;
  char* cpEnd = cpBeg + ulLen;
  ulong ulNum = 0;
  for(; cpBeg < cpEnd; cpBeg++)
  {
    ulNum *= 10;
    ulNum += (ulong)(*cpBeg - 48);
  }
  
  return ulNum;
}

static ulong ulBin(char* cpSrc, ulong ulLen)
{
  char* cpBeg = cpSrc;
  char* cpEnd = cpBeg + ulLen;
  ulong ulNum = 0;
  for(; cpBeg < cpEnd; cpBeg++)
  {
    ulNum <<= 1;
    ulNum += (ulong)(*cpBeg - 48);
  }
  
  return ulNum;
}

static ulong g_ulDigits[55]=
{
  0,1,2,3,4,5,6,7,8,9,
    -1,-1,-1,-1,-1,-1,-1,
    10,11,12,13,14,15,
    -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
    -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
    10,11,12,13,14,15
};
static ulong ulHex(char* cpSrc, ulong ulLen)
{
  char* cpBeg = cpSrc;
  char* cpEnd = cpBeg + ulLen;
  ulong ulNum = 0;
  for(; cpBeg < cpEnd; cpBeg++)
  {
    ulNum <<= 4;
    ulNum += g_ulDigits[*cpBeg - 48];
  }
  
  return ulNum;
}

// looks for a rule name in the rule list
// does a brute force, case-insensitive linear search
static ulong ulRuleLookup(GENCTX* spData, const uchar* ucpSrc, ulong ulSrcLen)
{
  ulong ulReturn = UNDEFINED;
  uchar* ucpStrTbl = (uchar*)vpVecFront(spData->vpStrTblCtx);
  INT_RULE_LIST* spRuleList = (INT_RULE_LIST*)vpVecFront(spData->vpIntRuleCtx);
  INT_RULE_LIST* spBeg = spRuleList;
  INT_RULE_LIST* spEnd = spBeg + ulVecSize(spData->vpIntRuleCtx);
  ulong ulIndex = 0;
  for(; spBeg < spEnd; ++spBeg, ++ulIndex)
  {
    // compare name lengths
    if(ulSrcLen == spBeg->ulNameLen)
    {
      // compare lower case characters
      const uchar* ucpR = ucpSrc;
      uchar* ucpL = ucpStrTbl + spBeg->ulNameOffset;
      uchar* ucpEnd = ucpL + ulSrcLen;
      for(; ucpL < ucpEnd; ++ucpL, ++ucpR)
      {
        uchar ucL = *ucpL;
        uchar ucR = *ucpR;
        if(ucL >= 65 && ucL <= 90){ucL += 32;}
        if(ucR >= 65 && ucR <= 90){ucR += 32;}
        if(ucL != ucR){break;}
      }
      
      if(ucpL == ucpEnd)
      {
        // match found
        ulReturn = ulIndex;
        break;
      }
    }
    // else - no match
  }
  
  return ulReturn;
}


static ulong ulAddToRuleList(GENCTX* spData, void* vpCtx, ulong ulOffset, ulong ulLen, ulong ulLineNo)
{
  ulong ulIndex = ulVecSize(vpCtx);
  INT_RULE_LIST* spRule = (INT_RULE_LIST*)vpVecPush(vpCtx, NULL);
  memset((void*)spRule, 0, sizeof(*spRule));
  spRule->ulLineNo = ulLineNo;
  spRule->ulNameOffset = ulOffset;
  spRule->ulNameLen = ulLen;
  spRule->usSyntax = 1;
  spRule->usSemantics = 1;
  return ulIndex;
}
/*
// get data pointer
GENCTX* spData = (GENCTX*)vpData;
switch(ulState)
{
case SEM_PRE:
  break;
case SEM_POST:
  break;
}
*/
//}}CallBack.body

// call back function for rule "File"
static ulong pfn_File(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_File
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    break;
  case SEM_POST:
    break;
  }
//}}CallBack.pfn_File
  return ulReturn;
}

// call back function for rule "Line"
static ulong pfn_Line(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_Line
//}}CallBack.pfn_Line
  return ulReturn;
}

// call back function for rule "Blank"
static ulong pfn_Blank(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_Blank
//}}CallBack.pfn_Blank
  return ulReturn;
}

// call back function for rule "LineError"
static ulong pfn_LineError(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_LineError
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    break;
  case SEM_POST:
    {
      // generate an error token
      SABNFTOKEN sTok;
      uchar ucIdx;
      memset((void*)&sTok, 0, sizeof(sTok));
      sTok.ulID = TOK_ERROR;
      ucIdx = (uchar)sTok.ulID;
      sTok.ulLineNo = spData->ulLineNo - 1;
      vpVecPush(spData->vpTokTblCtx, (void*)&sTok);
      vpVecPush(spData->vpTokIdxCtx, (void*)&ucIdx);
      
      // generate an error message
      ulGrammarErrorsReport(spData->vpErrorsCtx, spData->ulLineNo - 1, "Invalid line found.");
    };
    break;
  }
//}}CallBack.pfn_LineError
  return ulReturn;
}

// call back function for rule "Rule"
static ulong pfn_Rule(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_Rule
//}}CallBack.pfn_Rule
  return ulReturn;
}

// call back function for rule "NameDef"
static ulong pfn_NameDef(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_NameDef
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    break;
  case SEM_POST:
    {
      // declarations
      SABNFTOKEN  sTok;
      uchar       ucIdx;
      
      // initialize the name def token
      memset((void*)&sTok, 0, sizeof(sTok));
      sTok.ulLineNo = spData->ulLineNo;
      
      // always generate a name def token
      sTok.ulID = TOK_NAMEDEF;
      ucIdx = (uchar)sTok.ulID;
      sTok.ulOffset = ulOffset;
      sTok.ulLen = ulLen;
      vpVecPush(spData->vpTokTblCtx, (void*)&sTok);
      vpVecPush(spData->vpTokIdxCtx, (void*)&ucIdx);
    };
    break;
  }
//}}CallBack.pfn_NameDef
  return ulReturn;
}

// call back function for rule "DefAlt"
static ulong pfn_DefAlt(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_DefAlt
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    break;
  case SEM_POST:
    {
      // declarations
      SABNFTOKEN* spNameDefTok;
      SABNFTOKEN sTok;
      uchar ucIdx;
      ulong ulIndex;
      uchar ucZero = 0;
      
      // get the NameDef token
      spNameDefTok = (SABNFTOKEN*)vpVecBack(spData->vpTokTblCtx);
      if(spNameDefTok->ulID == TOK_NAMEDEF)
      {
        // normal processing
        // look up the name - validate name is NOT unique
        ulIndex = ulRuleLookup(spData, spData->ucpSrc + spNameDefTok->ulOffset, spNameDefTok->ulLen);
        if(ulIndex != UNDEFINED)
        {
          // generate a incremental alternate token
          memset((void*)&sTok, 0, sizeof(sTok));
          sTok.ulID = TOK_DEFALT;
          ucIdx = (uchar)sTok.ulID;
          sTok.ulLineNo = spData->ulLineNo;
          sTok.ulRuleIndex = ulIndex;
          vpVecPush(spData->vpTokTblCtx, (void*)&sTok);
          vpVecPush(spData->vpTokIdxCtx, (void*)&ucIdx);
        }
        else
        {
          // handle rule name error
          // generate an error message
          ulGrammarErrorsReport(spData->vpErrorsCtx, spNameDefTok->ulLineNo - 1,
            "Incremental alternate found before rule name was defined.");
        }
      }
      else if(spNameDefTok->ulID == TOK_ERROR)
      {
        // error has already been noted - just push another error token
        memset((void*)&sTok, 0, sizeof(sTok));
        sTok.ulID = TOK_ERROR;
        ucIdx = (uchar)sTok.ulID;
        sTok.ulLineNo = spData->ulLineNo;
        vpVecPush(spData->vpTokTblCtx, (void*)&sTok);
        vpVecPush(spData->vpTokIdxCtx, (void*)&ucIdx);
      }
      else
      {
        // generate an error message
        ulGrammarErrorsReport(spData->vpErrorsCtx, spNameDefTok->ulLineNo - 1,
          "Incremental alternate not preceded by rule name token.");
      }
    };
    break;
  }
//}}CallBack.pfn_DefAlt
  return ulReturn;
}

// call back function for rule "Def"
static ulong pfn_Def(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_Def
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    break;
  case SEM_POST:
    {
      // declarations
      SABNFTOKEN* spNameDefTok;
      SABNFTOKEN sTok;
      uchar ucIdx;
      ulong ulIndex;
      uchar ucZero = 0;
      ulong ulNameOffset;
      
      // get the NameDef token
      spNameDefTok = (SABNFTOKEN*)vpVecBack(spData->vpTokTblCtx);
      if(spNameDefTok->ulID == TOK_NAMEDEF)
      {
        // normal processing
        // look up the name - validate name is unique
        ulIndex = ulRuleLookup(spData, spData->ucpSrc + spNameDefTok->ulOffset, spNameDefTok->ulLen);
        if(ulIndex == UNDEFINED)
        {
          // add to string table
          ulNameOffset = ulVecSize(spData->vpStrTblCtx);
          vpVecPushn(spData->vpStrTblCtx, (void*)(spData->ucpSrc + spNameDefTok->ulOffset), spNameDefTok->ulLen);
          vpVecPush(spData->vpStrTblCtx, (void*)&ucZero);
          
          // add to rule list
          ulIndex = ulAddToRuleList(spData, spData->vpIntRuleCtx, ulNameOffset, spNameDefTok->ulLen, spNameDefTok->ulLineNo);
          
          // generate a defined-as token
          memset((void*)&sTok, 0, sizeof(sTok));
          sTok.ulID = TOK_DEF;
          ucIdx = (uchar)sTok.ulID;
          sTok.ulLineNo = spData->ulLineNo;
          sTok.ulRuleIndex = ulIndex;
          vpVecPush(spData->vpTokTblCtx, (void*)&sTok);
          vpVecPush(spData->vpTokIdxCtx, (void*)&ucIdx);
        }
        else
        {
          // handle rule name error
          // generate an error message
          ulGrammarErrorsReport(spData->vpErrorsCtx, spNameDefTok->ulLineNo - 1,
            "Rule name multiply defined.");
        }
      }
      else if(spNameDefTok->ulID == TOK_ERROR)
      {
        // error has already been noted - just push another error token
        memset((void*)&sTok, 0, sizeof(sTok));
        sTok.ulID = TOK_ERROR;
        ucIdx = (uchar)sTok.ulID;
        sTok.ulLineNo = spData->ulLineNo;
        vpVecPush(spData->vpTokTblCtx, (void*)&sTok);
        vpVecPush(spData->vpTokIdxCtx, (void*)&ucIdx);
      }
      else
      {
        // generate an error message
        ulGrammarErrorsReport(spData->vpErrorsCtx, spNameDefTok->ulLineNo - 1,
          "Def not preceded by rule name token.");
      }
    };
    break;
  }
//}}CallBack.pfn_Def
  return ulReturn;
}

// call back function for rule "NameDefError"
static ulong pfn_NameDefError(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_NameDefError
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    break;
  case SEM_POST:
    {
      SABNFTOKEN sTok;
      uchar ucIdx;
      // generate an error message
      ulGrammarErrorsReport(spData->vpErrorsCtx, spData->ulLineNo - 1,
        "Invalid rule name definition found.");
      
      // generate an error token
      memset((void*)&sTok, 0, sizeof(sTok));
      sTok.ulID = TOK_ERROR;
      ucIdx = (uchar)sTok.ulID;
      sTok.ulLineNo = spData->ulLineNo;
      vpVecPush(spData->vpTokTblCtx, (void*)&sTok);
      vpVecPush(spData->vpTokIdxCtx, (void*)&ucIdx);
    };
    break;
  }
//}}CallBack.pfn_NameDefError
  return ulReturn;
}

// call back function for rule "DefError"
static ulong pfn_DefError(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_DefError
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    break;
  case SEM_POST:
    {
      SABNFTOKEN sTok;
      uchar ucIdx;
      // generate an error message
      ulGrammarErrorsReport(spData->vpErrorsCtx, spData->ulLineNo - 1,
        "Invalid 'defined-as' found.");
      
      // generate an error token
      memset((void*)&sTok, 0, sizeof(sTok));
      sTok.ulID = TOK_ERROR;
      ucIdx = (uchar)sTok.ulID;
      sTok.ulLineNo = spData->ulLineNo;
      vpVecPush(spData->vpTokTblCtx, (void*)&sTok);
      vpVecPush(spData->vpTokIdxCtx, (void*)&ucIdx);
    };
    break;
  }
//}}CallBack.pfn_DefError
  return ulReturn;
}

// call back function for rule "OpAlt"
static ulong pfn_OpAlt(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_OpAlt
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    break;
  case SEM_POST:
    {
      SABNFTOKEN sTok;
      uchar ucIdx;
      memset((void*)&sTok, 0, sizeof(sTok));
      sTok.ulID = TOK_ALT;
      ucIdx = (uchar)sTok.ulID;
      sTok.ulLineNo = spData->ulLineNo;
      vpVecPush(spData->vpTokTblCtx, (void*)&sTok);
      vpVecPush(spData->vpTokIdxCtx, (void*)&ucIdx);
    };
    break;
  }
//}}CallBack.pfn_OpAlt
  return ulReturn;
}

// call back function for rule "OpNot"
static ulong pfn_OpNot(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_OpNot
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    break;
  case SEM_POST:
    {
      SABNFTOKEN sTok;
      uchar ucIdx;
      memset((void*)&sTok, 0, sizeof(sTok));
      sTok.ulID = TOK_NOT;
      ucIdx = (uchar)sTok.ulID;
      sTok.ulLineNo = spData->ulLineNo;
      vpVecPush(spData->vpTokTblCtx, (void*)&sTok);
      vpVecPush(spData->vpTokIdxCtx, (void*)&ucIdx);
    };
    break;
  }
//}}CallBack.pfn_OpNot
  return ulReturn;
}

// call back function for rule "OpAnd"
static ulong pfn_OpAnd(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_OpAnd
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    break;
  case SEM_POST:
    {
      SABNFTOKEN sTok;
      uchar ucIdx;
      memset((void*)&sTok, 0, sizeof(sTok));
      sTok.ulID = TOK_AND;
      ucIdx = (uchar)sTok.ulID;
      sTok.ulLineNo = spData->ulLineNo;
      vpVecPush(spData->vpTokTblCtx, (void*)&sTok);
      vpVecPush(spData->vpTokIdxCtx, (void*)&ucIdx);
    };
    break;
  }
//}}CallBack.pfn_OpAnd
  return ulReturn;
}

// call back function for rule "OpRparen"
static ulong pfn_OpRparen(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_OpRparen
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    break;
  case SEM_POST:
    {
      SABNFTOKEN sTok;
      uchar ucIdx;
      memset((void*)&sTok, 0, sizeof(sTok));
      sTok.ulID = TOK_RPAREN;
      ucIdx = (uchar)sTok.ulID;
      sTok.ulLineNo = spData->ulLineNo;
      vpVecPush(spData->vpTokTblCtx, (void*)&sTok);
      vpVecPush(spData->vpTokIdxCtx, (void*)&ucIdx);
    }
    break;
  }
//}}CallBack.pfn_OpRparen
  return ulReturn;
}

// call back function for rule "OpRoption"
static ulong pfn_OpRoption(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_OpRoption
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    break;
  case SEM_POST:
    {
      SABNFTOKEN sTok;
      uchar ucIdx;
      memset((void*)&sTok, 0, sizeof(sTok));
      sTok.ulID = TOK_ROPTION;
      ucIdx = (uchar)sTok.ulID;
      sTok.ulLineNo = spData->ulLineNo;
      vpVecPush(spData->vpTokTblCtx, (void*)&sTok);
      vpVecPush(spData->vpTokIdxCtx, (void*)&ucIdx);
    }
    break;
  }
//}}CallBack.pfn_OpRoption
  return ulReturn;
}

// call back function for rule "OpCat"
static ulong pfn_OpCat(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_OpCat
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    break;
  case SEM_POST:
    {
      SABNFTOKEN sTok;
      uchar ucIdx;
      memset((void*)&sTok, 0, sizeof(sTok));
      sTok.ulID = TOK_CAT;
      ucIdx = (uchar)sTok.ulID;
      sTok.ulLineNo = spData->ulLineNo;
      vpVecPush(spData->vpTokTblCtx, (void*)&sTok);
      vpVecPush(spData->vpTokIdxCtx, (void*)&ucIdx);
    };
    break;
  }
//}}CallBack.pfn_OpCat
  return ulReturn;
}

// call back function for rule "OpLparen"
static ulong pfn_OpLparen(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_OpLparen
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    break;
  case SEM_POST:
    {
      SABNFTOKEN sTok;
      uchar ucIdx;
      memset((void*)&sTok, 0, sizeof(sTok));
      sTok.ulID = TOK_LPAREN;
      ucIdx = (uchar)sTok.ulID;
      sTok.ulLineNo = spData->ulLineNo;
      vpVecPush(spData->vpTokTblCtx, (void*)&sTok);
      vpVecPush(spData->vpTokIdxCtx, (void*)&ucIdx);
    }
    break;
  }
//}}CallBack.pfn_OpLparen
  return ulReturn;
}

// call back function for rule "OpLoption"
static ulong pfn_OpLoption(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_OpLoption
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    break;
  case SEM_POST:
    {
      SABNFTOKEN sTok;
      uchar ucIdx;
      memset((void*)&sTok, 0, sizeof(sTok));
      sTok.ulID = TOK_LOPTION;
      ucIdx = (uchar)sTok.ulID;
      sTok.ulLineNo = spData->ulLineNo;
      vpVecPush(spData->vpTokTblCtx, (void*)&sTok);
      vpVecPush(spData->vpTokIdxCtx, (void*)&ucIdx);
    }
    break;
  }
//}}CallBack.pfn_OpLoption
  return ulReturn;
}

// call back function for rule "OpRep"
static ulong pfn_OpRep(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_OpRep
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    break;
  case SEM_POST:
    {
      SABNFTOKEN sTok;
      uchar ucIdx;
      memset((void*)&sTok, 0, sizeof(sTok));
      sTok.ulID = TOK_REP;
      ucIdx = (uchar)sTok.ulID;
      sTok.ulLineNo = spData->ulLineNo;
      vpVecPush(spData->vpTokTblCtx, (void*)&sTok);
      vpVecPush(spData->vpTokIdxCtx, (void*)&ucIdx);
    };
    break;
  }
//}}CallBack.pfn_OpRep
  return ulReturn;
}

// call back function for rule "OpNumber"
static ulong pfn_OpNumber(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_OpNumber
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    break;
  case SEM_POST:
    {
      ulong ulNum = ulDec((char*)(spData->ucpSrc + ulOffset), ulLen);
      uchar ucId = (uchar)TOK_NUM;
      memset((void*)&spData->sToken, 0, sizeof(spData->sToken));
      spData->sToken.ulMin = ulNum;
      spData->sToken.ulMax = ulNum;
      spData->sToken.ulID = TOK_NUM;
      spData->sToken.ulLineNo = spData->ulLineNo;
      vpVecPush(spData->vpTokTblCtx, (void*)&spData->sToken);
      vpVecPush(spData->vpTokIdxCtx, (void*)&ucId);
    }
    break;
  }
//}}CallBack.pfn_OpNumber
  return ulReturn;
}

// call back function for rule "dRange"
static ulong pfn_dRange(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_dRange
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    {
      // initialize the token
      memset((void*)&spData->sToken, 0, sizeof(spData->sToken));
      spData->sToken.ulID = TOK_TRG;
      spData->sToken.ulLineNo = spData->ulLineNo;
    }
    break;
  case SEM_POST:
    {
      // push the token on the token tables
      uchar ucIdx = (uchar)spData->sToken.ulID;
      if(ucIdx == TOK_ERROR)
      {
        ulGrammarErrorsReport(spData->vpErrorsCtx, spData->ulLineNo - 1,
          "Symbol index may not be greater than 255.");
      }
      vpVecPush(spData->vpTokTblCtx, (void*)&spData->sToken);
      vpVecPush(spData->vpTokIdxCtx, (void*)&ucIdx);
    }
    break;
  }
//}}CallBack.pfn_dRange
  return ulReturn;
}

// call back function for rule "dString"
static ulong pfn_dString(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_dString
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    {
      // initialize the token
      memset((void*)&spData->sToken, 0, sizeof(spData->sToken));
      spData->sToken.ulLineNo = spData->ulLineNo;
      spData->sToken.ulID = TOK_TBS;
      spData->sToken.ulOffset = ulVecSize(spData->vpStrTblCtx);
    }
    break;
  case SEM_POST:
    {
      // push the token on the token tables
      uchar ucIdx;
      ucIdx = (uchar)spData->sToken.ulID;
      if(ucIdx == TOK_ERROR)
      {
        ulGrammarErrorsReport(spData->vpErrorsCtx, spData->ulLineNo - 1,
          "Symbol index may not be greater than 255.");
      }
      vpVecPush(spData->vpTokTblCtx, (void*)&spData->sToken);
      vpVecPush(spData->vpTokIdxCtx, (void*)&ucIdx);
    }
    break;
  }
//}}CallBack.pfn_dString
  return ulReturn;
}

// call back function for rule "dChar"
static ulong pfn_dChar(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_dChar
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    {
      // initialize the token
      memset((void*)&spData->sToken, 0, sizeof(spData->sToken));
      spData->sToken.ulID = TOK_TRG;
      spData->sToken.ulLineNo = spData->ulLineNo;
    }
    break;
  case SEM_POST:
    {
      // push the token on the token tables
      uchar ucIdx = (uchar)spData->sToken.ulID;
      if(ucIdx == TOK_ERROR)
      {
        ulGrammarErrorsReport(spData->vpErrorsCtx, spData->ulLineNo - 1,
          "Symbol index may not be greater than 255.");
      }
      vpVecPush(spData->vpTokTblCtx, (void*)&spData->sToken);
      vpVecPush(spData->vpTokIdxCtx, (void*)&ucIdx);
    }
    break;
  }
//}}CallBack.pfn_dChar
  return ulReturn;
}

// call back function for rule "xRange"
static ulong pfn_xRange(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_xRange
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    {
      // initialize the token
      memset((void*)&spData->sToken, 0, sizeof(spData->sToken));
      spData->sToken.ulID = TOK_TRG;
      spData->sToken.ulLineNo = spData->ulLineNo;
    }
    break;
  case SEM_POST:
    {
      // push the token on the token tables
      uchar ucIdx = (uchar)spData->sToken.ulID;
      if(ucIdx == TOK_ERROR)
      {
        ulGrammarErrorsReport(spData->vpErrorsCtx, spData->ulLineNo - 1,
          "Symbol index may not be greater than 255.");
      }
      vpVecPush(spData->vpTokTblCtx, (void*)&spData->sToken);
      vpVecPush(spData->vpTokIdxCtx, (void*)&ucIdx);
    }
    break;
  }
//}}CallBack.pfn_xRange
  return ulReturn;
}

// call back function for rule "xString"
static ulong pfn_xString(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_xString
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    {
      // initialize the token
      memset((void*)&spData->sToken, 0, sizeof(spData->sToken));
      spData->sToken.ulLineNo = spData->ulLineNo;
      spData->sToken.ulID = TOK_TBS;
      spData->sToken.ulOffset = ulVecSize(spData->vpStrTblCtx);
    }
    break;
  case SEM_POST:
    {
      // push the token on the token tables
      uchar ucIdx;
      ucIdx = (uchar)spData->sToken.ulID;
      if(ucIdx == TOK_ERROR)
      {
        ulGrammarErrorsReport(spData->vpErrorsCtx, spData->ulLineNo - 1,
          "Symbol index may not be greater than 255.");
      }
      vpVecPush(spData->vpTokTblCtx, (void*)&spData->sToken);
      vpVecPush(spData->vpTokIdxCtx, (void*)&ucIdx);
    }
    break;
  }
//}}CallBack.pfn_xString
  return ulReturn;
}

// call back function for rule "xChar"
static ulong pfn_xChar(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_xChar
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    {
      // initialize the token
      memset((void*)&spData->sToken, 0, sizeof(spData->sToken));
      spData->sToken.ulID = TOK_TRG;
      spData->sToken.ulLineNo = spData->ulLineNo;
    }
    break;
  case SEM_POST:
    {
      // push the token on the token tables
      uchar ucIdx = (uchar)spData->sToken.ulID;
      if(ucIdx == TOK_ERROR)
      {
        ulGrammarErrorsReport(spData->vpErrorsCtx, spData->ulLineNo - 1,
          "Symbol index may not be greater than 255.");
      }
      vpVecPush(spData->vpTokTblCtx, (void*)&spData->sToken);
      vpVecPush(spData->vpTokIdxCtx, (void*)&ucIdx);
    }
    break;
  }
//}}CallBack.pfn_xChar
  return ulReturn;
}

// call back function for rule "bRange"
static ulong pfn_bRange(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_bRange
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    {
      // initialize the token
      memset((void*)&spData->sToken, 0, sizeof(spData->sToken));
      spData->sToken.ulID = TOK_TRG;
      spData->sToken.ulLineNo = spData->ulLineNo;
    }
    break;
  case SEM_POST:
    {
      // push the token on the token tables
      uchar ucIdx = (uchar)spData->sToken.ulID;
      if(ucIdx == TOK_ERROR)
      {
        ulGrammarErrorsReport(spData->vpErrorsCtx, spData->ulLineNo - 1,
          "Symbol index may not be greater than 255.");
      }
      vpVecPush(spData->vpTokTblCtx, (void*)&spData->sToken);
      vpVecPush(spData->vpTokIdxCtx, (void*)&ucIdx);
    }
    break;
  }
//}}CallBack.pfn_bRange
  return ulReturn;
}

// call back function for rule "bString"
static ulong pfn_bString(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_bString
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    {
      // initialize the token
      memset((void*)&spData->sToken, 0, sizeof(spData->sToken));
      spData->sToken.ulLineNo = spData->ulLineNo;
      spData->sToken.ulID = TOK_TBS;
      spData->sToken.ulOffset = ulVecSize(spData->vpStrTblCtx);
    }
    break;
  case SEM_POST:
    {
      // push the token on the token tables
      uchar ucIdx;
      ucIdx = (uchar)spData->sToken.ulID;
      if(ucIdx == TOK_ERROR)
      {
        ulGrammarErrorsReport(spData->vpErrorsCtx, spData->ulLineNo - 1,
          "Symbol index may not be greater than 255.");
      }
      vpVecPush(spData->vpTokTblCtx, (void*)&spData->sToken);
      vpVecPush(spData->vpTokIdxCtx, (void*)&ucIdx);
    }
    break;
  }
//}}CallBack.pfn_bString
  return ulReturn;
}

// call back function for rule "bChar"
static ulong pfn_bChar(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_bChar
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    {
      // initialize the token
      memset((void*)&spData->sToken, 0, sizeof(spData->sToken));
      spData->sToken.ulID = TOK_TRG;
      spData->sToken.ulLineNo = spData->ulLineNo;
    }
    break;
  case SEM_POST:
    {
      // push the token on the token tables
      uchar ucIdx = (uchar)spData->sToken.ulID;
      if(ucIdx == TOK_ERROR)
      {
        ulGrammarErrorsReport(spData->vpErrorsCtx, spData->ulLineNo - 1,
          "Symbol index may not be greater than 255.");
      }
      vpVecPush(spData->vpTokTblCtx, (void*)&spData->sToken);
      vpVecPush(spData->vpTokIdxCtx, (void*)&ucIdx);
    }
    break;
  }
//}}CallBack.pfn_bChar
  return ulReturn;
}

// call back function for rule "Name"
static ulong pfn_Name(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_Name
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    break;
  case SEM_POST:
    {
      SABNFTOKEN sTok;
      uchar ucIdx;
      
      memset((void*)&sTok, 0, sizeof(sTok));
      sTok.ulID = TOK_RNM;
      ucIdx = (uchar)sTok.ulID;
      sTok.ulOffset = ulOffset;
      sTok.ulLen = ulLen;
      sTok.ulLineNo = spData->ulLineNo;
      vpVecPush(spData->vpTokTblCtx, (void*)&sTok);
      vpVecPush(spData->vpTokIdxCtx, (void*)&ucIdx);
    };
    break;
  }
//}}CallBack.pfn_Name
  return ulReturn;
}

// call back function for rule "OpTls"
static ulong pfn_OpTls(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_OpTls
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    break;
  case SEM_POST:
    {
      SABNFTOKEN sTok;
      uchar ucIdx;
      const uchar* ucpBeg;
      const uchar* ucpEnd;
      ulong ulSrcLen;
      
      // push the string (without quotes) into the string table
      ulSrcLen = ulLen - 2; 
      memset((void*)&sTok, 0, sizeof(sTok));
      sTok.ulOffset = ulVecSize(spData->vpStrTblCtx);
      
      // push lower case string on string table
      ucpBeg = spData->ucpSrc + ulOffset + 1;
      ucpEnd = ucpBeg + ulSrcLen;
      for(; ucpBeg < ucpEnd; ++ucpBeg)
      {
        uchar ucL = *ucpBeg;
        if(ucL >= 65 && ucL <= 90){ucL += 32;}
        vpVecPush(spData->vpStrTblCtx, (void*)&ucL);
      }
      
      // generate a name token
      sTok.ulID = TOK_TLS;
      ucIdx = (uchar)sTok.ulID;
      sTok.ulLineNo = spData->ulLineNo;
      sTok.ulOffset = sTok.ulOffset;
      sTok.ulLen = ulSrcLen;
      vpVecPush(spData->vpTokTblCtx, (void*)&sTok);
      vpVecPush(spData->vpTokIdxCtx, (void*)&ucIdx);
    };
    break;
  }
//}}CallBack.pfn_OpTls
  return ulReturn;
}

// call back function for rule "ProsVal"
static ulong pfn_ProsVal(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_ProsVal
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    break;
  case SEM_POST:
    {
      SABNFTOKEN sTok;
      uchar ucIdx;
      // generate an error message
      ulGrammarErrorsReport(spData->vpErrorsCtx, spData->ulLineNo - 1,
        "ProsVal not a valid grammar element.");
      
      // generate an error token
      memset((void*)&sTok, 0, sizeof(sTok));
      sTok.ulID = TOK_ERROR;
      ucIdx = (uchar)sTok.ulID;
      sTok.ulLineNo = spData->ulLineNo;
      vpVecPush(spData->vpTokTblCtx, (void*)&sTok);
      vpVecPush(spData->vpTokIdxCtx, (void*)&ucIdx);
    };
    break;
  }
  //}}CallBack.pfn_ProsVal
  return ulReturn;
}

// call back function for rule "TokenError"
static ulong pfn_TokenError(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_TokenError
//}}CallBack.pfn_TokenError
  return ulReturn;
}

// call back function for rule "Token"
static ulong pfn_Token(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_Token
//}}CallBack.pfn_Token
  return ulReturn;
}

// call back function for rule "OpDec"
static ulong pfn_OpDec(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_OpDec
//}}CallBack.pfn_OpDec
  return ulReturn;
}

// call back function for rule "OpHex"
static ulong pfn_OpHex(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_OpHex
//}}CallBack.pfn_OpHex
  return ulReturn;
}

// call back function for rule "OpBin"
static ulong pfn_OpBin(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_OpBin
//}}CallBack.pfn_OpBin
  return ulReturn;
}

// call back function for rule "dmin"
static ulong pfn_dmin(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_dmin
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    break;
  case SEM_POST:
    {
      ulong ulNum = ulDec((char*)(spData->ucpSrc + ulOffset), ulLen);
      spData->sToken.ulMin = ulNum;
      spData->sToken.ulMax = ulNum;
      if(ulNum > 255)
      {
        // report the index range error
        spData->sToken.ulID = TOK_ERROR;
      }
    }
    break;
  }
//}}CallBack.pfn_dmin
  return ulReturn;
}

// call back function for rule "dmax"
static ulong pfn_dmax(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_dmax
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    break;
  case SEM_POST:
    {
      ulong ulNum = ulDec((char*)(spData->ucpSrc + ulOffset), ulLen);
      spData->sToken.ulMax = ulNum;
      if(ulNum > 255)
      {
        // report the index range error
        spData->sToken.ulID = TOK_ERROR;
      }
    }
    break;
  }
//}}CallBack.pfn_dmax
  return ulReturn;
}

// call back function for rule "dnum"
static ulong pfn_dnum(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_dnum
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    break;
  case SEM_POST:
    {
      ulong ulNum = ulDec((char*)(spData->ucpSrc + ulOffset), ulLen);
      if(ulNum > 255)
      {
        // symbol index cannot be larger than 255
        spData->sToken.ulID = TOK_ERROR;
      }
      else
      {
        // push the symbol index into the string table
        uchar ucIdx = (uchar)ulNum;
        vpVecPush(spData->vpStrTblCtx, (void*)&ucIdx);
        ++spData->sToken.ulLen;
      }
    }
    break;
  }
//}}CallBack.pfn_dnum
  return ulReturn;
}

// call back function for rule "bmin"
static ulong pfn_bmin(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_bmin
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    break;
  case SEM_POST:
    {
      ulong ulNum = ulBin((char*)(spData->ucpSrc + ulOffset), ulLen);
      spData->sToken.ulMin = ulNum;
      spData->sToken.ulMax = ulNum;
      if(ulNum > 255)
      {
        // report the index range error
        spData->sToken.ulID = TOK_ERROR;
      }
    }
    break;
  }
//}}CallBack.pfn_bmin
  return ulReturn;
}

// call back function for rule "bmax"
static ulong pfn_bmax(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_bmax
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    break;
  case SEM_POST:
    {
      ulong ulNum = ulBin((char*)(spData->ucpSrc + ulOffset), ulLen);
      spData->sToken.ulMax = ulNum;
      if(ulNum > 255)
      {
        // report the index range error
        spData->sToken.ulID = TOK_ERROR;
      }
    }
    break;
  }
//}}CallBack.pfn_bmax
  return ulReturn;
}

// call back function for rule "bnum"
static ulong pfn_bnum(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_bnum
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    break;
  case SEM_POST:
    {
      ulong ulNum = ulBin((char*)(spData->ucpSrc + ulOffset), ulLen);
      if(ulNum > 255)
      {
        // symbol index cannot be larger than 255
        spData->sToken.ulID = TOK_ERROR;
      }
      else
      {
        // push the symbol index into the string table
        uchar ucIdx = (uchar)ulNum;
        vpVecPush(spData->vpStrTblCtx, (void*)&ucIdx);
        ++spData->sToken.ulLen;
      }
    }
    break;
  }
//}}CallBack.pfn_bnum
  return ulReturn;
}

// call back function for rule "xmin"
static ulong pfn_xmin(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_xmin
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    break;
  case SEM_POST:
    {
      ulong ulNum = ulHex((char*)(spData->ucpSrc + ulOffset), ulLen);
      spData->sToken.ulMin = ulNum;
      spData->sToken.ulMax = ulNum;
      if(ulNum > 255)
      {
        // report the index range error
        spData->sToken.ulID = TOK_ERROR;
      }
    }
    break;
  }
//}}CallBack.pfn_xmin
  return ulReturn;
}

// call back function for rule "xmax"
static ulong pfn_xmax(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_xmax
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    break;
  case SEM_POST:
    {
      ulong ulNum = ulHex((char*)(spData->ucpSrc + ulOffset), ulLen);
      spData->sToken.ulMax = ulNum;
      if(ulNum > 255)
      {
        // report the index range error
        spData->sToken.ulID = TOK_ERROR;
      }
    }
    break;
  }
//}}CallBack.pfn_xmax
  return ulReturn;
}

// call back function for rule "xnum"
static ulong pfn_xnum(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_xnum
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    break;
  case SEM_POST:
    {
      ulong ulNum = ulHex((char*)(spData->ucpSrc + ulOffset), ulLen);
      if(ulNum > 255)
      {
        // symbol index cannot be larger than 255
        spData->sToken.ulID = TOK_ERROR;
      }
      else
      {
        // push the symbol index into the string table
        uchar ucIdx = (uchar)ulNum;
        vpVecPush(spData->vpStrTblCtx, (void*)&ucIdx);
        ++spData->sToken.ulLen;
      }
    }
    break;
  }
//}}CallBack.pfn_xnum
  return ulReturn;
}

// call back function for rule "alpha"
static ulong pfn_alpha(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_alpha
//}}CallBack.pfn_alpha
  return ulReturn;
}

// call back function for rule "wsp"
static ulong pfn_wsp(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_wsp
//}}CallBack.pfn_wsp
  return ulReturn;
}

// call back function for rule "comment"
static ulong pfn_comment(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_comment
//}}CallBack.pfn_comment
  return ulReturn;
}

// call back function for rule "LineContin"
static ulong pfn_LineContin(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_LineContin
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    break;
  case SEM_POST:
    // increment the line number
    ++spData->ulLineNo;
    break;
  }
//}}CallBack.pfn_LineContin
  return ulReturn;
}

// call back function for rule "LineEnd"
static ulong pfn_LineEnd(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_LineEnd
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    break;
  case SEM_POST:
    {
      const uchar* ucpTok;
      SABNFTOKEN sTok;
      uchar ucIdx;
      
      // if the previous token is OpCat delete it
      // this can happen if a rule definition ends in spaces
      ucpTok = (const uchar*)vpVecBack(spData->vpTokIdxCtx);
      if(ucpTok && *ucpTok == TOK_CAT)
      {
        vpVecPop(spData->vpTokIdxCtx);
        vpVecPop(spData->vpTokTblCtx);
      }
      
      // generate a line end token
      memset((void*)&sTok, 0, sizeof(sTok));
      sTok.ulID = TOK_LINEEND;
      ucIdx = (uchar)sTok.ulID;
      sTok.ulLineNo = spData->ulLineNo;
      vpVecPush(spData->vpTokTblCtx, (void*)&sTok);
      vpVecPush(spData->vpTokIdxCtx, (void*)&ucIdx);
      
      // increment the line number
      ++spData->ulLineNo;
    };
    break;
  }
//}}CallBack.pfn_LineEnd
  return ulReturn;
}

// call back function for rule "BlankLineEnd"
static ulong pfn_BlankLineEnd(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_BlankLineEnd
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    break;
  case SEM_POST:
    // increment the line number
    ++spData->ulLineNo;
    break;
  }
//}}CallBack.pfn_BlankLineEnd
  return ulReturn;
}

