/********************************************************************
  APG - an ABNF Parser Generator
  Copyright  (C) 2007 Coast to Coast Research, Inc.
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program.  If not, see
  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
  or write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

    author: Lowell Thomas
            lowell@coasttocoastresearch.com
            http://www.coasttocoastresearch.com

   created: 8/25/2007 13:03
  filename: D:\Projects\Apgv5_0\Generator\SEST.c
   purpose: All of the Single-Expansion Syntax Tree (SEST) functions.
  
*********************************************************************/
#include "Generator.h"

//////////////////////////////////////////////////////////////////////////
// static prototypes
//////////////////////////////////////////////////////////////////////////
static ulong ulSESTMetricsALT(ulong ulState, ulong ulTreeDepth, ulong ulChildCount, ulong ulCountMax, OP_INT_ALL* spOp, void* vpData);
static ulong ulSESTMetricsCAT(ulong ulState, ulong ulTreeDepth, ulong ulChildCount, ulong ulCountMax, OP_INT_ALL* spOp, void* vpData);
static ulong ulSESTMetricsRNM(ulong ulState, ulong ulTreeDepth, ulong ulChildCount, ulong ulCountMax, OP_INT_ALL* spOp, void* vpData);
static ulong ulSESTMetricsREP(ulong ulState, ulong ulTreeDepth, ulong ulChildCount, ulong ulCountMax, OP_INT_ALL* spOp, void* vpData);
static ulong ulSESTMetricsPRD(ulong ulState, ulong ulTreeDepth, ulong ulChildCount, ulong ulCountMax, OP_INT_ALL* spOp, void* vpData);
static ulong ulSESTMetricsTRG(ulong ulState, ulong ulTreeDepth, ulong ulChildCount, ulong ulCountMax, OP_INT_ALL* spOp, void* vpData);
static ulong ulSESTMetricsTBS(ulong ulState, ulong ulTreeDepth, ulong ulChildCount, ulong ulCountMax, OP_INT_ALL* spOp, void* vpData);
static ulong ulSESTMetricsTLS(ulong ulState, ulong ulTreeDepth, ulong ulChildCount, ulong ulCountMax, OP_INT_ALL* spOp, void* vpData);
static ulong ulSESTStartRNM(ulong ulState, ulong ulTreeDepth, ulong ulChildCount, ulong ulCountMax, OP_INT_ALL* spOp, void* vpData);
static ulong ulSESTNonRecursiveALT(ulong ulState, ulong ulTreeDepth, ulong ulChildCount, ulong ulCountMax, OP_INT_ALL* spOp, void* vpData);
static ulong ulSESTNonRecursiveCAT(ulong ulState, ulong ulTreeDepth, ulong ulChildCount, ulong ulCountMax, OP_INT_ALL* spOp, void* vpData);
static ulong ulSESTNonRecursiveRNM(ulong ulState, ulong ulTreeDepth, ulong ulChildCount, ulong ulCountMax, OP_INT_ALL* spOp, void* vpData);
static ulong ulSESTNonRecursiveREP(ulong ulState, ulong ulTreeDepth, ulong ulChildCount, ulong ulCountMax, OP_INT_ALL* spOp, void* vpData);
static ulong ulSESTNonRecursiveTRG(ulong ulState, ulong ulTreeDepth, ulong ulChildCount, ulong ulCountMax, OP_INT_ALL* spOp, void* vpData);
static ulong ulSESTNonRecursiveTLS(ulong ulState, ulong ulTreeDepth, ulong ulChildCount, ulong ulCountMax, OP_INT_ALL* spOp, void* vpData);
static ulong ulSESTRecursiveALT(ulong ulState, ulong ulTreeDepth, ulong ulChildCount, ulong ulCountMax, OP_INT_ALL* spOp, void* vpData);
static ulong ulSESTRecursiveCAT(ulong ulState, ulong ulTreeDepth, ulong ulChildCount, ulong ulCountMax, OP_INT_ALL* spOp, void* vpData);
static ulong ulSESTRecursiveRNM(ulong ulState, ulong ulTreeDepth, ulong ulChildCount, ulong ulCountMax, OP_INT_ALL* spOp, void* vpData);
static ulong ulSESTRecursiveREP(ulong ulState, ulong ulTreeDepth, ulong ulChildCount, ulong ulCountMax, OP_INT_ALL* spOp, void* vpData);
static ulong ulSESTRecursiveTRG(ulong ulState, ulong ulTreeDepth, ulong ulChildCount, ulong ulCountMax, OP_INT_ALL* spOp, void* vpData);
static ulong ulSESTRecursiveTLS(ulong ulState, ulong ulTreeDepth, ulong ulChildCount, ulong ulCountMax, OP_INT_ALL* spOp, void* vpData);
static ulong ulStartRuleAttributes(GENCTX* spData, ulong ulStartRule, ulong ulMaxNodes);
static ulong ulRecursiveAttributes(GENCTX* spData, ulong ulMaxNodes);
static ulong ulNonRecursiveAttributes(GENCTX* spData, ulong ulMaxNodes);
static ulong ulGrammarAttributes(GENCTX* spData, ulong ulStartRule);
static ulong ulCatRecursiveAttributes(R_ATTR* spAttr, CAT_HELPER* spHelper, ulong ulCountMax);

// !!!! DEBUG !!!!
// For visual examination of the branches
static ulong ulSESTBranchMetricsALT(ulong ulState, ulong ulTreeDepth, ulong ulChildCount, ulong ulCountMax, OP_INT_ALL* spOp, void* vpData);
static ulong ulSESTBranchMetricsCAT(ulong ulState, ulong ulTreeDepth, ulong ulChildCount, ulong ulCountMax, OP_INT_ALL* spOp, void* vpData);
static ulong ulSESTBranchMetricsREP(ulong ulState, ulong ulTreeDepth, ulong ulChildCount, ulong ulCountMax, OP_INT_ALL* spOp, void* vpData);
static ulong ulSESTBranchMetricsPRD(ulong ulState, ulong ulTreeDepth, ulong ulChildCount, ulong ulCountMax, OP_INT_ALL* spOp, void* vpData);
static ulong ulSESTBranchMetricsRNM(ulong ulState, ulong ulTreeDepth, ulong ulChildCount, ulong ulCountMax, OP_INT_ALL* spOp, void* vpData);
static ulong ulSESTBranchMetricsTRG(ulong ulState, ulong ulTreeDepth, ulong ulChildCount, ulong ulCountMax, OP_INT_ALL* spOp, void* vpData);
static ulong ulSESTBranchMetricsTBS(ulong ulState, ulong ulTreeDepth, ulong ulChildCount, ulong ulCountMax, OP_INT_ALL* spOp, void* vpData);
static ulong ulSESTBranchMetricsTLS(ulong ulState, ulong ulTreeDepth, ulong ulChildCount, ulong ulCountMax, OP_INT_ALL* spOp, void* vpData);
typedef struct
{
  ulong ulRecursive;
  char* cpRuleName;
} BranchEntry;

typedef struct
{
  ulong ulTreeDepthThreshold;
  ulong ulMaxTreeDepth;
  ulong ulBranches;
  ulong ulPrintCount;
  ulong ulMaxPrints;
  ulong ulPrintIncrement;
  char* cpStrTbl;
  void* vpRNMStack;
  INT_RULE_LIST* spRules;
} BranchData;

static BranchData g_sBranchData;

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: Prints a list of all (RNM) nodes along a specific branch of the SEST
//
// FUNCTION: static void vPrintBranch
//
//      ARG: 1) BranchEntry* spBranch
//              : The list of RNM nodes on the branch
//
//      ARG: 2) ulong ulLen
//              : the number of nodes in the branch
//
//      ARG: 3) ulong ulBranchNumber
//              : the index of a consequative list of all branches
//
//   RETURN: void
//
//////////////////////////////////////////////////////////////////////////
static void vPrintBranch(BranchEntry* spBranch, ulong ulLen, ulong ulBranchNumber)
{
  ulong ul;

  printf("\n");
  printf("*** Branch %lu\n", ulBranchNumber);
  for(ul = 0; ul < ulLen; ++ul, ++spBranch)
  {
    if(spBranch->ulRecursive)
    {
      printf("[%3lu] R %s\n", ul, spBranch->cpRuleName);
    }
    else
    {
      printf("[%3lu]   %s\n", ul, spBranch->cpRuleName);
    }
  }
}
// !!!! DEBUG !!!!

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: compute the SEST metrics (statistics) via a depth-first
//              traversal.
//              determines the set of unreferenced rules needed by PPT calculation
//
// FUNCTION: ulong ulSESTMetrics
//
//      ARG: 1) GENCTX* spData
//              : pointer to a valid generator component context
//
//   RETURN: true on success
//
//////////////////////////////////////////////////////////////////////////
ulong   ulSESTMetrics(GENCTX* spData)
{
  ulong           ulStartRule = spData->sConfig.ulStartRule;
  ulong           ulMaxTerminalNodes = spData->sConfig.ulMaxTerminalNodes;
  ulong           ulReturn;
  void*           vpWalkerCtx = NULL;
  SEST_METRICS*   spMetrics = &spData->sSESTMetrics;
  PFN_SESTCB      pfnCB[OP_INT_ID_MAX];
  INT_RULE_LIST*  spRules;
  ulong           ul;

  // create a Single Expansion Syntax Tree Walker context
  memset((void*)spMetrics, 0, sizeof(SEST_METRICS));
  spRules = (INT_RULE_LIST*)vpVecFront(spData->vpIntRuleCtx);
  GENASSERT(spRules);
  spMetrics->spRules = spRules;
  vpWalkerCtx = vpSESTCtor(spData, SEST_PPT, (void*)spMetrics);
  GENASSERT(vpWalkerCtx);
  spMetrics->ulStartRule = ulStartRule;
  
  // set the SEST Walker callbacks
  pfnCB[OP_INT_ID_RNM] = ulSESTMetricsRNM;
  pfnCB[OP_INT_ID_REP] = ulSESTMetricsREP;
  pfnCB[OP_INT_ID_PRD] = ulSESTMetricsPRD;
  pfnCB[OP_INT_ID_ALT] = ulSESTMetricsALT;
  pfnCB[OP_INT_ID_CAT] = ulSESTMetricsCAT;
  pfnCB[OP_INT_ID_TRG] = ulSESTMetricsTRG;
  pfnCB[OP_INT_ID_TBS] = ulSESTMetricsTBS;
  pfnCB[OP_INT_ID_TLS] = ulSESTMetricsTLS;
  vSESTSetCallBacks(vpWalkerCtx, pfnCB);

  // determine SEST metrics for root rule
  ulReturn = ulSESTWalk(vpWalkerCtx, ulStartRule, ulMaxTerminalNodes, APG_FALSE);
  if(ulReturn)
  {
    spMetrics->ulMaxTreeDepth = ulSESTMaxTreeDepth(vpWalkerCtx);
    if(spData->sConfig.ulDisplayMetrics)
    {
      // print metrics
      vPrintSESTMetrics(spData);
    }

    for(ul = 0; ul < spData->ulRuleCount; ++ul)
    {
      if(!spRules[ul].sAttr.ulRef)
      {
        ulGrammarErrorsReport(spData->vpWarningsCtx, spRules[ul].ulLineNo - 1, "un-referenced by start rule");
      }
    }
  }
  else
  {
    char caBuf[126];
    sprintf(caBuf, "SESTMetrics: SEST has more than max = %lu terminal nodes\n", ulMaxTerminalNodes);
    ulGrammarErrorsReport(spData->vpErrorsCtx, spRules[ulStartRule].ulLineNo - 1, caBuf);
  }

APG_EXIT:
  if(vpWalkerCtx)
  {
    vSESTDtor(vpWalkerCtx);
    vpWalkerCtx = NULL;
  }
  return ulReturn;
}

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: Used to discover and record the SEST branch depth and
//              the list of RNM nodes along selected branches.
//              Used to study the conditions for long SEST processing
//              of recursion analysis and PPT computation.
//
// FUNCTION: ulong ulBranchSESTMetrics
//
//      ARG: 1) GENCTX* spData
//              : pointer to a valid generator component context
//
//   RETURN: treu on success
//
//////////////////////////////////////////////////////////////////////////
ulong   ulBranchSESTMetrics(GENCTX* spData)
{
  ulong         ulStartRule = spData->sConfig.ulStartRule;
  ulong         ulMaxTerminalNodes = spData->sConfig.ulMaxTerminalNodes;
  ulong         ulReturn;
  void*         vpWalkerCtx = NULL;
  SEST_METRICS* spMetrics = &spData->sSESTMetrics;
  PFN_SESTCB    pfnCB[OP_INT_ID_MAX];
  
  // create a Single Expansion Syntax Tree Walker context
  memset((void*)spMetrics, 0, sizeof(SEST_METRICS));
  vpWalkerCtx = vpSESTCtor(spData, SEST_PPT, (void*)spMetrics);
  GENASSERT(vpWalkerCtx);
  spMetrics->ulStartRule = ulStartRule;

  // create a stack for printing branches
  memset((void*)&g_sBranchData, 0, sizeof(BranchData));
  g_sBranchData.vpRNMStack = vpVecCtor(spData->vpMemCtx, sizeof(BranchEntry), 200);
  DASSERT(g_sBranchData.vpRNMStack);
  g_sBranchData.ulPrintIncrement = 1;
  g_sBranchData.ulMaxPrints = 10;
  g_sBranchData.ulTreeDepthThreshold = 132;
  g_sBranchData.cpStrTbl = (char*)vpVecFront(spData->vpStrTblCtx);
  g_sBranchData.spRules = (INT_RULE_LIST*)vpVecFront(spData->vpIntRuleCtx);
  
  // set the SEST Walker callbacks
  pfnCB[OP_INT_ID_RNM] = ulSESTBranchMetricsRNM;
  pfnCB[OP_INT_ID_REP] = ulSESTBranchMetricsREP;
  pfnCB[OP_INT_ID_PRD] = ulSESTBranchMetricsPRD;
  pfnCB[OP_INT_ID_ALT] = ulSESTBranchMetricsALT;
  pfnCB[OP_INT_ID_CAT] = ulSESTBranchMetricsCAT;
  pfnCB[OP_INT_ID_TRG] = ulSESTBranchMetricsTRG;
  pfnCB[OP_INT_ID_TBS] = ulSESTBranchMetricsTBS;
  pfnCB[OP_INT_ID_TLS] = ulSESTBranchMetricsTLS;
  vSESTSetCallBacks(vpWalkerCtx, pfnCB);
  
  // determine SEST metrics for root rule
  ulReturn = ulSESTWalk(vpWalkerCtx, ulStartRule, ulMaxTerminalNodes, APG_FALSE);
  if(!ulReturn){goto APG_EXIT;}
  
APG_EXIT:
  if(g_sBranchData.vpRNMStack){vVecDtor(g_sBranchData.vpRNMStack);}
  memset((void*)&g_sBranchData, 0, sizeof(BranchData));
  if(vpWalkerCtx)
  {
    vSESTDtor(vpWalkerCtx);
    vpWalkerCtx = NULL;
  }
  return ulReturn;
}

//////////////////////////////////////////////////////////////////////////
// RECURSION ANALYSIS
//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: Generates a set of recursive attributes for each rule.
//
// FUNCTION: ulong ulSESTRecursionAnalysis
//
//      ARG: 1) GENCTX* spData
//              : pointer to a valid generator component context
//
//   RETURN: true on success
//
//////////////////////////////////////////////////////////////////////////
ulong ulSESTRecursionAnalysis(GENCTX* spData)
{
  ulong ulReturn = APG_FALSE;
  ulong ulTest;

  // find list of recursive and non-recursive rules referenced by the start rule
  ulTest = ulStartRuleAttributes(spData, spData->sConfig.ulStartRule, spData->sConfig.ulMaxTerminalNodes);
  GENASSERT(ulTest);
  
  // find non-recursive attributes of all rules referenced by the start rule
  ulTest = ulNonRecursiveAttributes(spData, spData->sConfig.ulMaxTerminalNodes);
  GENASSERT(ulTest);
  
  // determine recursive attributes of all recursive rules referenced by the start rule
  ulTest = ulRecursiveAttributes(spData, spData->sConfig.ulMaxTerminalNodes);
  GENASSERT(ulTest);
  
  // find cumulative grammar attributes
  ulTest = ulGrammarAttributes(spData, spData->sConfig.ulStartRule);
  GENASSERT(ulTest);
  
  if(spData->sConfig.ulDisplayRecursion)
  {
    vPrintRecursiveAttrs(spData);
  }
  
  if(spData->sConfig.ulDisplayRuleAttrs)
  {
    vPrintRuleAttrs(spData);
  }

  if(0 == ulGrammarErrorsCount(spData->vpErrorsCtx))
  {
    // success
    ulReturn = APG_TRUE;
  }

APG_EXIT:
  return ulReturn;
}


typedef struct 
{
  INT_RULE_LIST* spRuleList;
  ulong ulRuleCount;
  ulong ulStartRule;
} STARTRULECTX;

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: Determines which rules refer to themselves in the start rule SEST.
//              It does not determine any of the other recursive attributes. 
// NOTE: many otherwise recursive rules may not actually refer
//       to themselves in the SEST with the start rule as the
//       root node.
// eg.
// A = "a" B / "x"
// B = "b" A / "y"
// A and B appear to be mutually recursive rules. However, if A is selected
// as the start rule, B does not actually refer to itself in the SEST and
// would not be marked as a recursive rule.
//
// FUNCTION: static ulong ulStartRuleAttributes
//
//      ARG: 1) GENCTX* spData
//              : pointer to a valid generator component context
//
//      ARG: 2) ulong ulStartRule
//              : index of the start rule
//
//      ARG: 3) ulong ulMaxNodes
//              : maximum number of SEST nodes to visit before giving up
//
//   RETURN: true on success
//
//////////////////////////////////////////////////////////////////////////
static ulong ulStartRuleAttributes(GENCTX* spData, ulong ulStartRule, ulong ulMaxNodes)
{
  ulong           ulReturn = APG_FALSE;
  ulong           ulTest;
  void*           vpWalkerCtx = NULL;
  INT_RULE_LIST*  spRule;
  INT_RULE_LIST*  spEnd;
  STARTRULECTX    sCtx;
  PFN_SESTCB      pfnaIntCB[OP_INT_ID_MAX];

  // create a Single Expansion Syntax Tree Walker context
  vpWalkerCtx = vpSESTCtor(spData, SEST_RECURSION, (void*)&sCtx);
  GENASSERT(vpWalkerCtx);
  
  // set the SEST Walker callbacks
  memset((void*)pfnaIntCB, 0, sizeof(pfnaIntCB));
  pfnaIntCB[OP_INT_ID_RNM] = ulSESTStartRNM;
  vSESTSetCallBacks(vpWalkerCtx, pfnaIntCB);
  
  // set the tree walk data
  sCtx.spRuleList = (INT_RULE_LIST*)vpVecFront(spData->vpIntRuleCtx);
  sCtx.ulRuleCount = ulVecSize(spData->vpIntRuleCtx);
  sCtx.ulStartRule = ulStartRule;
  
  // clear the rule list attributes
  spRule = sCtx.spRuleList;      
  spEnd = spRule + sCtx.ulRuleCount;
  for(; spRule < spEnd; ++spRule)
  {
    spRule->ulLeaf = APG_FALSE;
    memset((void*)&spRule->sAttr, 0, sizeof(spRule->sAttr));
  }
  
  // walk the Single Expansion Syntax Tree
  ulTest = ulSESTWalk(vpWalkerCtx, ulStartRule, ulMaxNodes, APG_FALSE);
  GENASSERT(ulTest);

  // success
  ulReturn = APG_TRUE;

APG_EXIT:
  if(vpWalkerCtx)
  {
    // clean up resources
    vSESTDtor(vpWalkerCtx);
  }
  return ulReturn;
}

typedef struct 
{
  ulong ulStartRule;
  void* vpAttrStack;
  INT_RULE_LIST* spRules;
  GENCTX* spData;
} RECURSIVE_DATA;

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: Determines the recursive attributes of each rule which
//              refers to itself in the start rule SEST. The determination
//              is made by traversing the SEST of each recursive-marked rule
//              as if it were the start rule, and ignoring all other
//              recursive-marked rules. One might think of these as 
//              self-recursive attributes.
//
// FUNCTION: static ulong ulRecursiveAttributes
//
//      ARG: 1) GENCTX* spData
//              : pointer to a valid generator component context
//
//      ARG: 2) ulong ulMaxNodes
//              : maximum number of SEST nodes to process before giving up
//
//   RETURN: true on success
//
//////////////////////////////////////////////////////////////////////////
static ulong ulRecursiveAttributes(GENCTX* spData, ulong ulMaxNodes)
{
  ulong ulReturn = APG_FALSE;
  ulong ulTest;
  PFN_SESTCB  pfnaIntCB[OP_INT_ID_MAX];
  void* vpWalkerCtx = NULL;
  void* vpAttrs = NULL;
  R_ATTR* spRuleAttr;
  ulong ulRule;
  ulong ulRuleCount = ulVecSize(spData->vpIntRuleCtx);
  RECURSIVE_DATA sData;
  
  // create a vector of attribute vectors
  vpAttrs = vpVecCtor(spData->vpMemCtx, sizeof(R_ATTR), spData->sSESTMetrics.ulMaxTreeDepth + 2);
  GENASSERT(vpAttrs);
  vpVecPush(vpAttrs, NULL);   // push a dummy to prevent vector becoming empty
  sData.vpAttrStack = vpAttrs;
  sData.spRules = (INT_RULE_LIST*)vpVecFront(spData->vpIntRuleCtx);
  sData.spData = spData;
  
  // create a Single Expansion Syntax Tree Walker context
  vpWalkerCtx = vpSESTCtor(spData, SEST_RECURSION, (void*)&sData);
  GENASSERT(vpWalkerCtx);
  
  // set the SEST Walker callbacks
  pfnaIntCB[OP_INT_ID_RNM] = ulSESTRecursiveRNM;
  pfnaIntCB[OP_INT_ID_REP] = ulSESTRecursiveREP;
  pfnaIntCB[OP_INT_ID_PRD] = ulSESTRecursiveREP;
  pfnaIntCB[OP_INT_ID_ALT] = ulSESTRecursiveALT;
  pfnaIntCB[OP_INT_ID_CAT] = ulSESTRecursiveCAT;
  pfnaIntCB[OP_INT_ID_TRG] = ulSESTRecursiveTRG;
  pfnaIntCB[OP_INT_ID_TBS] = ulSESTRecursiveTRG;
  pfnaIntCB[OP_INT_ID_TLS] = ulSESTRecursiveTLS;
  vSESTSetCallBacks(vpWalkerCtx, pfnaIntCB);
  
  for(ulRule = 0; ulRule < ulRuleCount; ++ulRule)
  {
    INT_RULE_LIST* spRule = (INT_RULE_LIST*)vpVecAt(spData->vpIntRuleCtx, ulRule);
    if(spRule->sAttr.ulRef && spRule->sAttr.ulRecursive)
    {
      // walk the Single Expansion Syntax Tree
      sData.ulStartRule = ulRule;
      ulTest = ulSESTWalk(vpWalkerCtx, sData.ulStartRule, ulMaxNodes, APG_FALSE);
      GENASSERT(ulTest);
      
      // get the attrs for this rule
      ulTest = ulVecSize(vpAttrs);
      DASSERT(ulTest == 1);
      spRuleAttr = (R_ATTR*)vpVecFront(vpAttrs);
      ++spRuleAttr;
      spRule->sAttr.ulLeft = spRuleAttr->ulLeft;
      spRule->sAttr.ulRight = spRuleAttr->ulRight;
      spRule->sAttr.ulNested = spRuleAttr->ulNested;
      spRule->sAttr.ulCyclic = spRuleAttr->ulCyclic;
    }
  }
  
  
  // success
  ulReturn = APG_TRUE;
 
APG_EXIT:
  if(vpWalkerCtx)
  {
    // clean up resources
    vSESTDtor(vpWalkerCtx);
  }
  if(vpAttrs)
  {
    // clean up resources
    vVecDtor(vpAttrs);
  }
  return ulReturn;
}

typedef struct
{
  void* vpAttrs;
  GENCTX* spData;
} NONRECURSIVEATTRS;

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: Determines the non-recursive attributes (empty, terminal)
//              of each rule.
//
// FUNCTION: static ulong ulNonRecursiveAttributes
//
//      ARG: 1) GENCTX* spData
//              : pointer to a valid generator component context
//
//      ARG: 2) ulong ulMaxNodes
//              : maximum number of SEST nodes to process before giving up
//
//   RETURN: true on success
//
//////////////////////////////////////////////////////////////////////////
static ulong ulNonRecursiveAttributes(GENCTX* spData, ulong ulMaxNodes)
{
  ulong ulReturn = APG_FALSE;
  ulong ulTest;
  PFN_SESTCB  pfnaIntCB[OP_INT_ID_MAX];
  void* vpWalkerCtx = NULL;
  void* vpAttrs = NULL;
  R_ATTR* spRuleAttr;
  ulong ulRule;
  ulong ulRuleCount = ulVecSize(spData->vpIntRuleCtx);
  NONRECURSIVEATTRS sData;

  // create a vector of attribute vectors
  vpAttrs = vpVecCtor(spData->vpMemCtx, sizeof(R_ATTR), spData->sSESTMetrics.ulMaxTreeDepth + 2);
  GENASSERT(vpAttrs);
  ulTest = (ulong)vpVecPush(vpAttrs, NULL);   // push a dummy to prevent vector becoming empty
  GENASSERT(ulTest);
  
  // create a Single Expansion Syntax Tree Walker context
  sData.spData = spData;
  sData.vpAttrs = vpAttrs;
  vpWalkerCtx = vpSESTCtor(spData, SEST_RECURSION, (void*)&sData);
  GENASSERT(vpWalkerCtx);
  
  // set the SEST Walker callbacks
  pfnaIntCB[OP_INT_ID_RNM] = ulSESTNonRecursiveRNM;
  pfnaIntCB[OP_INT_ID_REP] = ulSESTNonRecursiveREP;
  pfnaIntCB[OP_INT_ID_PRD] = ulSESTNonRecursiveREP;
  pfnaIntCB[OP_INT_ID_ALT] = ulSESTNonRecursiveALT;
  pfnaIntCB[OP_INT_ID_CAT] = ulSESTNonRecursiveCAT;
  pfnaIntCB[OP_INT_ID_TRG] = ulSESTNonRecursiveTRG;
  pfnaIntCB[OP_INT_ID_TBS] = ulSESTNonRecursiveTLS;
  pfnaIntCB[OP_INT_ID_TLS] = ulSESTNonRecursiveTLS;
  vSESTSetCallBacks(vpWalkerCtx, pfnaIntCB);
  
  for(ulRule = 0; ulRule < ulRuleCount; ++ulRule)
  {
    INT_RULE_LIST* spRule = (INT_RULE_LIST*)vpVecAt(spData->vpIntRuleCtx, ulRule);
    
    // walk the Single Expansion Syntax Tree
    ulTest = ulSESTWalk(vpWalkerCtx, ulRule, ulMaxNodes, APG_FALSE);
    GENASSERT(ulTest);
    
    // get the attrs for this rule
    ulTest = ulVecSize(vpAttrs);
    DASSERT(ulTest == 1);
    spRuleAttr = (R_ATTR*)vpVecFront(vpAttrs);
    ++spRuleAttr;
    spRule->sAttr.ulEmpty = spRuleAttr->ulEmpty;
    spRule->sAttr.ulTerminal = spRuleAttr->ulTerminal;
  }
  
  // success
  ulReturn = APG_TRUE;

APG_EXIT:
  if(vpWalkerCtx)
  {
    // clean up resources
    vSESTDtor(vpWalkerCtx);
  }
  if(vpAttrs)
  {
    // clean up resources
    vVecDtor(vpAttrs);
  }
  return ulReturn;
}

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: Grammar attributes are the combined attributes of all
//              individual rules. If an attribute is true for ANY rule
//              it is true for the grammar as a whole.
//
// FUNCTION: static ulong ulGrammarAttributes
//
//      ARG: 1) GENCTX* spData
//              : pointer to a valid generator component context
//
//      ARG: 2) ulong ulStartRule
//              : the rule index of the start rule.
//
//   RETURN: true on success
//
//////////////////////////////////////////////////////////////////////////
static ulong ulGrammarAttributes(GENCTX* spData, ulong ulStartRule)
{
  ulong ulReturn = APG_FALSE;
  ulong ulRule;
  ulong ulRuleCount = spData->ulRuleCount;
  INT_RULE_LIST* spRuleBeg = (INT_RULE_LIST*)vpVecFront(spData->vpIntRuleCtx);
  INT_RULE_LIST* spRule;
  R_ATTR* spAttr;
  R_ATTR* spGrammar = &spData->sGrammarAttrs;

  while(APG_TRUE)
  {
    if(ulRuleCount == 0)
    {
      break;
    }

    memset((void*)spGrammar, 0, sizeof(*spGrammar));

    // if any rule attribute is true, the grammar attribute is true
    // except finite, if any is not finite, the grammar is not finite
    spGrammar->ulFinite = APG_TRUE;
    spRule = spRuleBeg;
    for(ulRule = 0; ulRule < ulRuleCount; ++ulRule, ++spRule)
    {
      spAttr = &spRule->sAttr;
      if(spAttr->ulRecursive){spGrammar->ulRecursive = APG_TRUE;}
      if(spAttr->ulLeft){spGrammar->ulLeft = APG_TRUE;}
      if(spAttr->ulRight){spGrammar->ulRight = APG_TRUE;}
      if(spAttr->ulNested){spGrammar->ulNested = APG_TRUE;}
      if(spAttr->ulCyclic){spGrammar->ulCyclic = APG_TRUE;}
      if(spAttr->ulEmpty){spGrammar->ulEmpty = APG_TRUE;}
      if(!spAttr->ulTerminal){spGrammar->ulFinite = APG_FALSE;}
      if(spAttr->ulLeft)
      {
        // grammar error
        ulGrammarErrorsReport(spData->vpErrorsCtx, spRule->ulLineNo - 1, "grammar is left recursive");
      }
      if(spAttr->ulCyclic)
      {
        // grammar error
        ulGrammarErrorsReport(spData->vpErrorsCtx, spRule->ulLineNo - 1, "grammar is cyclic");
      }
    }
    
    // success
    ulReturn = APG_TRUE;
    break;
  }

  
  return ulReturn;
}


//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: Call back function for a depth-first walk of the SEST nodes.
//
// FUNCTION: static ulong ulSESTStartRNM
//
//      ARG: 1) ulong ulState
//              : the traversal state SEM_PRE, SEM_POST
//
//      ARG: 2) ulong ulTreeDepth
//              : the SEST tree depth
//
//      ARG: 3) ulong ulChildCount
//              : for ALT and CAT this is the child count (1-based)
//                0 indicates before and after all children have been traversed
//
//      ARG: 4) ulong ulCountMax
//              : number of children in the child list
//
//      ARG: 5) OP_INT_ALL* spOp
//              : the opcode that this SEST tree node represents
//
//      ARG: 6) void* vpData
//              : special data context
//
//   RETURN: PFN_OK is normal return, PFN_SKIP requests caller to skip the
//           traversal of the SEST branch below this node.
//
//////////////////////////////////////////////////////////////////////////
static ulong ulSESTStartRNM(ulong ulState, ulong ulTreeDepth, ulong ulChildCount, ulong ulCountMax, OP_INT_ALL* spOp, void* vpData)
{
  ulong ulReturn = PFN_OK;
  STARTRULECTX* spData = (STARTRULECTX*)vpData;
  INT_RULE_LIST* spRule = &spData->spRuleList[spOp->ulRuleIndex];
  DASSERT(spOp->ulOpId == OP_INT_ID_RNM);
  
  if(ulChildCount == ROOT_RULE)
  {
    if(ulState == SEM_POST)
    {
      ++spRule->sAttr.ulRef;
    }
  }
  else
  {
    DASSERT(ulChildCount == LEAF_RULE);
    if(ulState == SEM_PRE)
    {
        ++spRule->sAttr.ulRecursive;
    }
  }
 
  return ulReturn;
}

static ulong ulSESTMetricsALT(ulong ulState, ulong ulTreeDepth, ulong ulChildCount, ulong ulCountMax, OP_INT_ALL* spOp, void* vpData)
{
  SEST_METRICS* spMetrics = (SEST_METRICS*)vpData;
  
  // validate
  DASSERT(spOp->ulOpId == OP_INT_ID_ALT);
  if(ulState == SEM_POST && ulChildCount == 0 && spOp->ulCount > 1)
  {
    ++spMetrics->ulALTNodes;
    ++spMetrics->ulTotalNodes;
    ++spMetrics->ulTotalNonTerminal;
  }

  return PFN_OK;
}

static ulong ulSESTMetricsCAT(ulong ulState, ulong ulTreeDepth, ulong ulChildCount, ulong ulCountMax, OP_INT_ALL* spOp, void* vpData)
{
  SEST_METRICS* spMetrics = (SEST_METRICS*)vpData;
  
  // validate
  DASSERT(spOp->ulOpId == OP_INT_ID_CAT);
  if(ulState == SEM_POST && ulChildCount == 0 && spOp->ulCount > 1)
  {
    ++spMetrics->ulCATNodes;
    ++spMetrics->ulTotalNodes;
    ++spMetrics->ulTotalNonTerminal;
  }
  
  return PFN_OK;
}

static ulong ulSESTMetricsREP(ulong ulState, ulong ulTreeDepth, ulong ulChildCount, ulong ulCountMax, OP_INT_ALL* spOp, void* vpData)
{
  SEST_METRICS* spMetrics = (SEST_METRICS*)vpData;
  
  // validate
  DASSERT(spOp->ulOpId == OP_INT_ID_REP);
  if(ulState == SEM_POST && ulChildCount == 0 && !(spOp->ulMin == 1 && spOp->ulMax == 1))
  {
    ++spMetrics->ulREPNodes;
    ++spMetrics->ulTotalNodes;
    ++spMetrics->ulTotalNonTerminal;
  }
  
  return PFN_OK;
}

static ulong ulSESTMetricsPRD(ulong ulState, ulong ulTreeDepth, ulong ulChildCount, ulong ulCountMax, OP_INT_ALL* spOp, void* vpData)
{
  SEST_METRICS* spMetrics = (SEST_METRICS*)vpData;
  
  // validate
  DASSERT(spOp->ulOpId == OP_INT_ID_PRD);
  if(ulState == SEM_POST && ulChildCount == 0)
  {
    ++spMetrics->ulPRDNodes;
    ++spMetrics->ulTotalNodes;
    ++spMetrics->ulTotalNonTerminal;
  }
  
  return PFN_OK;
}

static ulong ulSESTMetricsRNM(ulong ulState, ulong ulTreeDepth, ulong ulChildCount, ulong ulCountMax, OP_INT_ALL* spOp, void* vpData)
{
  SEST_METRICS* spMetrics = (SEST_METRICS*)vpData;
  
  // validate
  DASSERT(spOp->ulOpId == OP_INT_ID_RNM);
  if(ulState == SEM_POST)
  {
    // sub-tree root rule opcode (this rule is referenced by the start rule - needed for PPTs)
    spMetrics->spRules[spOp->ulRuleIndex].sAttr.ulRef = APG_TRUE;
    ++spMetrics->ulTotalNodes;
    if(ulChildCount == ROOT_RULE)
    {
      ++spMetrics->ulTotalNonTerminal;
      ++spMetrics->ulRNMNodes;
    }
    else
    {
      // tree leaf rule opcode (this rule is recursive)
      spMetrics->spRules[spOp->ulRuleIndex].sAttr.ulRecursive = APG_TRUE;
      ++spMetrics->ulTotalTerminal;
      ++spMetrics->ulRNMLeaves;
      if(spOp->ulRuleIndex == spMetrics->ulStartRule)
      {
        ++spMetrics->ulRootRNMLeaves;
      }
    }
  }
  
  return PFN_OK;
}

static ulong ulSESTMetricsTRG(ulong ulState, ulong ulTreeDepth, ulong ulChildCount, ulong ulCountMax, OP_INT_ALL* spOp, void* vpData)
{
  SEST_METRICS* spMetrics = (SEST_METRICS*)vpData;
  
  // validate
  DASSERT(spOp->ulOpId == OP_INT_ID_TRG);
  if(ulState == SEM_POST)
  {
    ++spMetrics->ulTRGLeaves;
    ++spMetrics->ulTotalNodes;
    ++spMetrics->ulTotalTerminal;
    ++spMetrics->ulTerminalLeaves;
  }
  
  return PFN_OK;
}

static ulong ulSESTMetricsTBS(ulong ulState, ulong ulTreeDepth, ulong ulChildCount, ulong ulCountMax, OP_INT_ALL* spOp, void* vpData)
{
  SEST_METRICS* spMetrics = (SEST_METRICS*)vpData;
  
  // validate
  DASSERT(spOp->ulOpId == OP_INT_ID_TBS);
  if(ulState == SEM_POST && spOp->ulLength > 0)
  {
    ++spMetrics->ulTBSLeaves;
    ++spMetrics->ulTotalNodes;
    ++spMetrics->ulTotalTerminal;
    ++spMetrics->ulTerminalLeaves;
  }
  
  return PFN_OK;
}

static ulong ulSESTMetricsTLS(ulong ulState, ulong ulTreeDepth, ulong ulChildCount, ulong ulCountMax, OP_INT_ALL* spOp, void* vpData)
{
  SEST_METRICS* spMetrics = (SEST_METRICS*)vpData;
  
  // validate
  DASSERT(spOp->ulOpId == OP_INT_ID_TLS);
  if(ulState == SEM_POST)
  {
    ++spMetrics->ulTLSLeaves;
    ++spMetrics->ulTotalNodes;
    ++spMetrics->ulTotalTerminal;
    ++spMetrics->ulTerminalLeaves;
    if(spOp->ulLength == 0)
    {
      ++spMetrics->ulEmptyStringLeaves;
    }
  }
  
  return PFN_OK;
}
//////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////
// NON-RECURSIVE CALL BACK FUNCTIONS
//////////////////////////////////////////////////////////////////////////
static ulong ulSESTNonRecursiveALT(ulong ulState, ulong ulTreeDepth, ulong ulChildCount, ulong ulCountMax, OP_INT_ALL* spOp, void* vpData)
{
  NONRECURSIVEATTRS* spNRA = (NONRECURSIVEATTRS*)vpData;
  GENCTX* spData = spNRA->spData;
  R_ATTR* spAttr;
  R_ATTR* spChild;
  DASSERT(spOp->ulOpId == OP_INT_ID_ALT);
  
  if(ulChildCount == 0)
  {
    switch(ulState)
    {
    case SEM_PRE:
      spAttr = (R_ATTR*)vpVecPush(spNRA->vpAttrs, NULL);
      GENASSERT(spAttr);

      // initialize to false
      memset((void*)spAttr, 0, sizeof(R_ATTR));
      break;
    case SEM_POST:
      vpVecPop(spNRA->vpAttrs);
      break;
    default:
      GENASSERT(APG_FALSE);
      break;
    }
  }
  else
  {
    if(ulState == SEM_POST)
    {
      // get the attrs for this ALT
      spAttr = (R_ATTR*)vpVecBack(spNRA->vpAttrs);
      
      // get the child attrs - popped but still resident on vector
      spChild = spAttr + 1;
      if(ulChildCount == 1)
      {
        *spAttr = *spChild;
      }
      else
      {
        // pairwise combination of ALT attrs with child attrs
        if(spChild->ulEmpty){spAttr->ulEmpty = APG_TRUE;}
        if(spChild->ulTerminal){spAttr->ulTerminal = APG_TRUE;}
      }
    }
  }
  
APG_EXIT:
  return PFN_OK;
}

static ulong ulSESTNonRecursiveCAT(ulong ulState, ulong ulTreeDepth, ulong ulChildCount, ulong ulCountMax, OP_INT_ALL* spOp, void* vpData)
{
  NONRECURSIVEATTRS* spNRA = (NONRECURSIVEATTRS*)vpData;
  GENCTX* spData = spNRA->spData;
  R_ATTR* spAttr;
  R_ATTR* spChild;
  DASSERT(spOp->ulOpId == OP_INT_ID_CAT);

  if(ulChildCount == 0)
  {
    switch(ulState)
    {
    case SEM_PRE:
      spAttr = (R_ATTR*)vpVecPush(spNRA->vpAttrs, NULL);
      memset((void*)spAttr, 0, sizeof(R_ATTR));
      GENASSERT(spAttr);
      break;
    case SEM_POST:
      spAttr = (R_ATTR*)vpVecPop(spNRA->vpAttrs);
      break;
    default:
      GENASSERT(APG_FALSE);
      break;
    }
  }
  else
  {
    if(ulState == SEM_POST)
    {
      // get the attrs for this CAT
      spAttr = (R_ATTR*)vpVecBack(spNRA->vpAttrs);

      // get the child attrs - popped but still resident on vector
      spChild = spAttr + 1;
      if(ulChildCount == 1)
      {
        *spAttr = *spChild;
      }
      else
      {
        // pairwise combination of CAT attrs with child attrs
        if(spAttr->ulTerminal && spChild->ulTerminal)
        {
          // if either CAT or child is terminal, CAT is terminal
          spAttr->ulTerminal = APG_TRUE;
        }
        else
        {
          spAttr->ulTerminal = APG_FALSE;
        }
        if(spAttr->ulEmpty && spChild->ulEmpty)
        {
          spAttr->ulEmpty = APG_TRUE;
        }
        else
        {
          spAttr->ulEmpty = APG_FALSE;
        }
      }
    }
  }
  
APG_EXIT:
  return PFN_OK;
}

static ulong ulSESTNonRecursiveRNM(ulong ulState, ulong ulTreeDepth, ulong ulChildCount, ulong ulCountMax, OP_INT_ALL* spOp, void* vpData)
{
  NONRECURSIVEATTRS* spNRA = (NONRECURSIVEATTRS*)vpData;
  GENCTX* spData = spNRA->spData;
  R_ATTR* spAttr;
  R_ATTR* spChild;
  DASSERT(spOp->ulOpId == OP_INT_ID_RNM);
  
  if(ulChildCount == ROOT_RULE)
  {
    switch(ulState)
    {
    case SEM_PRE:
      spAttr = (R_ATTR*)vpVecPush(spNRA->vpAttrs, NULL);
      GENASSERT(spAttr);
      memset((void*)spAttr, 0, sizeof(R_ATTR));
      break;
    case SEM_POST:
      // get the attrs for this node
      spAttr = (R_ATTR*)vpVecBack(spNRA->vpAttrs);
      
      // get the child attrs - popped but still resident on vector
      spChild = spAttr + 1;
      
      // copy the child attrs
      *spAttr = *spChild;
      vpVecPop(spNRA->vpAttrs);
      break;
    default:
      GENASSERT(APG_FALSE);
      break;
    }
  }
  else
  {
    if(ulState == SEM_POST)
    {
      spAttr = (R_ATTR*)vpVecPush(spNRA->vpAttrs, NULL);
      GENASSERT(spAttr);
      memset((void*)spAttr, 0, sizeof(R_ATTR));

      // recursive leaf nodes are infinite
      spAttr->ulEmpty = APG_FALSE;
      spAttr->ulTerminal = APG_FALSE;
      vpVecPop(spNRA->vpAttrs);
    }
  }
  
APG_EXIT:
  return PFN_OK;
}

static ulong ulSESTNonRecursiveREP(ulong ulState, ulong ulTreeDepth, ulong ulChildCount, ulong ulCountMax, OP_INT_ALL* spOp, void* vpData)
{
  NONRECURSIVEATTRS* spNRA = (NONRECURSIVEATTRS*)vpData;
  GENCTX* spData = spNRA->spData;
  R_ATTR* spAttr;
  R_ATTR* spChild;
  
  switch(ulState)
  {
  case SEM_PRE:
    spAttr = (R_ATTR*)vpVecPush(spNRA->vpAttrs, NULL);
    GENASSERT(spAttr);
    memset((void*)spAttr, 0, sizeof(R_ATTR));
    break;
  case SEM_POST:
    // get the attrs for this node
    spAttr = (R_ATTR*)vpVecBack(spNRA->vpAttrs);
    
    // get the child attrs - popped but still resident on vector
    spChild = spAttr + 1;
    
    // copy the child attrs
    *spAttr = *spChild;
    vpVecPop(spNRA->vpAttrs);
    break;
  default:
    GENASSERT(APG_FALSE);
    break;
  }
  
APG_EXIT:
  return PFN_OK;
}

static ulong ulSESTNonRecursiveTRG(ulong ulState, ulong ulTreeDepth, ulong ulChildCount, ulong ulCountMax, OP_INT_ALL* spOp, void* vpData)
{
  NONRECURSIVEATTRS* spNRA = (NONRECURSIVEATTRS*)vpData;
  GENCTX* spData = spNRA->spData;
  R_ATTR* spAttr;
  DASSERT(spOp->ulOpId == OP_INT_ID_TRG);
  switch(ulState)
  {
  case SEM_PRE:
    break;
  case SEM_POST:
    spAttr = (R_ATTR*)vpVecPush(spNRA->vpAttrs, NULL);
    GENASSERT(spAttr);
    memset((void*)spAttr, 0, sizeof(R_ATTR));
    spAttr->ulTerminal = APG_TRUE;
    vpVecPop(spNRA->vpAttrs);
    break;
  default:
    GENASSERT(APG_FALSE);
    break;
  }
  
APG_EXIT:
  return PFN_OK;
}

static ulong ulSESTNonRecursiveTLS(ulong ulState, ulong ulTreeDepth, ulong ulChildCount, ulong ulCountMax, OP_INT_ALL* spOp, void* vpData)
{
  NONRECURSIVEATTRS* spNRA = (NONRECURSIVEATTRS*)vpData;
  GENCTX* spData = spNRA->spData;
  R_ATTR* spAttr;
  switch(ulState)
  {
  case SEM_PRE:
  	break;
  case SEM_POST:
    spAttr = (R_ATTR*)vpVecPush(spNRA->vpAttrs, NULL);
    GENASSERT(spAttr);
    memset((void*)spAttr, 0, sizeof(R_ATTR));
    spAttr->ulTerminal = APG_TRUE;
    if(spOp->ulLength == 0)
    {
      spAttr->ulEmpty = APG_TRUE;
    }
    vpVecPop(spNRA->vpAttrs);
      break;
  default:
    GENASSERT(APG_FALSE);
      break;
  }
  
APG_EXIT:
  return PFN_OK;
}

static ulong ulSESTRecursiveALT(ulong ulState, ulong ulTreeDepth, ulong ulChildCount, ulong ulCountMax, OP_INT_ALL* spOp, void* vpData)
{
  R_ATTR* spAttr;
  R_ATTR* spChild;
  RECURSIVE_DATA* spRecData = (RECURSIVE_DATA*)vpData;
  GENCTX* spData = spRecData->spData;
  vpData = spRecData->vpAttrStack;
  DASSERT(spOp->ulOpId == OP_INT_ID_ALT);
  
  if(ulChildCount == 0)
  {
    switch(ulState)
    {
    case SEM_PRE:
      spAttr = (R_ATTR*)vpVecPush(vpData, NULL);
      GENASSERT(spAttr);
      
      // initially neither empty nor terminal (never infinite)
      memset((void*)spAttr, 0, sizeof(R_ATTR));
      break;
    case SEM_POST:
      spChild = (R_ATTR*)vpVecPop(vpData);
      GENASSERT(spChild);
      break;
    default:
      GENASSERT(APG_FALSE);
      break;
    }
  }
  else
  {
    if(ulState == SEM_POST)
    {
      // get the attrs for this ALT
      spAttr = (R_ATTR*)vpVecBack(vpData);
      
      // get the child attrs - popped but still resident on vector
      spChild = spAttr + 1;
      if(ulChildCount == 1)
      {
        *spAttr = *spChild;
      }
      else
      {
        // pairwise combination of ALT attrs with child attrs
        // NOTE: if any spAttr attribute is true it remains true
        if(spChild->ulEmpty){spAttr->ulEmpty = APG_TRUE;}
        if(spChild->ulTerminal){spAttr->ulTerminal = APG_TRUE;}
        if(spChild->ulRecursive){spAttr->ulRecursive = APG_TRUE;}
        if(spChild->ulLeft){spAttr->ulLeft = APG_TRUE;}
        if(spChild->ulRight){spAttr->ulRight = APG_TRUE;}
        if(spChild->ulNested){spAttr->ulNested = APG_TRUE;}
        if(spChild->ulCyclic){spAttr->ulCyclic = APG_TRUE;}
      }
    }
  }
  
APG_EXIT:
  return PFN_OK;
}

static ulong ulSESTRecursiveCAT(ulong ulState, ulong ulTreeDepth, ulong ulChildCount, ulong ulCountMax, OP_INT_ALL* spOp, void* vpData)
{
  R_ATTR*         spAttr;
  R_ATTR*         spChild;
  CAT_HELPER*     spHelper;
  ulong           ulTest;
  RECURSIVE_DATA* spRecData = (RECURSIVE_DATA*)vpData;
  GENCTX*         spData = spRecData->spData;
  vpData = spRecData->vpAttrStack;
  DASSERT(spOp->ulOpId == OP_INT_ID_CAT);
  
  if(ulChildCount == 0)
  {
    switch(ulState)
    {
    case SEM_PRE:
      spAttr = (R_ATTR*)vpVecPush(vpData, NULL);
      GENASSERT(spAttr);
      memset((void*)spAttr, 0, sizeof(R_ATTR));

      // create a vector to hold the attributes of all CAT children
      spAttr->vpCatChildStack = vpVecCtor(spData->vpMemCtx, sizeof(CAT_HELPER), 0);
      GENASSERT(spAttr->vpCatChildStack);
      break;
    case SEM_POST:
      spAttr = (R_ATTR*)vpVecPop(vpData);
      GENASSERT(spAttr);

      // compute the recursive attributes for this CAT
      spHelper = (CAT_HELPER*)vpVecFront(spAttr->vpCatChildStack);
      GENASSERT(spHelper);
      ulTest = ulCatRecursiveAttributes(spAttr, spHelper, ulCountMax);
      GENASSERT(ulTest);
      spAttr->ulFirstCat = APG_FALSE;

      // release the vector of CAT child attributes
      vVecDtor(spAttr->vpCatChildStack);
      break;
    default:
      GENASSERT(APG_FALSE);
      break;
    }
  }
  else
  {
    if(ulState == SEM_POST)
    {
      // get the CAT attrs
      spAttr = (R_ATTR*)vpVecBack(vpData);
      GENASSERT(spAttr);
      
      // get & save the child attrs - popped but still resident on vector
      spChild = spAttr + 1;
      spHelper = (CAT_HELPER*)vpVecPush(spAttr->vpCatChildStack, NULL);
      spHelper->sChild = *spChild;
    }
  }
  
APG_EXIT:
  return PFN_OK;
}

static ulong ulSESTRecursiveRNM(ulong ulState, ulong ulTreeDepth, ulong ulChildCount, ulong ulCountMax, OP_INT_ALL* spOp, void* vpData)
{
  ulong ulReturn = PFN_OK;
  R_ATTR* spAttr;
  R_ATTR* spChild;
  INT_RULE_LIST* spRule;
  RECURSIVE_DATA* spRecData = (RECURSIVE_DATA*)vpData;
  GENCTX* spData = spRecData->spData;

  char* cpName;
  char* cpStrTbl;

  DASSERT(spOp->ulOpId == OP_INT_ID_RNM);
  vpData = spRecData->vpAttrStack;
  spRule = &spRecData->spRules[spOp->ulRuleIndex];

  cpStrTbl = (char*)vpVecFront(spData->vpStrTblCtx);
  cpName = cpStrTbl + spRule->ulNameOffset;
  
  if(ulChildCount == ROOT_RULE)
  {
    switch(ulState)
    {
    case SEM_PRE:
      spAttr = (R_ATTR*)vpVecPush(vpData, NULL);
      GENASSERT(spAttr);
      memset((void*)spAttr, 0, sizeof(R_ATTR));
      break;
    case SEM_POST:
      // get the attrs for this node
      spAttr = (R_ATTR*)vpVecBack(vpData);
      
      // get the child attrs - popped but still resident on vector
      spChild = spAttr + 1;

      // copy the child attrs
      *spAttr = *spChild;
      vpVecPop(vpData);
      break;
    default:
      GENASSERT(APG_FALSE);
      break;
    }
  }
  else
  {
    DASSERT(ulChildCount == LEAF_RULE);
//    DASSERT(spOp->ulRuleIndex == spRecData->ulStartRule);
    if(ulState == SEM_POST)
    {
      spAttr = (R_ATTR*)vpVecPush(vpData, NULL);
      GENASSERT(spAttr);
      memset((void*)spAttr, 0, sizeof(R_ATTR));
      spAttr->ulEmpty = spRule->sAttr.ulEmpty;
      spAttr->ulTerminal = spRule->sAttr.ulTerminal;

      if(spOp->ulRuleIndex == spRecData->ulStartRule)
      {
        // set initial recursive attributes for a root rule
        spAttr->ulRecursive = APG_TRUE;
        spAttr->ulLeft = APG_TRUE;
        spAttr->ulRight = APG_TRUE;
        spAttr->ulNested = APG_FALSE;
        spAttr->ulCyclic = APG_TRUE;
        spAttr->ulFirstCat = APG_TRUE;
      }
      
      // else ignore recursive attributes if not the start rule
      vpVecPop(vpData);
    }
  }
  
APG_EXIT:
  return ulReturn;
}

static ulong ulSESTRecursiveREP(ulong ulState, ulong ulTreeDepth, ulong ulChildCount, ulong ulCountMax, OP_INT_ALL* spOp, void* vpData)
{
  R_ATTR* spAttr;
  R_ATTR* spChild;
  RECURSIVE_DATA* spRecData = (RECURSIVE_DATA*)vpData;
  GENCTX* spData = spRecData->spData;
  vpData = spRecData->vpAttrStack;
  
  switch(ulState)
  {
  case SEM_PRE:
    spAttr = (R_ATTR*)vpVecPush(vpData, NULL);
    GENASSERT(spAttr);
    memset((void*)spAttr, 0, sizeof(R_ATTR));
    break;
  case SEM_POST:
    // get the attrs for this node
    spAttr = (R_ATTR*)vpVecBack(vpData);
    
    // get the child attrs - popped but still resident on vector
    spChild = spAttr + 1;
    
    // copy the child attrs
    *spAttr = *spChild;
    vpVecPop(vpData);
    break;
  default:
    GENASSERT(APG_FALSE);
    break;
  }
  
APG_EXIT:
  return PFN_OK;
}

static ulong ulSESTRecursiveTRG(ulong ulState, ulong ulTreeDepth, ulong ulChildCount, ulong ulCountMax, OP_INT_ALL* spOp, void* vpData)
{
  RECURSIVE_DATA* spRecData = (RECURSIVE_DATA*)vpData;
  GENCTX* spData = spRecData->spData;
  R_ATTR* spAttr;
//  DASSERT(spOp->ulOpId == OP_INT_ID_TRG);
  switch(ulState)
  {
  case SEM_PRE:
    break;
  case SEM_POST:
    spAttr = (R_ATTR*)vpVecPush(spRecData->vpAttrStack, NULL);
    GENASSERT(spAttr);
    memset((void*)spAttr, 0, sizeof(R_ATTR));
    spAttr->ulTerminal = APG_TRUE;
    vpVecPop(spRecData->vpAttrStack);
    break;
  default:
    GENASSERT(APG_FALSE);
    break;
  }
  
APG_EXIT:
  return PFN_OK;
}

static ulong ulSESTRecursiveTLS(ulong ulState, ulong ulTreeDepth, ulong ulChildCount, ulong ulCountMax, OP_INT_ALL* spOp, void* vpData)
{
  RECURSIVE_DATA* spRecData = (RECURSIVE_DATA*)vpData;
  GENCTX* spData = spRecData->spData;
  R_ATTR* spAttr;
  switch(ulState)
  {
  case SEM_PRE:
    break;
  case SEM_POST:
    spAttr = (R_ATTR*)vpVecPush(spRecData->vpAttrStack, NULL);
    GENASSERT(spAttr);
    memset((void*)spAttr, 0, sizeof(R_ATTR));
    spAttr->ulTerminal = APG_TRUE;
    if(spOp->ulLength == 0)
    {
      spAttr->ulEmpty = APG_TRUE;
    }
    vpVecPop(spRecData->vpAttrStack);
    break;
  default:
    GENASSERT(APG_FALSE);
    break;
  }
  
APG_EXIT:
  return PFN_OK;
}

static ulong ulCatRecursiveAttributes(R_ATTR* spAttr, CAT_HELPER* spHelper, ulong ulCountMax)
{
  ulong       ulReturn = APG_FALSE;
  ulong       ulIndex;
  void*       vpSave;
  CAT_HELPER* spHelp;
  R_ATTR      sCAT;

  // initialize
  memset((void*)&sCAT, 0, sizeof(R_ATTR));
  sCAT.ulEmpty = APG_TRUE;
  sCAT.ulTerminal = APG_TRUE;

  // get CAT recursive & non-recursive attributes
  spHelp = spHelper;
  for(ulIndex = 0; ulIndex < ulCountMax; ++ulIndex, ++spHelp)
  {
    if(spHelp->sChild.ulRecursive)
    {
      sCAT.ulRecursive = APG_TRUE;
      sCAT.ulTerminal = APG_FALSE;
    }

    if(!spHelp->sChild.ulEmpty)
    {
      sCAT.ulEmpty = APG_FALSE;
    }
  }
  
  if(!sCAT.ulRecursive)
  {
    // if not recursive we are all done
    ulReturn = APG_TRUE;
    goto APG_EXIT;
  }

  // compute left prefixes
  spHelp = spHelper;
  spHelp->sLeftPrefix.ulEmpty = APG_TRUE;
  for(ulIndex = 1; ulIndex < ulCountMax; ++ulIndex, ++spHelp)
  {
    if(spHelp->sLeftPrefix.ulEmpty && spHelp->sChild.ulEmpty)
    {
      spHelp[1].sLeftPrefix.ulEmpty = APG_TRUE;
    }
    else
    {
      spHelp[1].sLeftPrefix.ulEmpty = APG_FALSE;
    }
  }

  // compute right suffixes
  spHelp = spHelper + ulCountMax - 1;
  spHelp->sRightSuffix.ulEmpty = APG_TRUE;
  for(ulIndex = 1; ulIndex < ulCountMax; ++ulIndex, --spHelp)
  {
    if(spHelp->sRightSuffix.ulEmpty && spHelp->sChild.ulEmpty)
    {
      spHelp[-1].sRightSuffix.ulEmpty = APG_TRUE;
    }
    else
    {
      spHelp[-1].sRightSuffix.ulEmpty = APG_FALSE;
    }
  }

  // compute left recursive attribute
  spHelp = spHelper;
  for(ulIndex = 0; ulIndex < ulCountMax; ++ulIndex, ++spHelp)
  {
    if(spHelp->sChild.ulRecursive)
    {
      if(spHelp->sChild.ulLeft && spHelp->sLeftPrefix.ulEmpty == APG_TRUE)
      {
        sCAT.ulLeft = APG_TRUE;
        break;
      }
    }
  }

  // compute right recursive attribute
  spHelp = spHelper + ulCountMax - 1;
  for(ulIndex = 0; ulIndex < ulCountMax; ++ulIndex, --spHelp)
  {
    if(spHelp->sChild.ulRecursive)
    {
      if(spHelp->sChild.ulRight && spHelp->sRightSuffix.ulEmpty == APG_TRUE)
      {
        sCAT.ulRight = APG_TRUE;
        break;
      }
    }
  }
  
  // compute nested recursive attribute
  spHelp = spHelper;
  for(ulIndex = 0; ulIndex < ulCountMax; ++ulIndex, ++spHelp)
  {
    if(spHelp->sChild.ulRecursive)
    {
      if(spHelp->sChild.ulNested)
      {
        // if any child is nested, CAT is nested
        sCAT.ulNested = APG_TRUE;
        break;
      }
      
      if(!spHelp->sLeftPrefix.ulEmpty && !spHelp->sRightSuffix.ulEmpty)
      {
        sCAT.ulNested = APG_TRUE;
        break;
      }

      if(!spHelp->sChild.ulFirstCat)
      {
        if(spHelp->sChild.ulLeft && !spHelp->sLeftPrefix.ulEmpty)
        {
          sCAT.ulNested = APG_TRUE;
          break;
        }
        
        if(spHelp->sChild.ulRight && !spHelp->sRightSuffix.ulEmpty)
        {
          sCAT.ulNested = APG_TRUE;
          break;
        }
      }
    }
  }
  
  // compute cyclic recursive attribute
  spHelp = spHelper;
  for(ulIndex = 0; ulIndex < ulCountMax; ++ulIndex, ++spHelp)
  {
    if(spHelp->sChild.ulCyclic && spHelp->sLeftPrefix.ulEmpty && spHelp->sRightSuffix.ulEmpty)
    {
      sCAT.ulCyclic = APG_TRUE;
      break;
    }
  }

  // !!!! DEBUG !!!!
  // 
  if(sCAT.ulRecursive)
  {
    ulong ulTest = sCAT.ulLeft + sCAT.ulRight + sCAT.ulNested + sCAT.ulCyclic;
    if(!ulTest)
    {
      DASSERT(APG_FALSE);
    }
    
  }
  // !!!! DEBUG !!!!

  // success
  ulReturn = APG_TRUE;

APG_EXIT:
  vpSave = spAttr->vpCatChildStack;
  *spAttr = sCAT;
  spAttr->vpCatChildStack = vpSave;
  return ulReturn;
}

// !!!! DEBUG !!!!
//////////////////////////////////////////////////////////////////////////
// For printing selected branches of the SEST
//////////////////////////////////////////////////////////////////////////
static ulong ulSESTBranchMetricsRNM(ulong ulState, ulong ulTreeDepth, ulong ulChildCount, ulong ulCountMax, OP_INT_ALL* spOp, void* vpData)
{
  SEST_METRICS* spMetrics = (SEST_METRICS*)vpData;
  BranchEntry* spBranch;
  INT_RULE_LIST* spRule;
  
  // validate
  DASSERT(spOp->ulOpId == OP_INT_ID_RNM);
  if(ulState == SEM_POST)
  {
    spBranch = (BranchEntry*)vpVecPop(g_sBranchData.vpRNMStack);
    DASSERT(spBranch);
    ++spMetrics->ulTotalNodes;
    if(ulChildCount == ROOT_RULE)
    {
      ++spMetrics->ulTotalNonTerminal;
      ++spMetrics->ulRNMNodes;
    }
    else
    {
      ++spMetrics->ulTotalTerminal;
      ++spMetrics->ulRNMLeaves;
      if(spOp->ulRuleIndex == spMetrics->ulStartRule)
      {
        ++spMetrics->ulRootRNMLeaves;
      }
    }
  }
  else
  {
    spRule = &g_sBranchData.spRules[spOp->ulRuleIndex];
    spBranch = (BranchEntry*)vpVecPush(g_sBranchData.vpRNMStack, NULL);
    DASSERT(spBranch);
    spBranch->cpRuleName = g_sBranchData.cpStrTbl + spRule->ulNameOffset;
    spBranch->ulRecursive = APG_FALSE;
    if(spRule->sAttr.ulRecursive)
    {
      spBranch->ulRecursive = APG_TRUE;
    }

    if(ulTreeDepth >= g_sBranchData.ulTreeDepthThreshold)
    {
      ++g_sBranchData.ulBranches;
      if(g_sBranchData.ulBranches < g_sBranchData.ulMaxPrints)
      {
        ulong ulLen;
        spBranch = vpVecFront(g_sBranchData.vpRNMStack);
        DASSERT(spBranch);
        
        ulLen = ulVecSize(g_sBranchData.vpRNMStack);
        vPrintBranch(spBranch, ulLen, spMetrics->ulTerminalLeaves + spMetrics->ulRNMLeaves);
        
      }
    }
  }
  
  return PFN_OK;
}

static ulong ulSESTBranchMetricsTRG(ulong ulState, ulong ulTreeDepth, ulong ulChildCount, ulong ulCountMax, OP_INT_ALL* spOp, void* vpData)
{
  SEST_METRICS* spMetrics = (SEST_METRICS*)vpData;
  
  // validate
  DASSERT(spOp->ulOpId == OP_INT_ID_TRG);
  if(ulState == SEM_POST)
  {
    ++spMetrics->ulTRGLeaves;
    ++spMetrics->ulTotalNodes;
    ++spMetrics->ulTotalTerminal;
    ++spMetrics->ulTerminalLeaves;
  }
  else
  {
    if(ulTreeDepth > g_sBranchData.ulMaxTreeDepth)
    {
      g_sBranchData.ulMaxTreeDepth = ulTreeDepth;
    }
    if(ulTreeDepth >= g_sBranchData.ulTreeDepthThreshold)
    {
      ++g_sBranchData.ulBranches;
      if(g_sBranchData.ulBranches < g_sBranchData.ulMaxPrints)
      {
        BranchEntry* spBranch;
        ulong ulLen;
        spBranch = vpVecFront(g_sBranchData.vpRNMStack);
        DASSERT(spBranch);
        
        ulLen = ulVecSize(g_sBranchData.vpRNMStack);
        vPrintBranch(spBranch, ulLen, spMetrics->ulTerminalLeaves + spMetrics->ulRNMLeaves);

      }
    }
  }
  
  return PFN_OK;
}

static ulong ulSESTBranchMetricsTBS(ulong ulState, ulong ulTreeDepth, ulong ulChildCount, ulong ulCountMax, OP_INT_ALL* spOp, void* vpData)
{
  SEST_METRICS* spMetrics = (SEST_METRICS*)vpData;
  
  // validate
  DASSERT(spOp->ulOpId == OP_INT_ID_TBS);
  if(ulState == SEM_POST && spOp->ulLength > 0)
  {
    ++spMetrics->ulTBSLeaves;
    ++spMetrics->ulTotalNodes;
    ++spMetrics->ulTotalTerminal;
    ++spMetrics->ulTerminalLeaves;
  }
  else
  {
    if(ulTreeDepth > g_sBranchData.ulMaxTreeDepth)
    {
      g_sBranchData.ulMaxTreeDepth = ulTreeDepth;
    }
    if(ulTreeDepth >= g_sBranchData.ulTreeDepthThreshold)
    {
      ++g_sBranchData.ulBranches;
      if(g_sBranchData.ulBranches < g_sBranchData.ulMaxPrints)
      {
        BranchEntry* spBranch;
        ulong ulLen;
        spBranch = vpVecFront(g_sBranchData.vpRNMStack);
        DASSERT(spBranch);
        
        ulLen = ulVecSize(g_sBranchData.vpRNMStack);
        vPrintBranch(spBranch, ulLen, spMetrics->ulTerminalLeaves + spMetrics->ulRNMLeaves);
        
      }
    }
  }
  
  return PFN_OK;
}

static ulong ulSESTBranchMetricsTLS(ulong ulState, ulong ulTreeDepth, ulong ulChildCount, ulong ulCountMax, OP_INT_ALL* spOp, void* vpData)
{
  SEST_METRICS* spMetrics = (SEST_METRICS*)vpData;
  
  // validate
  DASSERT(spOp->ulOpId == OP_INT_ID_TLS);
  if(ulState == SEM_POST)
  {
    ++spMetrics->ulTLSLeaves;
    ++spMetrics->ulTotalNodes;
    ++spMetrics->ulTotalTerminal;
    ++spMetrics->ulTerminalLeaves;
    if(spOp->ulLength == 0)
    {
      ++spMetrics->ulEmptyStringLeaves;
    }
  }
  else
  {
    if(ulTreeDepth > g_sBranchData.ulMaxTreeDepth)
    {
      g_sBranchData.ulMaxTreeDepth = ulTreeDepth;
    }
    if(ulTreeDepth >= g_sBranchData.ulTreeDepthThreshold)
    {
      ++g_sBranchData.ulBranches;
      if(g_sBranchData.ulBranches < g_sBranchData.ulMaxPrints)
      {
        BranchEntry* spBranch;
        ulong ulLen;
        spBranch = vpVecFront(g_sBranchData.vpRNMStack);
        DASSERT(spBranch);
        
        ulLen = ulVecSize(g_sBranchData.vpRNMStack);
        vPrintBranch(spBranch, ulLen, spMetrics->ulTerminalLeaves + spMetrics->ulRNMLeaves);
        
      }
    }
  }
  
  return PFN_OK;
}

static ulong ulSESTBranchMetricsALT(ulong ulState, ulong ulTreeDepth, ulong ulChildCount, ulong ulCountMax, OP_INT_ALL* spOp, void* vpData)
{
  SEST_METRICS* spMetrics = (SEST_METRICS*)vpData;
  BranchEntry* spBranch;
  static char* cpName = "ALT";
  
  // validate
  DASSERT(spOp->ulOpId == OP_INT_ID_ALT);
  if(ulState == SEM_POST && ulChildCount == 0 && spOp->ulCount > 1)
  {
    spBranch = (BranchEntry*)vpVecPop(g_sBranchData.vpRNMStack);
    DASSERT(spBranch);
    ++spMetrics->ulALTNodes;
    ++spMetrics->ulTotalNodes;
    ++spMetrics->ulTotalNonTerminal;
  }
  else if(ulState == SEM_PRE && ulChildCount == 0 && spOp->ulCount > 1)
  {
    spBranch = (BranchEntry*)vpVecPush(g_sBranchData.vpRNMStack, NULL);
    DASSERT(spBranch);
    spBranch->cpRuleName = cpName;
    spBranch->ulRecursive = APG_FALSE;
  }
  
  return PFN_OK;
}

static ulong ulSESTBranchMetricsCAT(ulong ulState, ulong ulTreeDepth, ulong ulChildCount, ulong ulCountMax, OP_INT_ALL* spOp, void* vpData)
{
  SEST_METRICS* spMetrics = (SEST_METRICS*)vpData;
  BranchEntry* spBranch;
  static char* cpName = "CAT";
  
  // validate
  DASSERT(spOp->ulOpId == OP_INT_ID_CAT);
  if(ulState == SEM_POST && ulChildCount == 0 && spOp->ulCount > 1)
  {
    spBranch = (BranchEntry*)vpVecPop(g_sBranchData.vpRNMStack);
    DASSERT(spBranch);
    ++spMetrics->ulCATNodes;
    ++spMetrics->ulTotalNodes;
    ++spMetrics->ulTotalNonTerminal;
  }
  else if(ulState == SEM_PRE && ulChildCount == 0 && spOp->ulCount > 1)
  {
    spBranch = (BranchEntry*)vpVecPush(g_sBranchData.vpRNMStack, NULL);
    DASSERT(spBranch);
    spBranch->cpRuleName = cpName;
    spBranch->ulRecursive = APG_FALSE;
  }
  
  return PFN_OK;
}

static ulong ulSESTBranchMetricsREP(ulong ulState, ulong ulTreeDepth, ulong ulChildCount, ulong ulCountMax, OP_INT_ALL* spOp, void* vpData)
{
  SEST_METRICS* spMetrics = (SEST_METRICS*)vpData;
  BranchEntry* spBranch;
  static char* cpName = "REP";
  
  // validate
  DASSERT(spOp->ulOpId == OP_INT_ID_REP);
  if(ulState == SEM_POST && ulChildCount == 0 && !(spOp->ulMin == 1 && spOp->ulMax == 1))
  {
    spBranch = (BranchEntry*)vpVecPop(g_sBranchData.vpRNMStack);
    DASSERT(spBranch);
    ++spMetrics->ulREPNodes;
    ++spMetrics->ulTotalNodes;
    ++spMetrics->ulTotalNonTerminal;
  }
  else if(ulState == SEM_PRE && ulChildCount == 0 && !(spOp->ulMin == 1 && spOp->ulMax == 1))
  {
    spBranch = (BranchEntry*)vpVecPush(g_sBranchData.vpRNMStack, NULL);
    DASSERT(spBranch);
    spBranch->cpRuleName = cpName;
    spBranch->ulRecursive = APG_FALSE;
  }
  
  return PFN_OK;
}

static ulong ulSESTBranchMetricsPRD(ulong ulState, ulong ulTreeDepth, ulong ulChildCount, ulong ulCountMax, OP_INT_ALL* spOp, void* vpData)
{
  SEST_METRICS* spMetrics = (SEST_METRICS*)vpData;
  BranchEntry* spBranch;
  static char* cpName = "PRD";
  
  // validate
  DASSERT(spOp->ulOpId == OP_INT_ID_PRD);
  if(ulState == SEM_POST && ulChildCount == 0)
  {
    spBranch = (BranchEntry*)vpVecPop(g_sBranchData.vpRNMStack);
    DASSERT(spBranch);
    ++spMetrics->ulPRDNodes;
    ++spMetrics->ulTotalNodes;
    ++spMetrics->ulTotalNonTerminal;
  }
  else if(ulState == SEM_PRE && ulChildCount == 0)
  {
    spBranch = (BranchEntry*)vpVecPush(g_sBranchData.vpRNMStack, NULL);
    DASSERT(spBranch);
    spBranch->cpRuleName = cpName;
    spBranch->ulRecursive = APG_FALSE;
  }
  
  return PFN_OK;
}
// !!!! DEBUG !!!!
