/********************************************************************
  APG - an ABNF Parser Generator
  Copyright  (C) 2007 Coast to Coast Research, Inc.
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program.  If not, see
  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
  or write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

    author: Lowell Thomas
            lowell@coasttocoastresearch.com
            http://www.coasttocoastresearch.com

   created: 8/26/2007 13:00
  filename: D:\Projects\Apgv5_0\Samples\CPlusPlus\main.cpp
   purpose: Tests the C++ wrapper for the "currency" parser.
  
*********************************************************************/
#include "Apg.h"
#include "ApgUtilities.h"
#include "Currency.h"

#define MASSERT(cond) UASSERT((cond), oCurrency.m_vpMemCtx, 101)

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: Driver program
//
// FUNCTION: int main
//
//      ARG: 1) int argc
//              : number of command line arguments
//
//      ARG: 2) char** argv
//              : pointer to a list of pointers to command line arguments
//
//   RETURN: 
//
//////////////////////////////////////////////////////////////////////////
int main(int argc, char** argv)
{
  FILE*     spRedirectOut = NULL;
  APG_BSTR  sSrc;
  char*     cpInput;
  ulong     ulTest;
  ulong     ulTrace = APG_FALSE;
  void*     vpSyntaxData = NULL;
  void*     vpSemanticData = NULL; // they need not be the same
  
  // CONSTRUCTOR
  // NOTE: constructor must occur before first MASSERT due to the "goto"
  //       which conflicts with the compiler's scope handling
  ApgCurrency::SEMANTIC_DATA sSemanticData;
  ApgCurrency oCurrency;
  MASSERT(oCurrency.m_vpParserCtx);

  cpInput = "CurrencyInput.txt";

  // GPL and copyright notice
  vLicenseNotice();

  // redirect output
//   printf("%s - output redirect file\n", "main.log");
//   spRedirectOut = freopen("main.log", "w", stdout);
//   MASSERT(spRedirectOut);
//   vLicenseNotice();

  // set the custom alert handler
  pfnSetAlertHandler(vTerminalAlert);

  // get the input string
  sSrc.ulLen = ulGetFileSize(cpInput);
  MASSERT(sSrc.ulLen);
  sSrc.ucpStr = (uchar*)new char[sSrc.ulLen + 10];
  MASSERT(sSrc.ucpStr);
  sSrc.ulLen = ulGetFile(cpInput, (void*)sSrc.ucpStr);
  MASSERT(sSrc.ulLen);
  sSrc.ucpStr[sSrc.ulLen] = 0;

  // SYNTAX ANALYSIS
  ulTest = oCurrency.ulSyntaxAnalysis(ApgCurrency::ID_LIST, sSrc.ucpStr, sSrc.ulLen);
  MASSERT(ulTest);

  // print the state and stats
  vPrintState(&oCurrency.m_sState, "Currency Test");
  vPrintStats(oCurrency.m_vpParserCtx);

  // SEMANTIC ANALYSIS
  if(oCurrency.m_sState.ulState == MATCH)
  {
    sSemanticData.ucpSrc = sSrc.ucpStr;
    sSemanticData.ulSrcLen = sSrc.ulLen;
    vpSemanticData = (void*)&sSemanticData;
    ulTest = oCurrency.ulSemanticAnalysis(vpSemanticData);
    MASSERT(ulTest);
  }

APG_EXIT:
  if(oCurrency.m_vpMemCtx)
  {
    // display any error messages
    vPrintErrors(oCurrency.m_vpMemCtx);
  }

  return 0;
}

