/********************************************************************
  APG - an ABNF Parser Generator
  Copyright  (C) 2007 Coast to Coast Research, Inc.
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program.  If not, see
  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
  or write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

    author: Lowell Thomas
            lowell@coasttocoastresearch.com
            http://www.coasttocoastresearch.com

   created: 8/26/2007 17:48
  filename: D:\Projects\Apgv5_0\Samples\IPv6\Simple.c
   purpose: Test the simple version of the IPv4 & IPv6 addresses
                with validations in the semantic analysis.
  
*********************************************************************/
#include "main.h"

static void vRunSimple(TESTDATA* spTestData);

void vTestSimple(void* vpMemCtx, char* cpFileName)
{
  APG_BSTR          sSrc;
  void*             vpBstr;
  ulong             ulCount;
  TESTDATA          sData;

  // some valid and malformed IPv6 address
  sSrc.ulLen = ulGetFileSize(cpFileName);
  MAINASSERT(sSrc.ulLen);
  sSrc.ucpStr = (uchar*)vpMemAlloc(vpMemCtx, sSrc.ulLen + 10);
  MAINASSERT(sSrc.ucpStr);
  sSrc.ulLen = ulGetFile(cpFileName, (void*)sSrc.ucpStr);
  MAINASSERT(sSrc.ulLen);
  sSrc.ucpStr[sSrc.ulLen] = 0;

  // set up an error report
  sData.vpErrorsCtx = vpErrorsCtor(vpMemCtx);
  MAINASSERT(sData.vpErrorsCtx);
  
  // get the list of announcements
  vpBstr = vpVecCtor(vpMemCtx, sizeof(APG_BSTR), 0);
  MAINASSERT(vpBstr);
  ulCount = ulReadAddresses(&sSrc, vpBstr);
  MAINASSERT(ulCount);
  
  sData.vpMsgs = vpBstr;
  sData.ulRepetitions = 1;
  
  // syntax test with PPT
  sData.cpTitle = "//////////////////////////////////////////////////////////////////////////\n"\
    "// SIMPLE IPV4 & IPV6 GRAMMARS\n"\
    "//////////////////////////////////////////////////////////////////////////";
  vRunSimple(&sData);
  
  // clean up resources
  vVecDtor(vpBstr);
  vMemFree(vpMemCtx, sSrc.ucpStr);
}

void vRunSimple(TESTDATA* spTestData)
{
  APG_BSTR*         spBstr;
  ushort            uspAddress[IPV6_SHORTS];
  char              cpIPv6String[IPV6_MAXLEN + 1];
  ulong             ulIsMulticast;
  ulong             ulTest;
  ulong             ulCount;
  ulong             ulAdd;
  DASSERT(spTestData);
  DASSERT(spTestData->vpMsgs);
  
  if(spTestData->cpTitle && *spTestData->cpTitle != 0)
  {
    // print a header
    printf("\n");
    printf("%s\n", spTestData->cpTitle);
  }

  
  // get the list of addresses
  spBstr = (APG_BSTR*)vpVecFront(spTestData->vpMsgs);
  MAINASSERT(spBstr);
  ulCount = ulVecSize(spTestData->vpMsgs);
  MAINASSERT(ulCount);
  
  // translate each address
  for(ulAdd = 0; ulAdd < ulCount; ++ulAdd, ++spBstr)
  {
    ulTest = ulIPv6StringToBinary(spBstr, uspAddress, &ulIsMulticast);
    if(ulTest)
    {
      printf("\n");
      printf("[%03lu]      original - %s\n", ulAdd, (char*)spBstr->ucpStr);

      // multicast
      if(ulIsMulticast)
      {
        printf("          Multicast - yes\n");
      }
      else
      {
        printf("          Multicast - no\n");
      }
      ulIPv6BinaryToString(uspAddress, cpIPv6String);
      printf("        normal form - %s\n", cpIPv6String);
    }
    else
    {
      printf("\n");
      printf("[%03lu]     malformed - %s\n", ulAdd, (char*)spBstr->ucpStr);
    }
  }
  
}
