/********************************************************************
  APG - an ABNF Parser Generator
  Copyright  (C) 2007 Coast to Coast Research, Inc.
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program.  If not, see
  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
  or write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

    author: Lowell Thomas
            lowell@coasttocoastresearch.com
            http://www.coasttocoastresearch.com

   created: 8/26/2007 11:36
  filename: D:\Projects\Apgv5_0\Samples\NonContextFree\TestAnBnCn.c
   purpose: Tests the non-context-free grammar for a**nb**nc**n, n > 1.
  
*********************************************************************/
#include "main.h"

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: Runs a test of the AnBnCn grammar on a variety of input strings.
//
// FUNCTION: void vTestAnBnCn
//
//      ARG: 1) void* vpMemCtx
//              : pointer to a valid memory component context
//
//   RETURN: void
//
//////////////////////////////////////////////////////////////////////////
void vTestAnBnCn(void* vpMemCtx)
{
  ulong ulTest;
  ulong ulIndex;
  char* cpInput;
  APG_BSTR sSrc;
  void* vpParserCtx;
  APG_PARSER_STATE sState;

  printf("\n");
  printf("Test parser for non-context free grammar a^nb^nc^n ,n >= 1\n");
  ulTest = ulParserInit(vpAnBnCnInit, vpMemCtx);
  MAINASSERT(ulTest);
  vpParserCtx = vpParserCtor(vpAnBnCnInit, vpMemCtx);
  for(ulIndex = 0; ulIndex < 4; ++ulIndex)
  {
    switch(ulIndex)
    {
    case 0:
      cpInput = "AnBnCn.txt";
      break;
    case 1:
      cpInput = "AxBnCn.txt";
      break;
    case 2:
      cpInput = "AnBxCn.txt";
      break;
    case 3:
      cpInput = "AnBnCx.txt";
      break;
    default:
      break;
    }
    
    // get an input string
    sSrc.ulLen = ulGetFileSize(cpInput);
    MAINASSERT(sSrc.ulLen);
    sSrc.ucpStr = (uchar*)vpMemAlloc(vpMemCtx, sSrc.ulLen + 10);
    MAINASSERT(sSrc.ucpStr);
    sSrc.ulLen = ulGetFile(cpInput, (void*)sSrc.ucpStr);
    MAINASSERT(sSrc.ulLen);
    sSrc.ucpStr[sSrc.ulLen] = 0;
    
    ulTest = ulParserSyntaxAnalysis(vpParserCtx, 0, sSrc.ucpStr, sSrc.ulLen, (void*)NULL, APG_FALSE);
    MAINASSERT(ulTest);
    ulTest = ulParserState(vpParserCtx, &sState);
    vPrintState(&sState, (char*)sSrc.ucpStr);
  }
  
  
  // clean up
  vParserDtor(vpParserCtx);

}
