/********************************************************************
  APG - an ABNF Parser Generator
  Copyright  (C) 2007 Coast to Coast Research, Inc.
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program.  If not, see
  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
  or write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  
  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.

    author: Lowell Thomas
            lowell@coasttocoastresearch.com
            http://www.coasttocoastresearch.com

   created: 8/27/2007 10:13
  filename: D:\Projects\Apgv5_0\Samples\SDP\Accuracy.c
  purpose: Tests the parser for accuracy. Reads a list of
  announcements and verifies that the parser correctly
  identifies syntactically correct announcements and
                rejects those with errors.
  
*********************************************************************/
#include "main.h"

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: Tests the syntactic accuracy of an ADP announcement
//
// FUNCTION: ulong ulAccuracyTest
//
//      ARG: 1) APG_BSTR* spBstr
//              : binary string defining the announcement
//
//      ARG: 2) void* vpMemCtx
//              : pointer to a valid memory component context
//
//      ARG: 3) void* vpParserCtx
//              : pointer to a valid parser component context
//
//   RETURN: true on success
//
//////////////////////////////////////////////////////////////////////////
ulong ulAccuracyTest(APG_BSTR* spBstr, void* vpMemCtx, void* vpParserCtx)
{
  ulong ulReturn = APG_FALSE;
  APG_PARSER_STATE sState;
  ulong ulTest;
  
  ulTest = ulParserSyntaxAnalysis(vpParserCtx, 0, spBstr->ucpStr, spBstr->ulLen, NULL, APG_FALSE);
  SDPASSERT(ulTest);
  ulTest = ulParserState(vpParserCtx, &sState);
  SDPASSERT(ulTest);
  if(sState.ulState == MATCH)
  {
    printf("Accuracy Test - syntax success\n");
  }
  else
  {
    printf("Accuracy Test - syntax failed\n");
  }

  ulReturn = APG_TRUE;
  return ulReturn;
}

