/********************************************************************
  APG - an ABNF Parser Generator
  Copyright  (C) 2007 Coast to Coast Research, Inc.
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program.  If not, see
  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
  or write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  
  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.

    author: Lowell Thomas
            lowell@coasttocoastresearch.com
            http://www.coasttocoastresearch.com

   created: 8/27/2007 10:14
  filename: D:\Projects\Apgv5_0\Samples\SDP\Components.c
  purpose: Tests finding the announcement components using
                the parser catalog feature.
  
*********************************************************************/
#include "main.h"

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: Driver for tests finding the announcement components using
//              the parser catalog feature.
//
// FUNCTION: void vTestComponents
//
//      ARG: 1) void* vpMemCtx
//              : pointer to a valid memory component context
//
//      ARG: 2) char* cpInput
//              : pointer to the input file name
//                containing a list of announcements
//
//      ARG: 3) PFN_TEST pfnTest
//              : pointer to one of several test functions
//
//   RETURN: 
//
//////////////////////////////////////////////////////////////////////////
void vTestComponents(void* vpMemCtx, char* cpInput, PFN_TEST pfnTest)
{
  void*     vpParserCtx = NULL;
  APG_BSTR  sSrc;
  APG_BSTR* spBstr;
  void*     vpBstr;
  ulong     ulTest;
  ulong     ulCount;
  ulong     ulAnn;
  
  // get an input string
  sSrc.ulLen = ulGetFileSize(cpInput);
  SDPASSERT(sSrc.ulLen);
  sSrc.ucpStr = (uchar*)vpMemAlloc(vpMemCtx, sSrc.ulLen + 10);
  SDPASSERT(sSrc.ucpStr);
  sSrc.ulLen = ulGetFile(cpInput, (void*)sSrc.ucpStr);
  SDPASSERT(sSrc.ulLen);
  sSrc.ucpStr[sSrc.ulLen] = 0;
  
  // get the list of announcements
  vpBstr = vpVecCtor(vpMemCtx, sizeof(APG_BSTR), 0);
  SDPASSERT(vpBstr);
  ulCount = ulReadAnnouncements(&sSrc, vpBstr);
  SDPASSERT(ulCount);

  // get a parser component
  vpParserCtx = vpParserCtor(vpSDPInit, vpMemCtx);
  SDPASSERT(vpParserCtx);

  // iterate through the announcements
  spBstr = (APG_BSTR*)vpVecFront(vpBstr);
  SDPASSERT(spBstr);
  for(ulAnn = 0; ulAnn < ulCount; ++ulAnn, ++spBstr)
  {
    printf("\n");
    printf("SDP %lu\n", ulAnn);
    vDisplaySourceLines((char*)spBstr->ucpStr, stdout);
    ulTest = pfnTest(spBstr, vpMemCtx, vpParserCtx);
    SDPASSERT(ulTest);
  }
  
  // clean up
  if(vpParserCtx){vParserDtor(vpParserCtx);}
}
