/********************************************************************
  APG - an ABNF Parser Generator
  Copyright  (C) 2007 Coast to Coast Research, Inc.
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program.  If not, see
  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
  or write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

    author: Lowell Thomas
            lowell@coasttocoastresearch.com
            http://www.coasttocoastresearch.com

   created: 8/27/2007 10:16
  filename: D:\Projects\Apgv5_0\Samples\SDP\SDPLines.c
   purpose: A simple iterator to select and identify one line
                at a time of an SDP announcement.
  
*********************************************************************/
#include "main.h"

#undef  LASSERT
#undef  CR
#undef  LF
#define CR 13
#define LF 10
#define LOOK_SECTION  0 // looking for next section field
#define LOOK_MEDIA    1 // looking for next media field
#define LASSERT(cond) {if(!(int)(cond))goto APG_EXIT;}

static LINE_NAME g_spLinesTranslator[SESSION_COUNT] =
{
  {SESSION_VERSION,     "v=", "version"},
  {SESSION_ORIGIN,      "o=", "origin"},
  {SESSION_NAME,        "s=", "session name"},
  {SESSION_INFORMATION, "i=", "information"},
  {SESSION_URI,         "u=", "uri"},
  {SESSION_EMAIL,       "e=", "email"},
  {SESSION_PHONE,       "p=", "phone"},
  {SESSION_CONNECTION,  "c=", "connection"},
  {SESSION_BANDWIDTH,   "b=", "bandwidth"},
  {SESSION_TIME,        "t=", "time"},
  {SESSION_REPEAT,      "r=", "repeat times"},
  {SESSION_ZONE,        "z=", "zone adjustments"},
  {SESSION_KEY,         "k=", "key"},
  {SESSION_ATTRIBUTE,   "a=", "attribute"},
  {MEDIA_MEDIA,         "m=", "media"},
  {MEDIA_INFORMATION,   "i=", "information"},
  {MEDIA_CONNECTION,    "c=", "connection"},
  {MEDIA_BANDWIDTH,     "b=", "bandwidth"},
  {MEDIA_KEY,           "k=", "key"},
  {MEDIA_ATTRIBUTE,     "a=", "attribute"}
};
LINE_NAME* spLinesTranslator = &g_spLinesTranslator[0];

static ulong ulFindLine(LINE_ITERATOR* spIter, APG_BSTR* spLine, long* ulpNextID);

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: The line iterator finds one line at a time. Used
//              for lazy parsing. Allows each line to be skipped or
//              parsed as needed.
//
//              Initialize a line iterator. 
//
// FUNCTION: ulong ulLinesInit
//
//      ARG: 1) LINE_ITERATOR* spIter
//              : pointer to a line iterator structs
//
//      ARG: 2) APG_BSTR* spAnnouncement
//              : binary string defining the SDP announcement
//
//   RETURN: true on success
//
//////////////////////////////////////////////////////////////////////////
ulong ulLinesInit(LINE_ITERATOR* spIter, APG_BSTR* spAnnouncement)
{
  ulong ulReturn = APG_FALSE;
  DASSERT(spIter);
  DASSERT(spAnnouncement);
  DASSERT(spAnnouncement->ucpStr);
  DASSERT(spAnnouncement->ulLen);
  
  // success
  spIter->sNextLine = *spAnnouncement;
  spIter->ulMode = LOOK_SECTION;
  ulReturn = APG_TRUE;
  
  return ulReturn;
}

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: Find the next line in a specified announcement.
//
// FUNCTION: ulong ulLinesNext
//
//      ARG: 1) LINE_ITERATOR* spIter
//              : pointer to a line iterator struct that has been
//                previously initialized by a call to ulLinesInit().
//
//      ARG: 2) APG_BSTR* spLine
//              : on return a binary string defining the next line.
//
//      ARG: 3) ulong* ulpID
//              : the parser component rule name identifier of the next line
//                see SDP.h for named rule identifiers.
//
//   RETURN: true on success
//
//////////////////////////////////////////////////////////////////////////
ulong ulLinesNext(LINE_ITERATOR* spIter, APG_BSTR* spLine, ulong* ulpID)
{
  ulong ulReturn = APG_FALSE;
  ulong ulTest;
  DASSERT(spIter);
  DASSERT(spLine);
  DASSERT(ulpID);

  // default
  spLine->ucpStr = NULL;
  spLine->ulLen = 0;
  *ulpID = SESSION_COUNT;
  if(spIter->sNextLine.ucpStr)
  {
    // find the next line
    DASSERT(spIter->sNextLine.ulLen);
    DASSERT(spIter->ulMode <= LOOK_MEDIA);
    ulTest = ulFindLine(spIter, spLine, ulpID);
    LASSERT(ulTest);
  }
  // else return NULL line

  // success
  ulReturn = APG_TRUE;
  
APG_EXIT:
  return ulReturn;
}

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: Helper function for ulLinesInit(). Actually does the work.
//
// FUNCTION: static ulong ulFindLine
//
//      ARG: 1) LINE_ITERATOR* spIter
//              : pointer to a line iterator struct that has been
//                previously initialized by a call to ulLinesInit().
//
//      ARG: 2) APG_BSTR* spLine
//              : on return a binary string defining the next line.
//
//      ARG: 3) ulong* ulpNextID
//              : the parser component rule name identifier of the next line
//                see SDP.h for named rule identifiers.
//
//   RETURN: true on success
//
//////////////////////////////////////////////////////////////////////////
static ulong ulFindLine(LINE_ITERATOR* spIter, APG_BSTR* spLine, long* ulpNextID)
{
  ulong ulReturn = APG_FALSE;
  ulong ulID;
  ulong ulChangeMode;
  char* cpLine;
  char* cpLineEnd;
  char* cpEnd;
  DASSERT(spIter);
  DASSERT(spIter->sNextLine.ucpStr);
  DASSERT(spIter->sNextLine.ulLen);
  DASSERT(spLine);
  DASSERT(ulpNextID);

  // initialize
  cpLine = (char*)spIter->sNextLine.ucpStr;
  cpEnd = cpLine + spIter->sNextLine.ulLen;
  ulChangeMode = APG_FALSE;
  spLine->ucpStr = NULL;
  spLine->ulLen = 0;
  *ulpNextID = SESSION_COUNT;

  while(cpLine < cpEnd)
  {
    // identify the line
    LASSERT(cpLine[1] == '=');
    if(spIter->ulMode == LOOK_SECTION)
    {
      // catalog all session section lines
      switch(*cpLine)
      {
      case 'v':
        ulID = SESSION_VERSION;
        break;
      case 'o':
        ulID = SESSION_ORIGIN;
        break;
      case 's':
        ulID = SESSION_NAME;
        break;
      case 'i':
        ulID = SESSION_INFORMATION;
        break;
      case 'u':
        ulID = SESSION_URI;
        break;
      case 'e':
        ulID = SESSION_EMAIL;
        break;
      case 'p':
        ulID = SESSION_PHONE;
        break;
      case 'c':
        ulID = SESSION_CONNECTION;
        break;
      case 'b':
        ulID = SESSION_BANDWIDTH;
        break;
      case 't':
        ulID = SESSION_TIME;
        break;
      case 'r':
        ulID = SESSION_REPEAT;
        break;
      case 'z':
        ulID = SESSION_ZONE;
        break;
      case 'k':
        ulID = SESSION_KEY;
        break;
      case 'a':
        ulID = SESSION_ATTRIBUTE;
        break;
      case 'm':
        // change mode
        ulChangeMode = APG_TRUE;
        spIter->ulMode = LOOK_MEDIA;
        break;
      default:
        LASSERT(APG_FALSE); // error
        break;
      }
    }
    else
    {
      // catalog all media section lines
      switch(*cpLine)
      {
      case 'm':
        ulID = MEDIA_MEDIA;
        break;
      case 'i':
        ulID = MEDIA_INFORMATION;
        break;
      case 'c':
        ulID = MEDIA_CONNECTION;
        break;
      case 'b':
        ulID = MEDIA_BANDWIDTH;
        break;
      case 'k':
        ulID = MEDIA_KEY;
        break;
      case 'a':
        ulID = MEDIA_ATTRIBUTE;
        break;
      default:
        LASSERT(APG_FALSE);
        break;
      }
    }

    if(ulChangeMode)
    {
      // switch to "looking for media fields" mode
      spIter->ulMode = LOOK_MEDIA;
      ulChangeMode = APG_FALSE;
      continue;
    }

    // find the line end
    cpLineEnd = cpLine + 2;
    while(*cpLineEnd != CR)
    {
      LASSERT(cpLineEnd < cpEnd);
      ++cpLineEnd;
    }

    // validations
    LASSERT(cpLineEnd[1] == LF);        // line ends in CRLF
    cpLine += 2;
    LASSERT(cpLine < cpLineEnd);  // line has at least three characters (x=yCRLF)

    // return the line (stripped of "x=" and CRLF
    spLine->ucpStr = (uchar*)cpLine;
    spLine->ulLen = (ulong)(cpLineEnd - cpLine);
    *ulpNextID = ulID;
    
    // update the iterator
    ulChangeMode = spLine->ulLen + 4;
    spIter->sNextLine.ulLen -= ulChangeMode;
    if(spIter->sNextLine.ulLen > 0)
    {
      spIter->sNextLine.ucpStr += ulChangeMode;
    } 
    else
    {
      spIter->sNextLine.ucpStr = NULL;
    }
    break;
  }

  // success
  ulReturn = APG_TRUE;

APG_EXIT:
  return ulReturn;
}
