/********************************************************************
  APG - an ABNF Parser Generator
  Copyright  (C) 2007 Coast to Coast Research, Inc.
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program.  If not, see
  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
  or write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

    author: Lowell Thomas
            lowell@coasttocoastresearch.com
            http://www.coasttocoastresearch.com

   created: 8/27/2007 10:17
  filename: D:\Projects\Apgv5_0\Samples\SDP\Statistics.c
   purpose: Compares statistics between with and without PPT
  
*********************************************************************/
#include "main.h"

static void vPrintOperatorLine(char* cpOp, APG_NODE_STATS* spNode, APG_NODE_STATS* spTotal);
static void vPrintStatistics(APG_PARSER_STATS* spStats);

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: Driver for the tests.
//              Compares statistics between with and without PPT
//
// FUNCTION: void vTestStatistics
//
//      ARG: 1) void* vpMemCtx
//              : pointer to a valid memory comonent context
//
//      ARG: 2) char* cpInput
//              : file name of the list of SDP announcments
//
//   RETURN: void
//
//////////////////////////////////////////////////////////////////////////
void  vTestStatistics(void* vpMemCtx, char* cpInput)
{
  void*     vpSDPCtx = NULL;
  void*     vpPPTCtx = NULL;
  APG_PARSER_STATE sState;
  APG_PARSER_STATS sStats;
  APG_PARSER_STATS sPPTStats;
  APG_PARSER_STATS sTotalStats;
  APG_PARSER_STATS sTotalPPTStats;
  APG_BSTR  sSrc;
  APG_BSTR* spBstr;
  void*     vpBstr;
  ulong     ulTest;
  ulong     ulIndex;
  ulong     ulSize;
  ulong     ulCount;
  ulong     ulAnn;
  ulong     ulNodeVists;
  ulong     ulPPTVisits;
  ulong     ulTotalVisits;
  double    dFactor;
  
  // get an input string
  sSrc.ulLen = ulGetFileSize(cpInput);
  SDPASSERT(sSrc.ulLen);
  sSrc.ucpStr = (uchar*)vpMemAlloc(vpMemCtx, sSrc.ulLen + 10);
  SDPASSERT(sSrc.ucpStr);
  sSrc.ulLen = ulGetFile(cpInput, (void*)sSrc.ucpStr);
  SDPASSERT(sSrc.ulLen);
  sSrc.ucpStr[sSrc.ulLen] = 0;
  
  // get the list of announcements
  vpBstr = vpVecCtor(vpMemCtx, sizeof(APG_BSTR), 0);
  SDPASSERT(vpBstr);
  ulCount = ulReadAnnouncements(&sSrc, vpBstr);
  SDPASSERT(ulCount);
  
  // get a parser components
  vpSDPCtx = vpParserCtor(vpSDPInit, vpMemCtx);
  SDPASSERT(vpSDPCtx);
  vpPPTCtx = vpParserCtor(vpSDPPPTInit, vpMemCtx);
  SDPASSERT(vpPPTCtx);
  
  //////////////////////////////////////////////////////////////////////////
  // SDP test
  //////////////////////////////////////////////////////////////////////////
  memset((void*)&sTotalStats, 0, sizeof(sTotalStats));
  memset((void*)&sTotalPPTStats, 0, sizeof(sTotalPPTStats));
  ulSize = sizeof(sTotalPPTStats)/sizeof(ulong);
  spBstr = (APG_BSTR*)vpVecFront(vpBstr);
  SDPASSERT(spBstr);
  for(ulAnn = 0; ulAnn < ulCount; ++ulAnn, ++spBstr)
  {
    ulTest = ulParserSyntaxAnalysis(vpSDPCtx, 0, spBstr->ucpStr, spBstr->ulLen, NULL, APG_FALSE);
    SDPASSERT(ulTest);
    ulTest = ulParserState(vpSDPCtx, &sState);
    SDPASSERT(ulTest);
    if(sState.ulState == MATCH)
    {
      ulTest = ulParserStats(vpSDPCtx, &sStats);
      SDPASSERT(ulTest);
      for(ulIndex = 0; ulIndex < ulSize; ++ulIndex)
      {
        ulong* ulpState = (ulong*)&sStats;
        ulong* ulpTotal = (ulong*)&sTotalStats;
        ulpTotal[ulIndex] += ulpState[ulIndex];
      }
    }
  }

  // print the total stats
  vPrintStatistics(&sTotalStats);

  //////////////////////////////////////////////////////////////////////////
  // PPT test
  //////////////////////////////////////////////////////////////////////////
  spBstr = (APG_BSTR*)vpVecFront(vpBstr);
  SDPASSERT(spBstr);
  for(ulAnn = 0; ulAnn < ulCount; ++ulAnn, ++spBstr)
  {
    ulTest = ulParserSyntaxAnalysis(vpPPTCtx, 0, spBstr->ucpStr, spBstr->ulLen, NULL, APG_FALSE);
    SDPASSERT(ulTest);
    ulTest = ulParserState(vpPPTCtx, &sState);
    SDPASSERT(ulTest);
    if(sState.ulState == MATCH)
    {
      ulTest = ulParserStats(vpPPTCtx, &sPPTStats);
      SDPASSERT(ulTest);
      for(ulIndex = 0; ulIndex < ulSize; ++ulIndex)
      {
        ulong* ulpState = (ulong*)&sPPTStats;
        ulong* ulpTotal = (ulong*)&sTotalPPTStats;
        ulpTotal[ulIndex] += ulpState[ulIndex];
      }
    }
  }

  // print the total stats
  vPrintStatistics(&sTotalPPTStats);
  
  printf("\n");
  printf("    node visits, PPT tests, total\n");
  ulNodeVists = sTotalStats.sAlt.ulVisits +
    sTotalStats.sCat.ulVisits +
    sTotalStats.sRnm.ulVisits +
    sTotalStats.sRep.ulVisits +
    sTotalStats.sPrd.ulVisits +
    sTotalStats.sTrg.ulVisits +
    sTotalStats.sTbs.ulVisits +
    sTotalStats.sTls.ulVisits;
  ulPPTVisits = sTotalStats.sPpt.ulVisits;
  ulTotalVisits = ulNodeVists + ulPPTVisits;
  printf("SDP %11lu, %9lu, %5lu\n", ulNodeVists, ulPPTVisits, ulTotalVisits);
  dFactor = (double)ulTotalVisits;
  ulNodeVists = sTotalPPTStats.sAlt.ulVisits +
    sTotalPPTStats.sCat.ulVisits +
    sTotalPPTStats.sRnm.ulVisits +
    sTotalPPTStats.sRep.ulVisits +
    sTotalPPTStats.sPrd.ulVisits +
    sTotalPPTStats.sTrg.ulVisits +
    sTotalPPTStats.sTbs.ulVisits +
    sTotalPPTStats.sTls.ulVisits;
  ulPPTVisits = sTotalPPTStats.sPpt.ulVisits;
  ulTotalVisits = ulNodeVists + ulPPTVisits;
  printf("PPT %11lu, %9lu, %5lu\n", ulNodeVists, ulPPTVisits, ulTotalVisits);

  dFactor = dFactor / (double)ulTotalVisits;
  printf("\n");
  printf("reduction factor = total(SDP)/total(PPT)\n");
  printf("reduction factor = %1.4g\n", dFactor);

  // clean up
  if(vpSDPCtx){vParserDtor(vpSDPCtx);}
  if(vpPPTCtx){vParserDtor(vpPPTCtx);}
}

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: vPrintStatistics() helper for individual node types.
//
// FUNCTION: static void vPrintOperatorLine
//
//      ARG: 1) char* cpOp
//              : name of the node type (RNM, ALT, etc.)
//
//      ARG: 2) APG_NODE_STATS* spNode
//              : pointer to the statistics for this node
//
//      ARG: 3) APG_NODE_STATS* spTotal
//              : statistics struct to collect node totals in
//
//   RETURN: void
//
//////////////////////////////////////////////////////////////////////////
static void vPrintOperatorLine(char* cpOp, APG_NODE_STATS* spNode, APG_NODE_STATS* spTotal)
{
  DASSERT(cpOp && spNode);
  
  if(spTotal)
  {
    spTotal->ulVisits += spNode->ulVisits;
    spTotal->ulEmpty += spNode->ulEmpty;
    spTotal->ulMatched += spNode->ulMatched;
    spTotal->ulNotMatched += spNode->ulNotMatched;
  }
  printf("%5s %7lu %7lu %7lu %7lu",
    cpOp,
    spNode->ulVisits,
    spNode->ulMatched,
    spNode->ulEmpty,
    spNode->ulNotMatched);
}

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: 
// print a parser's statistics
//
// FUNCTION: static void vPrintStatistics
//
//      ARG: 1) APG_PARSER_STATS* spStats
//              : pointer to a parser statistics struct
//
//   RETURN: void
//
//////////////////////////////////////////////////////////////////////////
static void vPrintStatistics(APG_PARSER_STATS* spStats)
{
  ulong           ulTemp;
  APG_NODE_STATS* spNode;
  APG_NODE_STATS  sNodeTotal;
  DASSERT(spStats);
  
  printf("\n");
  printf("*************************\n");
  printf("*** Parser Statistics ***\n");
  printf("*************************\n");
  printf("\n");
  printf("BACK TRACKING\n");
  printf("back track occurrences = %lu\n", spStats->ulBackTrackCount);
  printf("back tracked characters= %lu\n", spStats->ulBackTrackChars);
  
  printf("\n");
  printf("OPERATORS TESTED\n");
  printf("%5s %7s %7s %7s %7s %7s\n", "", "total", "MATCH", "EMPTY", "NOMATCH", "CHILDREN");
  memset((void*)&sNodeTotal, 0, sizeof(sNodeTotal));
  vPrintOperatorLine("RNM", &spStats->sRnm, &sNodeTotal);
  printf("\n");
  vPrintOperatorLine("ALT", &spStats->sAlt, &sNodeTotal);
  printf(" %7lu\n", spStats->sAlt.ulChildCount);
  vPrintOperatorLine("CAT", &spStats->sCat, &sNodeTotal);
  printf(" %7lu\n", spStats->sCat.ulChildCount);
  vPrintOperatorLine("REP", &spStats->sRep, &sNodeTotal);
  printf(" %7lu\n", spStats->sRep.ulChildCount);
  vPrintOperatorLine("PRD", &spStats->sPrd, &sNodeTotal);
  printf("\n");
  vPrintOperatorLine("TRG", &spStats->sTrg, &sNodeTotal);
  printf("\n");
  vPrintOperatorLine("TBS", &spStats->sTbs, &sNodeTotal);
  printf("\n");
  vPrintOperatorLine("TLS", &spStats->sTls, &sNodeTotal);
  printf("\n");
  vPrintOperatorLine("TOTAL", &sNodeTotal, NULL);
  ulTemp = spStats->sAlt.ulChildCount + spStats->sCat.ulChildCount + spStats->sRep.ulChildCount;
  printf(" %7lu\n", ulTemp);
  
  printf("\n");
  printf("PREDICTIVE PARSING TABLES TESTED\n");
  printf("%5s %7s %7s %7s %7s %7s\n", "", "total", "MATCH", "EMPTY", "NOMATCH", "ACTIVE");
  spNode = &spStats->sPpt;
  printf("%5s %7lu %7lu %7lu %7lu %7lu\n",
    "PPT",
    spNode->ulVisits,
    spNode->ulMatched,
    spNode->ulEmpty,
    spNode->ulNotMatched,
    spNode->ulPossibleMatched);
  
  printf("\n");
  printf("EOF CHARACTERS TESTED\n");
  printf("%5s %7s %7s\n", "", "total", "MATCH");
  spNode = &spStats->sEof;
  printf("%5s %7lu %7lu\n",
    "EOF",
    spNode->ulVisits,
    spNode->ulMatched);
}

