/********************************************************************
  APG - an ABNF Parser Generator
  Copyright  (C) 2007 Coast to Coast Research, Inc.
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program.  If not, see
  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
  or write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

    author: Lowell Thomas
            lowell@coasttocoastresearch.com
            http://www.coasttocoastresearch.com

   created: 8/27/2007 10:18
  filename: D:\Projects\Apgv5_0\Samples\SDP\Uri.c
   purpose: Prints uri field information from a Parser Catalog.
  
*********************************************************************/
#include "main.h"

static void vPrintComponent(APG_BSTR* spSrc, char* cpName, char* cpIndent);

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: URI test driver
//
// FUNCTION: void ulUriTest
//
//      ARG: 1) APG_BSTR* spSrc
//              : binary string defining a SDP announcement
//
//      ARG: 2) void* vpMemCtx
//              : pointer to a valid memory component context
//
//      ARG: 3) void* vpParserCtx
//              : pointer to a valid parser component context
//
//   RETURN: true on success
//
//////////////////////////////////////////////////////////////////////////
ulong ulUriTest(APG_BSTR* spSrc, void* vpMemCtx, void* vpParserCtx)
{
  ulong   ulReturn = APG_FALSE;
  APG_PARSER_STATE sState;
  URI_DATA sUriData;
  ulong ulTest;
  ulong ulPhraseCount;
  void* vpCatalogCtx = NULL;
  void* vpUriCtx = NULL;
  char    caBuf[SDPURI_BUFFER_SIZE + 1];
  APG_CAT_ITERATOR sPhraseIter;
  APG_CAT_ITERATOR* spPhraseIter;

  //////////////////////////////////////////////////////////////////////////
  // full parse of the announcement
  //////////////////////////////////////////////////////////////////////////
  ulTest = ulParserSyntaxAnalysis(vpParserCtx, RULE_SDP_ANNOUNCEMENT,
    spSrc->ucpStr, spSrc->ulLen, NULL, APG_FALSE);
  SDPASSERT(ulTest);
  ulTest = ulParserState(vpParserCtx, &sState);
  SDPASSERT(ulTest);
  if(sState.ulState != MATCH){goto APG_EXIT;}

  //////////////////////////////////////////////////////////////////////////
  // horizontal discovery of uri
  //////////////////////////////////////////////////////////////////////////
  vpCatalogCtx = vpCatalogCtor(vpParserCtx);
  SDPASSERT(vpCatalogCtx);
  ulPhraseCount = ulCatalog(vpCatalogCtx);
  SDPASSERT(ulPhraseCount);
  spPhraseIter = spCatalogGetPhraseList(vpCatalogCtx, RULE_SDP_URI_FIELD, &sPhraseIter);
  SDPASSERT(spPhraseIter);
  if(!spPhraseIter){goto APG_EXIT;}

  // make sure it came from the session-section (and not the key-field)
  if(spPhraseIter->ulParentID != RULE_SDP_SESSION_SECTION){goto APG_EXIT;}

  //////////////////////////////////////////////////////////////////////////
  // full parse of uri
  //////////////////////////////////////////////////////////////////////////
//   ulTest = ulCatalogGetString(&spPhraseIter->sPhrase, caBuf);
//   SDPASSERT(ulTest);
//   printf("\n");
//   printf("URI: - pre-parsed\n");
//   printf("%s\n", caBuf);
  vpUriCtx = vpParserCtor(vpSDPURIInit, vpMemCtx);
  SDPASSERT(vpUriCtx);
  ulTest = ulParserSyntaxAnalysis(vpUriCtx, RULE_SDPURI_URI_REFERENCE,
    spPhraseIter->sPhrase.ucpStr, spPhraseIter->sPhrase.ulLen, NULL, APG_FALSE);
  SDPASSERT(ulTest);
  ulTest = ulParserState(vpUriCtx, &sState);
  SDPASSERT(ulTest);
  if(sState.ulState != MATCH){goto APG_EXIT;}

  //////////////////////////////////////////////////////////////////////////
  // vertical discovery of "authority"
  //////////////////////////////////////////////////////////////////////////
  sUriData.sSrc = spPhraseIter->sPhrase;
  ulTest = ulParserSemanticAnalysis(vpUriCtx, &sUriData);
  SDPASSERT(ulTest);
  SDPASSERT(sUriData.ulReturn);

  //////////////////////////////////////////////////////////////////////////
  // display of authority components
  // assumes all string lengths <= SDPURI_BUFFER_SIZE - no testing
  //////////////////////////////////////////////////////////////////////////

  // print the full uri
  ulTest = ulCatalogGetString(&sUriData.sSrc, caBuf);
  SDPASSERT(ulTest);
  if(sUriData.ulURI)
  {
    vPrintComponent(&sUriData.sSrc, "URI-reference: URI", "");
  }
  else
  {
    vPrintComponent(&sUriData.sSrc, "URI-reference: relative-ref", "");
  }

  // scheme
  vPrintComponent(&sUriData.sScheme, "scheme", "  ");
  
  // authority
  vPrintComponent(&sUriData.sAuthority, "authority", "  ");
  if(sUriData.sAuthority.ulLen)
  {
    vPrintComponent(&sUriData.sUserInfo, "userinfo", "    ");

    if(sUriData.sIPvFuture.ulLen)
    {
      vPrintComponent(&sUriData.sHost, "host: IPvFuture", "    ");
    }

    if(sUriData.sIPv4Address.ulLen)
    {
      vPrintComponent(&sUriData.sHost, "host: IPv4Address", "    ");
    }

    if(sUriData.sIPv6Address.ulLen)
    {
      vPrintComponent(&sUriData.sHost, "host: IPv6Address", "    ");
    }

    if(sUriData.sRegName.ulLen)
    {
      vPrintComponent(&sUriData.sHost, "host: reg-name", "    ");
    }

    vPrintComponent(&sUriData.sPort, "port", "    ");
  }
  
  // path
  vPrintComponent(&sUriData.sPath, "path", "  ");
  
  // query
  vPrintComponent(&sUriData.sQuery, "query", "  ");
  
  // fragment
  vPrintComponent(&sUriData.sFragment, "fragment", "  ");
  
  // success
  ulReturn = APG_TRUE;

APG_EXIT:
  if(vpCatalogCtx){vCatalogDtor(vpCatalogCtx);}
  if(vpUriCtx){vParserDtor(vpUriCtx);}
  return ulReturn;
}

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: Print a single URI component
//              (scheme, authority, etc.)
//
// FUNCTION: void vPrintComponent
//
//      ARG: 1) APG_BSTR* spSrc
//              : binary string defining a URI component
//
//      ARG: 2) char* cpName
//              : name of the component
//
//      ARG: 3) char* cpIndent
//              : a prefix to the component
//                meant to be spaces which will indent the display
//
//   RETURN: void
//
//////////////////////////////////////////////////////////////////////////
static void vPrintComponent(APG_BSTR* spSrc, char* cpName, char* cpIndent)
{
  char caBuf[SDPURI_BUFFER_SIZE + 1];
  ulong ulTest;
  DASSERT(spSrc);
  DASSERT(cpName);

  printf("\n");
  printf("%s%s:\n", cpIndent, cpName);
  if(spSrc->ulLen)
  {
    ulTest = ulCatalogGetString(spSrc, caBuf);
    SDPASSERT(ulTest);
    printf("%s%s\n", cpIndent, caBuf);
  }
  else
  {
    printf("%s<none>\n", cpIndent);
  }

}
