/********************************************************************
  APG - an ABNF Parser Generator
  Copyright  (C) 2007 Coast to Coast Research, Inc.
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program.  If not, see
  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
  or write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

    author: Lowell Thomas
            lowell@coasttocoastresearch.com
            http://www.coasttocoastresearch.com

   created: 8/27/2007 10:15
  filename: D:\Projects\Apgv5_0\Samples\SDP\main.c
   purpose: Driver (main) program.
  
*********************************************************************/
#include "main.h"

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: The driver (main) program.
//
// FUNCTION: int main
//
//      ARG: 1) int argc
//              : number of command line arguments
//
//      ARG: 2) char** argv
//              : pointer to a list of pointers to the command line arguments
//
//   RETURN: 0
//
//////////////////////////////////////////////////////////////////////////
int main(int argc, char** argv)
{
  void* vpMemCtx;
  ulong ulTest;
  FILE* spRedirectOut = NULL;

  // default is to run all tests
  ulong ulTest1 = APG_TRUE;
  ulong ulTest2 = APG_TRUE;
  ulong ulTest3 = APG_TRUE;
  ulong ulTest4 = APG_TRUE;
  ulong ulTest5 = APG_TRUE;
  ulong ulTest6 = APG_TRUE;
  ulong ulTest7 = APG_TRUE;
  ulong ulTest8 = APG_TRUE;

  // set the custom alert handler
  pfnSetAlertHandler(vTerminalAlert);
  
  // redirect output
  printf("%s - output redirect file\n", "main.log");
  spRedirectOut = freopen("main.log", "w", stdout);
  SDPASSERT(spRedirectOut);

  // GPL and copyright notice
  vLicenseNotice();

  // check the command line
  if(argc > 1)
  {
    ulong ulArg = (ulong)atol(argv[1]);
    
    // run only the test specified on the command line
    ulTest1 = APG_FALSE;
    ulTest2 = APG_FALSE;
    ulTest3 = APG_FALSE;
    ulTest4 = APG_FALSE;
    ulTest5 = APG_FALSE;
    ulTest6 = APG_FALSE;
    ulTest7 = APG_FALSE;
    ulTest8 = APG_FALSE;
    switch(ulArg)
    {
    case 1:
      ulTest1 = APG_TRUE;
      break;
    case 2:
      ulTest2 = APG_TRUE;
      break;
    case 3:
      ulTest3 = APG_TRUE;
      break;
    case 4:
      ulTest4 = APG_TRUE;
      break;
    case 5:
      ulTest5 = APG_TRUE;
      break;
    case 6:
      ulTest6 = APG_TRUE;
      break;
    case 7:
      ulTest7 = APG_TRUE;
      break;
    case 8:
      ulTest8 = APG_TRUE;
      break;
    default:
      printf("unrecognized input - %s\n", argv[1]);
      SDPASSERT(APG_FALSE);
      break;
    }
  }
  
  // get memory and error contexts
  vpMemCtx = vpMemCtor();
  SDPASSERT(vpMemCtx);

  // initialize all parsers that might be used in any test
  ulTest = ulParserInit(vpSDPInit, vpMemCtx);
  SDPASSERT(ulTest);
  ulTest = ulParserInit(vpSDPPPTInit, vpMemCtx);
  SDPASSERT(ulTest);
  ulTest = ulParserInit(vpSDPURIInit, vpMemCtx);
  SDPASSERT(ulTest);

  if(ulTest1)
  {
    printf("\n");
    printf("**************************************\n");
    printf("*** Test well-formed announcements ***\n");
    printf("**************************************\n");
    vTestComponents(vpMemCtx, "Announcements.txt", ulAccuracyTest);
    
    printf("\n");
    printf("**************************************\n");
    printf("*** Test malformed announcements   ***\n");
    printf("**************************************\n");
    vTestComponents(vpMemCtx, "Errors.txt", ulAccuracyTest);
  }

  if(ulTest2)
  {
    printf("\n");
    printf("****************************\n");
    printf("*** Print Session Fields ***\n");
    printf("****************************\n");
    vTestComponents(vpMemCtx, "Components.txt", ulSessionTest);
  }
  
  if(ulTest3)
  {
    printf("\n");
    printf("**************************\n");
    printf("*** Print Origin Field ***\n");
    printf("**************************\n");
    vTestComponents(vpMemCtx, "Components.txt", ulOriginTest);
  }
  
  if(ulTest4)
  {
    printf("\n");
    printf("**************************\n");
    printf("*** Print Media Fields ***\n");
    printf("**************************\n");
    vTestComponents(vpMemCtx, "Components.txt", ulMediaTest);
  }
  
  if(ulTest5)
  {
    printf("\n");
    printf("*****************************************\n");
    printf("*** Print Connection Field IP Address ***\n");
    printf("*****************************************\n");
    vTestComponents(vpMemCtx, "Components.txt", ulConnectionTest);
  }
  
  if(ulTest6)
  {
    printf("\n");
    printf("***********************\n");
    printf("*** Print Uri Field ***\n");
    printf("***********************\n");
    vTestComponents(vpMemCtx, "Uris.txt", ulUriTest);
  }

  if(ulTest7)
  {
    printf("\n");
    printf("*************************\n");
    printf("*** Print Timing Test ***\n");
    printf("*************************\n");
    vTestTiming(vpMemCtx, "Announcements.txt", 5000);
  }
  
  if(ulTest8)
  {
    printf("\n");
    printf("********************************\n");
    printf("*** Print Statistics Test    ***\n");
    printf("*** (debug version required) ***\n");
    printf("*********************************\n");
    vTestStatistics(vpMemCtx, "Announcements.txt");
  }
  
  // clean up resources
  if(vpMemCtx)
  {
    vPrintMemStats(vpMemCtx);
    vMemDtor(vpMemCtx);
  }
  if(spRedirectOut){fclose(spRedirectOut);}

  return 0;
}
