/********************************************************************
  APG - an ABNF Parser Generator
  Copyright  (C) 2007 Coast to Coast Research, Inc.
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program.  If not, see
  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
  or write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

    author: Lowell Thomas
            lowell@coasttocoastresearch.com
            http://www.coasttocoastresearch.com

   created: 8/28/2007 9:35
  filename: D:\Projects\Apgv5_0\Samples\SIP\Messages.c
   purpose: Separate a file of semicolon-separated SIP messages
                into a vector of BStrings.
  
*********************************************************************/
#include "main.h"

static char* cpLookForStart(char* cpBeg, char* cpEnd);
static char* cpLookForEnd(char* cpBeg, char* cpEnd);

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: Separate a file of semicolon-separated SIP messages
//              into a vector of BStrings.
//
// FUNCTION: ulong ulMessages
//
//      ARG: 1) char* cpFile
//              : pointer to the list of SIP messages
//
//      ARG: 2) ulong ulLen
//              : number of characters in the list of messages
//
//      ARG: 3) void* vpBstr
//              : pointer to a valid vector of binary strings context
//
//   RETURN: number of messages in the list.
//
//////////////////////////////////////////////////////////////////////////
ulong ulMessages(char* cpFile, ulong ulLen, void* vpBstr)
{
  ulong ulReturn;
  ulong ulCount;
  ulong ulIndex;
  APG_BSTR* spMsg;
  char* cpStart;
  char* cpEnd;
  char* cpNext;
  char* cpEndFile;
  DASSERT(cpFile);
  DASSERT(ulLen);
  DASSERT(vpBstr);

  cpNext = cpFile;
  cpEndFile = cpNext + ulLen;
  // single-point-of-exit loop
  while(APG_TRUE)
  {
    cpStart = cpLookForStart(cpNext, cpEndFile);
    if(!cpStart){break;}
  
    cpEnd = cpLookForEnd(cpStart, cpEndFile);
    if(!cpEnd){break;}

    // push a message on the vector
    spMsg = (APG_BSTR*)vpVecPush(vpBstr, NULL);
    if(!cpEnd){goto APG_EXIT;}
    spMsg->ucpStr = (uchar*)cpStart;
    spMsg->ulLen = (ulong)(cpEnd - cpStart);

    cpNext = cpEnd;
  } // end of single-point-of-exit loop

  // null-term each message
  // - OK since this character must be semicolon
  // - OK because this is a temporary copy of the file
  ulCount = ulVecSize(vpBstr);
  SIPASSERT(ulCount);
  spMsg = (APG_BSTR*)vpVecFront(vpBstr);
  SIPASSERT(spMsg);
  for(ulIndex = 0; ulIndex < ulCount; ++ulIndex, ++spMsg)
  {
    cpEnd = (char*)spMsg->ucpStr + spMsg->ulLen;
    *cpEnd = 0;
  }

  // success
  ulReturn = ulCount;

APG_EXIT:
  return ulReturn;
}

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: Search for the beginning of a message
//
// FUNCTION: static char* cpLookForStart
//
//      ARG: 1) char* cpBeg
//              : pointer to the text to begin searching
//
//      ARG: 2) char* cpEnd
//              : pointer to the end of the text to search
//
//   RETURN: on success, pointer to the beginning of the message
//           on failure, NULL
//
//////////////////////////////////////////////////////////////////////////
static char* cpLookForStart(char* cpBeg, char* cpEnd)
{
  char* cpReturn = NULL;
  char cPrev = LF;
  char* cpNext = cpBeg;

  for(; cpNext < cpEnd; ++cpNext)
  {
    if(cPrev == LF && *cpNext != SEMI)
    {
      // found begin of msg
      break;
    }

    cPrev = *cpNext;
  }

  if(cpNext < cpEnd)
  {
    // found a msg
    cpReturn = cpNext;
  }
  // else found end of text

  return cpReturn;
}

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: Look for the end of a message.
//
// FUNCTION: static char* cpLookForEnd
//
//      ARG: 1) char* cpBeg
//              : pointer to a message
//
//      ARG: 2) char* cpEnd
//              : output, pointer to the end of the list of message
//
//   RETURN: on success, pointer to the end of the message
//           on failure, NULL
//
//////////////////////////////////////////////////////////////////////////
static char* cpLookForEnd(char* cpBeg, char* cpEnd)
{
  char* cpReturn = NULL;
  char cPrev = LF;
  char* cpNext = cpBeg;
  
  for(; cpNext < cpEnd; ++cpNext)
  {
    if(cPrev == LF && *cpNext == SEMI)
    {
      // found end of msg
      break;
    }
    
    cPrev = *cpNext;
  }
  
  if(cpBeg < cpNext)
  {
    // found a msg
    cpReturn = cpNext;
  }
  // else found end of text
  
  return cpReturn;
}
