/********************************************************************
  APG - an ABNF Parser Generator
  Copyright  (C) 2007 Coast to Coast Research, Inc.
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program.  If not, see
  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
  or write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

    author: Lowell Thomas
            lowell@coasttocoastresearch.com
            http://www.coasttocoastresearch.com

   created: 8/28/2007 9:38
  filename: D:\Projects\Apgv5_0\Samples\SIP\Utilities.c
   purpose: Collection of utility functions that may be used by
                more than one component of SIP application.
  
*********************************************************************/
#include "main.h"

static void vPrintOneLevel(void* vpCatalogCtx, APG_CAT_ITERATOR* spLevel, ulong ulIn);

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: 
// prints all children of each occurrance of a phrase
//
// FUNCTION: ulong ulPrintChildPhrases
//
//      ARG: 1) void* vpCatalogCtx
//              : pointer to a valid catalog component context
//
//      ARG: 2) ulong ulStartRule
//              : rule name index for the phrase
//
//   RETURN: true on success
//
//////////////////////////////////////////////////////////////////////////
ulong ulPrintChildPhrases(void* vpCatalogCtx, ulong ulStartRule)
{
  ulong   ulReturn = APG_FALSE;
  ulong   ulTest;
  char    caBuf[BUFSIZE];
  
  // these are the permanent place-holders for the iterators
  APG_CAT_ITERATOR sStartRuleIter;
  
  // these will either point to one of the above iterators or be NULL
  APG_CAT_ITERATOR* spStartRule;
  
  // get the list of start rule children
  spStartRule = spCatalogGetPhraseList(vpCatalogCtx, ulStartRule, &sStartRuleIter);
  SIPASSERT(spStartRule);
  for(; spStartRule != NULL; spStartRule = spCatalogIterNext(vpCatalogCtx, spStartRule))
  {
    // iterate over 1st level children
    SIPASSERT(spStartRule->sPhrase.ulLen < BUFSIZE);
    ulTest = ulCatalogGetString(&spStartRule->sPhrase, caBuf);
    if(ulTest)
    {
      vPrintableBuf((uchar*)caBuf, ulTest);
      printf("[%s]: ", spStartRule->cpPhraseName);
      printf("%s\n", caBuf);
      vPrintOneLevel(vpCatalogCtx, spStartRule, 2);
    }
  }
  
  // success
  ulReturn = APG_TRUE;
  
APG_EXIT:
  return ulReturn;
}

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: Converts all non-printing characters in a string to
//              printing (":") characters.
//
// FUNCTION: void vPrintableBuf
//
//      ARG: 1) uchar* ucpBuf
//              : pointer to the string
//
//      ARG: 2) ulong ulLen
//              : length of the string
//
//   RETURN: void
//
//////////////////////////////////////////////////////////////////////////
void vPrintableBuf(uchar* ucpBuf, ulong ulLen)
{
  if(ulLen)
  {
    // check for non-printing characters
    {
      ulong ul;
      for(ul = 0; ul < ulLen; ++ul)
      {
        if(ucpBuf[ul] < 32 || ucpBuf[ul] > 127)
        {
          if(!(ucpBuf[ul] == CR || ucpBuf[ul] == LF))
          {
            ucpBuf[ul] = '.';
          }
        }
      }
    }
  }
  else
  {
    memcpy((void*)ucpBuf, (void*)"<none>", 7);
  }
}

//////////////////////////////////////////////////////////////////////////
// STATIC FUNCTIONS
//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: 
// recursively prints all children of the given "spLevel" phrases
//
// FUNCTION: static void vPrintOneLevel
//
//      ARG: 1) void* vpCatalogCtx
//              : pointer to a valid catalog component context
//
//      ARG: 2) APG_CAT_ITERATOR* spLevel
//              : pointer to a catalog iterator
//
//      ARG: 3) ulong ulIn
//              : number of characters to indent the print lines.
//
//   RETURN: void
//
//////////////////////////////////////////////////////////////////////////
static void vPrintOneLevel(void* vpCatalogCtx, APG_CAT_ITERATOR* spLevel, ulong ulIn)
{
  APG_CAT_ITERATOR  sNextLevel;
  APG_CAT_ITERATOR* spNextLevel;
  ulong ulTest;
  char    caBuf[BUFSIZE];
  
  spNextLevel = spCatalogGetChildList(vpCatalogCtx, spLevel, &sNextLevel);
  for(; spNextLevel != NULL; spNextLevel = spCatalogIterNext(vpCatalogCtx, spNextLevel))
  {
    // iterate over 2nd level children
    SIPASSERT(spNextLevel->sPhrase.ulLen < BUFSIZE);
    ulTest = ulCatalogGetString(&spNextLevel->sPhrase, caBuf);
    if(ulTest)
    {
      vPrintableBuf((uchar*)caBuf, ulTest);
      vIndent(ulIn);
      printf("[%s]: ", spNextLevel->cpPhraseName);
      printf("%s\n", caBuf);
      vPrintOneLevel(vpCatalogCtx, spNextLevel, ulIn + 2);
    }
  }
APG_EXIT:
  return;
}

