/********************************************************************
  APG - an ABNF Parser Generator
  Copyright  (C) 2007 Coast to Coast Research, Inc.
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program.  If not, see
  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
  or write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

    author: Lowell Thomas
            lowell@coasttocoastresearch.com
            http://www.coasttocoastresearch.com

   created: 8/27/2007 9:02
  filename: D:\Projects\Apgv5_0\Samples\URI\ReadUris.c
  purpose: Reads a list of IPv6 addresses and generates a vector
  of Binary Strings, one for each address.
  Addresses may be separated with blank lines or
  comment lines. Comment lines must begin with ";" as
  the first character. No comments allowed other
  than those beginning with the first character.
  
*********************************************************************/
#include "main.h"

#define CR 13
#define LF 10
//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: Read a file of URI examples, one line per URI,
//              separated by comment lines beginning with ";".
// one text IPv6 address per line
// each line must end with CRLF
// no leading or trailing white space allowed
// no blank lines or comments
//
// FUNCTION: ulong ulReadUris
//
//      ARG: 1) APG_BSTR* spSrc
//              : binary string pointing to the list of URIs
//
//      ARG: 2) void* vpBstr
//              : pointer to a vector of binary strings.
//                on output contains one binary string per URI
//
//   RETURN: number of URIs in the vector
//
//////////////////////////////////////////////////////////////////////////
ulong ulReadUris(APG_BSTR* spSrc, void* vpBstr)
{
  static  char  caLineEnd[3] = {CR, LF, 0};
  ulong ulCount = 0;
  ulong ulLen;
  char* cpBegin;
  char* cpEnd;
  APG_BSTR* spBstr;
  
  // single-point-of-exit loop
  cpBegin = (char*)spSrc->ucpStr;
  while(APG_TRUE)
  {
    // find the end of this line
    cpEnd = strstr(cpBegin, caLineEnd);
    if(cpEnd == NULL){break;} // normal end of list
    
    // null term the string
    cpEnd[0] = 0;
    cpEnd[1] = 0;
    
    ulLen = (ulong)strlen(cpBegin);
    if(ulLen && *cpBegin != ';')
    {
      // push a string on the vector
      spBstr = (APG_BSTR*)vpVecPush(vpBstr, NULL);
      spBstr->ulLen = ulLen;
      spBstr->ucpStr = (uchar*)cpBegin;
      ++ulCount;
    }
    // else ignore blanks and comments beginning with ;
    
    // update for next line
    cpBegin = cpEnd + 2;
  } // end of single-point-of-exit loop
  
  return ulCount;
}

