/*! \file Agent.h
 <pre>
 <b>File:</b> Agent.h
 <b>Project:</b> Robocup 3D
 <b>Authors:</b> Hviezdna Jedenastka
 <b>Created:</b> 03/10/2008
 <b>Contents:</b> Implementation of main agent class.
 </pre>
 */

#ifndef __AGENT_H__
#define __AGENT_H__

#include "Config.h"
#include "Communication.h"
#include "Parser.h"
#include "Skills.h"
#include "WorldModel.h"
#include "PlayerModel.h"

/*!
 * Main agent class
 */
class Agent {
    
protected:
	Parser			* parser;					//!< Parser module
	WorldModel		* world;					//!< WorldModel module
	PlayerModel		* player;					//!< PlayerModel module
    
	LowSkill	* actualLowSkill;
	MiddleSkill * actualMiddleSkill;
	HighSkill	* actualHighSkill;
	
	bool running;								//!< agent main loop conndition
	char message[NET_BUFFER_SIZE+1];			//!< Buffer for incomming and outgoing message
	
	int behave();								//!< agent thinking
	
public:
	Agent();
	~Agent();
    
	int runMultiThreaded();						//!< runs agent in multithreaded mode
	int runSingleThreaded();					//!< runs agent in single thread
	int stop();									//!< stops agent
	
	void testSkill(char * skillName);			//!< test skill
};

#endif /*__AGENT_H__*/

