/*! \file LSBasics.h
 <pre>
 <b>File:</b> LSBasics.h
 <b>Project:</b> Robocup 3D
 <b>Authors:</b> Hviezdna Jedenastka
 <b>Created:</b> 04/13/2008
 <b>Contents:</b> Implementation of some basic skills
 </pre>
 */ 

#include "Skills.h"
#include "PlayerModel.h"
#include "WorldModel.h"


//this skill makes a knee-bend of agent, where the torso of agent stays in vertical plane
//and the torso are on the same vertical axis as the end of foot, 

class LSMoveDown: public LowSkill {
    
public:
	
	double ankle_angle;		//reccomended from 25 to 90 (90 - upright stand)
	double knee_angle;
	double hip_angle;
	double speed_ratio;		//higher speed ratio means higher speed of joints moving
	
	
	LSMoveDown(WorldModel * _world, PlayerModel * _player, double _ankle_angle, double _speed_ratio);
	LSMoveDown(WorldModel * _world, PlayerModel * _player, double _ankle_angle, double _speed_ratio, int _stepsToWait);
    
	int calculate(Skill **lowerSkill);
	
    int skillType() {
        
		return SKILL_LS_MOVEDOWN;
	}
};

//this skill returns agent from knee-bend to basic upright stand

class LSMoveUp: public LowSkill {
    
public:
    
	double speed_ratio;		//higher speed ratio means higher speed of joints moving
    
	LSMoveUp(WorldModel * _world, PlayerModel * _player, double _speed_ratio);
	
    int calculate(Skill **lowerSkill);
	
    int skillType() {
        
		return SKILL_LS_MOVEUP;
	}
};

class MSRotateToBall: public MiddleSkill {
    
public:
    
	double ankle_angle;
	double knee_angle;
	double hip_angle;
	double knee_angle_delta;
	double hip_angle_delta;
	double leg_angle_delta;
    
	double speed_ratio_startposition;
	double speed_ratio_steps;
    
	double timeout;
	bool clockwise;
	
	MSRotateToBall(WorldModel * _world, PlayerModel * _player);
    
	int calculate(Skill **lowerSkill);
	
    int skillType() {
        
		return SKILL_UNDEF;
	}
    
	bool stopRotating();
};

//jumping backwards:)))))
class MSJumpingBackwards: public MiddleSkill {
    
public:
	
	int last_ls;
	double speed_ratio;
	int steps_to_wait;      //number of steps to wait between skill proceeding
	int steps_wait_remaining;	 //actual number of steps remaining
	
	MSJumpingBackwards(WorldModel * _world, PlayerModel * _player);
    
	int calculate(Skill **lowerSkill);
    
	int skillType() {
        
		return SKILL_UNDEF;
	}
};

//this skill was temporarily created for testing purposes

class LsHandicappedWalk: public LowSkill {
    
public:
    
	//angles for starting position
	double ankle_angle;
	double knee_angle;
	double hip_angle;
    
	//delta for making steps
	double knee_angle_delta;
	double hip_angle_delta;
	double shoulder_angle_delta;
    
	//angles for step
	double ankle_angle_mov;
	double knee_angle_mov;
	double hip_angle_mov;
	double hip_angle_mov_delta;
    
	double angle_mov_delta;
	
	double distanceFloor_Hips; 
    
	double timeout;
	double speed_ratio_startposition;
	double speed_ratio_steps;
	double speed_ratio_shoulders;
	double speed_ratio_steps_movedown;
    
	LsHandicappedWalk(WorldModel * _world, PlayerModel * _player);
	
    int calculate(Skill **lowerSkill);
	
    int skillType() {
        
		return SKILL_UNDEF;
	}
};