/*! \file LSStandUp.cpp
 <pre>
 <b>File:</b> LSStandUp.cpp
 <b>Project:</b> Robocup 3D
 <b>Authors:</b> Hviezdna Jedenastka
 <b>Created:</b> 04/13/2008
 <b>Contents:</b> Standing skill
 </pre>
 */

#include "LSStandUp.h"

//temporary macros for tunning
#define LEG_DOWN 0.964
#define LEG_UP	 1.3
#define ANKLE	 75.0
#define BEND	 40.0
#define SPEED	 8.0
#define HIP		 (RAD2DEG(asin((LEG_DOWN*sin(DEG2RAD((ANKLE))))/(LEG_UP))))
#define KNEE	 (-(HIP)-(ANKLE)-15.0)

using namespace std;

LSStandUp::LSStandUp(WorldModel * _world, PlayerModel * _player):LowSkill(_world, _player) {
    
}

int LSStandUp::skillType() {
    
    return SKILL_LS_STANDUP;
}

int LSStandUp::calculate(Skill ** lowerSkill) {
    
	double timeout = 10.0;
	
	SKILL_BEGIN(0)

	//starting position
    SKILL_BEGIN_PHASE(1)
		SKILL_CALCULATE(ankleL->moveTo(90.0, 5.0, 0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(ankleR->moveTo(90.0, 5.0, 0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(kneeL->moveTo(0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(kneeR->moveTo(0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(legL->moveTo(0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(legR->moveTo(0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(hipL->moveTo(0.0, 5.0, 0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(hipR->moveTo(0.0, 5.0, 0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(armL->moveTo(0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(armR->moveTo(0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(elbowL->moveTo(0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(elbowR->moveTo(0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(shoulderL->moveTo(0.0, 5.0, 0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(shoulderR->moveTo(0.0, 5.0, 0.0, 5.0, false, APPROXIMATING, timeout))
	SKILL_END_PHASE
	
	//jump
    SKILL_BEGIN_PHASE(2)
		SKILL_CALCULATE(ankleL->moveTo(90.0, 5.0, 0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(ankleR->moveTo(90.0, 5.0, 0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(kneeL->moveTo(0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(kneeR->moveTo(0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(legL->moveTo(0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(legR->moveTo(0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(hipL->moveTo(0.0, 5.0, 0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(hipR->moveTo(0.0, 5.0, 0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(armL->moveTo(0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(armR->moveTo(0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(elbowL->moveTo(0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(elbowR->moveTo(0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(shoulderL->moveTo(90.0, 24.0, 0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(shoulderR->moveTo(90.0, 24.0, 0.0, 5.0, false, APPROXIMATING, timeout))
	SKILL_END_PHASE
	
	//landing
    SKILL_BEGIN_PHASE(3)
		SKILL_CALCULATE(ankleL->moveTo(-20.0, 6.0, 0.0, 1.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(ankleR->moveTo(-20.0, 6.0, 0.0, 1.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(kneeL->moveTo(0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(kneeR->moveTo(0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(legL->moveTo(0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(legR->moveTo(0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(hipL->moveTo(0.0, 5.0, 0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(hipR->moveTo(0.0, 5.0, 0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(armL->moveTo(0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(armR->moveTo(0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(elbowL->moveTo(0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(elbowR->moveTo(0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(shoulderL->moveTo(90.0, 5.0, 0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(shoulderR->moveTo(90.0, 5.0, 0.0, 5.0, false, APPROXIMATING, timeout))
	SKILL_END_PHASE
	
	//crouch
    SKILL_BEGIN_PHASE(4)
		SKILL_CALCULATE(ankleL->moveTo(ANKLE, SPEED, 0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(ankleR->moveTo(ANKLE, SPEED, 0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(kneeL->moveTo(KNEE, SPEED, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(kneeR->moveTo(KNEE, SPEED, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(legL->moveTo(0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(legR->moveTo(0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(hipL->moveTo(HIP + BEND, SPEED, 0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(hipR->moveTo(HIP + BEND, SPEED, 0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(armL->moveTo(0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(armR->moveTo(0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(elbowL->moveTo(0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(elbowR->moveTo(0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(shoulderL->moveTo(0.0, SPEED, 0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(shoulderR->moveTo(0.0, SPEED, 0.0, 5.0, false, APPROXIMATING, timeout))
	SKILL_END_PHASE
	
	//stabilizing
    SKILL_BEGIN_PHASE(5)
		SKILL_CALCULATE(ankleL->moveTo(ANKLE, SPEED, 0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(ankleR->moveTo(ANKLE, SPEED, 0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(kneeL->moveTo(KNEE, SPEED, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(kneeR->moveTo(KNEE, SPEED, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(legL->moveTo(0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(legR->moveTo(0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(hipL->moveTo(HIP + BEND, SPEED, 0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(hipR->moveTo(HIP + BEND, SPEED, 0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(armL->moveTo(0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(armR->moveTo(0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(elbowL->moveTo(0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(elbowR->moveTo(0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(shoulderL->moveTo(0.0, SPEED, 0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(shoulderR->moveTo(0.0, SPEED, 0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_WAIT(3.0)
	SKILL_END_PHASE

    SKILL_END(7)
}
