/*! \file UDPSocket.h
 <pre>
 <b>File:</b> UDPSocket.h
 <b>Project:</b> Robocup 3D
 <b>Authors:</b> Hviezdna Jedenastka
 <b>Created:</b> 03/10/2008
 <b>Contents:</b> <TEXT>
 </pre>
 */

#include "UDPSocket.h"
#include <iostream>

using namespace std;

UDPSocket::UDPSocket() {
    
#ifdef WIN32
	WSAData wsaData;
	WSAStartup(MAKEWORD(2, 0), &wsaData);
#endif
    
	fileDescriptor = ::socket(PF_INET, SOCK_DGRAM, IPPROTO_UDP);
    
#ifdef WIN32
	created = fileDescriptor != INVALID_SOCKET;
#else
	created = fileDescriptor != -1;
#endif
	
    if (created) {
        
		cout << "UDP socket created." << endl;
	} else {
        
#ifdef WIN32
		WSACleanup();
#endif
        
		cerr << "Failed to create UDP socket." << endl;
	}
    
	sendandReceive = created;
}

bool UDPSocket::connect(sockaddr_in *serveraddr_in) {
    
	this->serveraddr_in = serveraddr_in;
	return true;
}

bool UDPSocket::send(char *msg) {
    
	if (sendandReceive) {
        
		int msgLen = (int) strlen(msg);
		int msgLen2 = htonl(msgLen);
		memcpy(sendingBuffer, &msgLen2, sizeof(msgLen2));
		memcpy(sendingBuffer + sizeof(msgLen2), msg, msgLen);
		int sentBytes = ::sendto(fileDescriptor, sendingBuffer, sizeof(msgLen2) + msgLen, 0, (sockaddr *) serveraddr_in, sizeof(*serveraddr_in));
		
        switch (sentBytes) {
                
#ifdef WIN32
			case SOCKET_ERROR:
#else
			case -1:
#endif
				cerr << "Failed to send data." << endl;
				sendandReceive = false;
				break;
                
			default:
				if (sentBytes != (int) sizeof(msgLen2) + msgLen) {
                    
					cerr << "Data sending error. The length of sent data does not match." << endl;
					sendandReceive = false;
				}
		}
	}
    
	return sendandReceive;
}

bool UDPSocket::receive(char *msg) {
    
	if (sendandReceive) {
        
		fd_set fileDescriptorSet;
		FD_ZERO(&fileDescriptorSet);
		FD_SET(fileDescriptor, &fileDescriptorSet);
		timeval timeLimit;
		timeLimit.tv_sec = 1;
		timeLimit.tv_usec = 0;
        
		switch (select((int) fileDescriptor + 1, &fileDescriptorSet, 0, 0, &timeLimit)) {
                
#ifdef WIN32
			case SOCKET_ERROR:
#else
			case -1:
#endif
				cerr << "Time limit error." << endl;
				sendandReceive = false;
				break;
                
			case 0:
				cout << "Time limit expired." << endl;
				sendandReceive = false;
				break;
                
			default:
				timeLimit.tv_sec = 0;
				timeLimit.tv_usec = 0;
#ifdef WIN32
				int sizeofServeraddr_in = sizeof(*serveraddr_in);
#else
				socklen_t sizeofServeraddr_in = sizeof(*serveraddr_in);
#endif
				do {
					int receivedBytes = ::recvfrom(fileDescriptor, receivingBuffer, 4096, 0, (sockaddr *) serveraddr_in, &sizeofServeraddr_in);
                    
					switch (receivedBytes) {
#ifdef WIN32
						case SOCKET_ERROR:
#else
						case -1:
#endif
							cerr << "Failed to receive data." << endl;
							sendandReceive = false;
							break;
                            
						case 0:
							cout << "No data available." << endl;
							sendandReceive = false;
							break;
                            
						default:
							int msgLen2;
							memcpy(&msgLen2, receivingBuffer, sizeof(msgLen2));
							int msgLen = ntohl(msgLen2);
							memcpy(msg, receivingBuffer + sizeof(msgLen2), msgLen);
							msg[msgLen] = '\0';
					}
				}
                
				while (sendandReceive && select((int) fileDescriptor + 1, &fileDescriptorSet, 0, 0, &timeLimit) > 0);
		}
	}
    
	return sendandReceive;
}

bool UDPSocket::close()
{
	if (created)
	{
#ifdef WIN32
		created = ::closesocket(fileDescriptor) == SOCKET_ERROR;
#else
		created = ::close(fileDescriptor) == -1;
#endif
		if (!created)
		{
#ifdef WIN32
			WSACleanup();
#endif
			cout << "UDP socket closed." << endl;
			sendandReceive = false;
		}
		else
		{
			cerr << "Failed to close UDP socket." << endl;
		}
	}
	return !created;
}
