/*! \file Logger.h
 <pre>
 <b>File:</b> Logger.h
 <b>Project:</b> Robocup 3D
 <b>Authors:</b> Hviezdna Jedenastka
 <b>Created:</b> 04/13/2008
 <b>Contents:</b> This file contains logger class.
 </pre>
 */

#ifndef	LOGGER_H
#define LOGGER_H

#include <string>
#include <iostream>
#include <stdarg.h>

using namespace std;

//uncomment this for "logger" to link
//or set it in project settings (reccomended)
//#define _LOGGER

class Logger {
    
protected:
    
    Logger(char* filename);
    
public:
    
    ~Logger();
    
    //don't change order
    //LOG_COUNT has to be the last item!!!!
    typedef enum LoggerSwitches {
        
        LOG_RECIEVED_DATA = 0,
        LOG_SENT_DATA,
        LOG_PARSE_TEST,
        LOG_COMMUNICATION,
        LOG_CONFIG,
        LOG_THREADS,
        LOG_PLAYER_SIMTIME,
        LOG_PLAYER_GYROSCOPE,
        LOG_PLAYER_FRP,
        LOG_PLAYER_ANKLES,
        LOG_PLAYER_KNEES,
        LOG_PLAYER_LEGS,
        LOG_PLAYER_HIPS,
        LOG_PLAYER_SHOULDERS,
        LOG_PLAYER_ELBOWS,
        LOG_PLAYER_ARMS,
        LOG_WORLD_BALL_POSITION,
        LOG_TEMP,
        LOG_COUNT
    } SwitchID;
    
    void SetSwitches();	//temp
    void SetSwitchesFromXML(); //to do
    
    static void logoutput(int switch_id, const char *txt, ...);
    static void LoggerInit(char* filename);
    
    static char * getSwitchName(int switch_id);
    static void SetSwitch(int swich_id, bool value);
    
private:
    
    static FILE *f_descr;
    static Logger* pInstance;
    
    //switches
    static bool switches[LOG_COUNT];
    
};

extern const char *log_recieved;
extern const char *log_sent;
extern const char *log_parse_test;
extern const char *log_comm_starting;
extern const char *log_comm_started;
extern const char *log_comm_start_failed;
extern const char *log_comm_stopping;
extern const char *log_comm_stopped;
extern const char *log_comm_stop_failed;
extern const char *log_config_failed;
extern const char *log_undefined_usethreads;
extern const char *log_using_config;
extern const char *log_comm_tcpsock_create;
extern const char *log_comm_tcpsock_create_fail;
extern const char *log_comm_tcpsock_close;
extern const char *log_comm_tcpsock_close_fail;
extern const char *log_comm_connected;
extern const char *log_comm_connected2;
extern const char *log_comm_connect_fail;
extern const char *log_comm_connect_fail2;
extern const char *log_comm_fail_sending;
extern const char *log_comm_sending_error;
extern const char *log_comm_fail_receiving;
extern const char *log_comm_receiving_error;
extern const char *log_comm_receiving_no_data;
extern const char *log_comm_out_msg_too_long;
extern const char *log_comm_received_data_too_long;
extern const char *log_comm_error_select;
extern const char *log_thread_creating;
extern const char *log_thread_creating_error;
extern const char *log_thread_created;
extern const char *log_thread_joining;
extern const char *log_thread_joining_error;
extern const char *log_thread_joined;
extern const char *log_simtime;
extern const char *log_playermodel_gyroscope;
extern const char *log_playermodel_FRP;
extern const char *log_playermodel_ankles;
extern const char *log_playermodel_knees;
extern const char *log_playermodel_legs;
extern const char *log_playermodel_hips;
extern const char *log_playermodel_shoulders;
extern const char *log_playermodel_elbows;
extern const char *log_playermodel_arms;
extern const char *log_worldmodel_ball;

extern const char *log_simplestring;
extern const char *log_simplestring_with_float;
extern const char *log_simplestring_3_float;
extern const char *log_simplestring_with_int;

#endif // LOGGER_H
