/*! \file ServerMsgParser.cpp
 <pre>
 <b>File:</b> ServerMsgParser.cpp
 <b>Project:</b> Robocup 3D
 <b>Authors:</b> Hviezdna Jedenastka
 <b>Created:</b> 03/10/2008
 <b>Contents:</b> Implementation of parser class ApgServerMsgParser.
 </pre>
 */

/********************************************************************
 APG - an ABNF Parser Generator
 Copyright  (C) 2007 Coast to Coast Research, Inc.
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 2 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program.  If not, see
 <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
 or write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 created: Sun Mar  2 16:22:08 2008
 filename: ServerMsgParser.cpp
 author: Lowell Thomas
 lowell@coasttocoastresearch.com
 http://www.coasttocoastresearch.com
 
 purpose: Implementation of parser class ApgServerMsgParser.
 
 *********************************************************************/
#include "Apg.h"

//{{Include.pre

//}}Include.pre

#include "ServerMsgParser.h"

//{{Include.post
char *psr_msg_string;		//msg from server
char parse_buffer[1024];	//temp buffer
ServerMsgParserData *pStoredData;
int curr_obj_id;
int curr_joint_id;
int frp_curr_foot;		// 0-left, 1-right
//}}Include.post

//  GENERAL INFORMATION:
//       static data = 6204 (bytes)
//                   = 6.06E+00 (KB)
//                   = 5.92E-03 (MB)
//   character count = 123 (grammar alphabet character set)
// unused characters = 55 (grammar alphabet character set)
//        rule count = 60
//      opcode count = 300
//         ALT count = 11(43) (alternations)
//         CAT count = 16(143) (concatenations)
//         RNM count = 132
//         REP count = 54
//         PRD count = 0
//         TRG count = 4
//         TBS count = 0
//         TLS count = 83
//      string table = 628 (bytes)
//           opcodes = 4056 (bytes)
//         rule list = 1200 (bytes)
//              PPTs = <not used>

// CALL BACK FUNCTION POINTERS
static ulong g_ulCallbacks = 26;
static PFN_CALLBACK g_pfnCallbacks[26] =
{ &ApgServerMsgParser::pfn_Time, &ApgServerMsgParser::pfn_UnumNr, &ApgServerMsgParser::pfn_TeamLeft, &ApgServerMsgParser::pfn_TeamRight,
    &ApgServerMsgParser::pfn_GSTime, &ApgServerMsgParser::pfn_PlayMode, &ApgServerMsgParser::pfn_GYRx, &ApgServerMsgParser::pfn_GYRy,
	&ApgServerMsgParser::pfn_GYRz,
    &ApgServerMsgParser::pfn_ObjectName, &ApgServerMsgParser::pfn_SEEx, &ApgServerMsgParser::pfn_SEEy, &ApgServerMsgParser::pfn_SEEz,
	&ApgServerMsgParser::pfn_UJName,
    &ApgServerMsgParser::pfn_UJax1, &ApgServerMsgParser::pfn_UJax2, &ApgServerMsgParser::pfn_HJName, &ApgServerMsgParser::pfn_HJax,
	&ApgServerMsgParser::pfn_FNameLF,
    &ApgServerMsgParser::pfn_FNameRF, &ApgServerMsgParser::pfn_FRPCx, &ApgServerMsgParser::pfn_FRPCy, &ApgServerMsgParser::pfn_FRPCz,
	&ApgServerMsgParser::pfn_FRPFx,
    &ApgServerMsgParser::pfn_FRPFy, &ApgServerMsgParser::pfn_FRPFz,
};


// EXECUTABLE DATA
static ulong g_ulaExeData[1491] =
{ 0x0000174C, 0xE266CE2B, 0x00000000, 0x0000003C, 0x0000012C, 0x0000007B, 0x00000000, 0x0000007B, 0x0000007A, 0x00000000, 0x00000000,
    0x00000050, 0x00000050, 0x00000274, 0x000002C4, 0x00000FD8, 0x0000129C, 0x000004B0, 0x00000000, 0x00000000, 0x7373654D, 0x00656761, 0x4567734D,
	0x4D006D6C, 0x69546773,
    0x2800656D, 0x776F6E28, 0x69742929, 0x7453656D, 0x69740072, 0x6954656D, 0x4D00656D, 0x61476773, 0x7453656D, 0x00657461, 0x53472928, 0x4567734D,
	0x4700626D, 0x67734D53,
    0x656D6954, 0x29742800, 0x734D5347, 0x4D6C5067, 0x0065646F, 0x296D7028, 0x734D5347, 0x756E5567, 0x7528006D, 0x296D756E, 0x734D5347, 0x61655467,
	0x7428006D, 0x296D6165,
    0x6D756E55, 0x5400724E, 0x4C6D6165, 0x00746665, 0x7466656C, 0x6D616554, 0x68676952, 0x69720074, 0x47746867, 0x72745353, 0x47736700, 0x6D695453,
	0x6C500065, 0x6F4D7961,
    0x4D006564, 0x79476773, 0x61526F72, 0x28006574, 0x6F746E28, 0x296F7372, 0x29747228, 0x72794729, 0x00727453, 0x47727967, 0x00785259, 0x79525947,
	0x52594700, 0x734D007A,
    0x65655367, 0x53292800, 0x74536565, 0x65730072, 0x65655365, 0x4F656E4F, 0x63656A62, 0x28280074, 0x296C6F70, 0x6A624F29, 0x4E746365, 0x00656D61,
	0x78454553, 0x45455300,
    0x45530079, 0x4D007A45, 0x6E556773, 0x72657669, 0x4A6C6173, 0x746E696F, 0x6E282800, 0x78612829, 0x61282931, 0x29293278, 0x74534A55, 0x6A750072,
	0x614E4A55, 0x5500656D,
    0x3178614A, 0x614A5500, 0x4D003278, 0x69486773, 0x0065676E, 0x296E2828, 0x29786128, 0x534A4829, 0x68007274, 0x4E4A486A, 0x00656D61, 0x78614A48,
	0x67734D00, 0x63726F46,
    0x73655265, 0x61747369, 0x0065636E, 0x296E2828, 0x28296328, 0x46292966, 0x74535052, 0x72660072, 0x614E4670, 0x4600656D, 0x656D614E, 0x6C00464C,
	0x614E4666, 0x4652656D,
    0x46667200, 0x78435052, 0x50524600, 0x46007943, 0x7A435052, 0x50524600, 0x46007846, 0x79465052, 0x50524600, 0x4D007A46, 0x6E556773, 0x776F6E6B,
	0x4D292800, 0x61546773,
    0x53006C69, 0x4C410050, 0x00414850, 0x49474944, 0x65720054, 0x724E6C61, 0x612E2D00, 0x6168706C, 0x736D756E, 0x6E697274, 0x72620067, 0x656B6361,
	0x74732D74, 0x676E6972,
    0x73292800, 0x6E697274, 0x69772D67, 0x756F6874, 0x72622D74, 0x732E5F00, 0x6E697274, 0x29280067, 0x64692E5F, 0x69746E65, 0x72656966, 0x00005F00,
	0x00000003, 0x0000001C,
    0x00000000, 0xFFFFFFFF, 0x00000000, 0x0000001C, 0x00000001, 0x00000001, 0x00000090, 0x00000000, 0x00000030, 0x00000002, 0x00000000, 0x0000003C,
	0x00000005, 0x00000000,
    0x00000048, 0x00000011, 0x00000000, 0x00000054, 0x0000001D, 0x00000000, 0x00000060, 0x00000022, 0x00000000, 0x0000006C, 0x00000016, 0x00000000,
	0x00000078, 0x00000026,
    0x00000000, 0x00000084, 0x00000031, 0x00000000, 0x00000090, 0x00000032, 0x00000002, 0x00000128, 0x00000007, 0x000000A8, 0x00000017, 0x00000001,
	0x00000000, 0x000000B4,
    0x00000003, 0x00000003, 0x000000D0, 0x00000000, 0xFFFFFFFF, 0x00000000, 0x000000D0, 0x00000033, 0x00000007, 0x000000E0, 0x00000018, 0x00000001,
	0x00000007, 0x000000F0,
    0x00000019, 0x00000003, 0x00000003, 0x0000010C, 0x00000000, 0xFFFFFFFF, 0x00000000, 0x0000010C, 0x00000033, 0x00000000, 0x00000118, 0x00000004,
	0x00000007, 0x00000128,
    0x0000001C, 0x00000002, 0x00000007, 0x00000138, 0x00000026, 0x00000004, 0x00000000, 0x00000144, 0x00000036, 0x00000002, 0x000001D4, 0x00000007,
	0x0000015C, 0x0000003C,
    0x00000001, 0x00000000, 0x00000168, 0x0000000E, 0x00000003, 0x00000184, 0x00000000, 0xFFFFFFFF, 0x00000000, 0x00000184, 0x00000033, 0x00000003,
	0x000001C4, 0x00000000,
    0xFFFFFFFF, 0x00000002, 0x000001C4, 0x00000000, 0x000001A8, 0x00000006, 0x00000003, 0x000001C4, 0x00000000, 0xFFFFFFFF, 0x00000000, 0x000001C4,
	0x00000033, 0x00000007,
    0x000001D4, 0x0000003D, 0x00000001, 0x00000001, 0x0000020C, 0x00000000, 0x000001E8, 0x00000007, 0x00000000, 0x000001F4, 0x00000008, 0x00000000,
	0x00000200, 0x00000009,
    0x00000000, 0x0000020C, 0x0000000A, 0x00000002, 0x0000026C, 0x00000007, 0x00000224, 0x00000051, 0x00000001, 0x00000007, 0x00000234, 0x00000052,
	0x00000001, 0x00000003,
    0x00000250, 0x00000000, 0xFFFFFFFF, 0x00000000, 0x00000250, 0x00000033, 0x00000000, 0x0000025C, 0x0000000F, 0x00000007, 0x0000026C, 0x00000053,
	0x00000001, 0x00000002,
    0x000002CC, 0x00000007, 0x00000284, 0x00000060, 0x00000001, 0x00000007, 0x00000294, 0x00000061, 0x00000002, 0x00000003, 0x000002B0, 0x00000000,
	0xFFFFFFFF, 0x00000000,
    0x000002B0, 0x00000033, 0x00000000, 0x000002BC, 0x00000010, 0x00000007, 0x000002CC, 0x00000063, 0x00000001, 0x00000002, 0x0000032C, 0x00000007,
	0x000002E4, 0x0000006E,
    0x00000001, 0x00000007, 0x000002F4, 0x0000006F, 0x00000004, 0x00000003, 0x00000310, 0x00000000, 0xFFFFFFFF, 0x00000000, 0x00000310, 0x00000033,
	0x00000000, 0x0000031C,
    0x0000000B, 0x00000007, 0x0000032C, 0x00000073, 0x00000001, 0x00000002, 0x000003A0, 0x00000007, 0x00000344, 0x0000007E, 0x00000001, 0x00000007,
	0x00000354, 0x0000007F,
    0x00000004, 0x00000003, 0x00000370, 0x00000000, 0xFFFFFFFF, 0x00000000, 0x00000370, 0x00000033, 0x00000001, 0x00000390, 0x00000000, 0x00000384,
	0x0000000C, 0x00000000,
    0x00000390, 0x0000000D, 0x00000007, 0x000003A0, 0x00000083, 0x00000001, 0x00000003, 0x000003BC, 0x00000001, 0xFFFFFFFF, 0x00000000, 0x000003BC,
	0x00000035, 0x00000007,
    0x000003CC, 0x00000094, 0x00000004, 0x00000007, 0x000003DC, 0x000000A2, 0x00000005, 0x00000007, 0x000003EC, 0x000000AD, 0x00000002, 0x00000000,
	0x000003F8, 0x00000036,
    0x00000000, 0x00000404, 0x0000003B, 0x00000002, 0x00000564, 0x00000007, 0x0000041C, 0x000000CB, 0x00000001, 0x00000000, 0x00000428, 0x00000012,
	0x00000003, 0x00000444,
    0x00000000, 0xFFFFFFFF, 0x00000000, 0x00000444, 0x00000033, 0x00000007, 0x00000454, 0x000000CC, 0x00000001, 0x00000007, 0x00000464, 0x000000CD,
	0x00000001, 0x00000003,
    0x00000480, 0x00000000, 0xFFFFFFFF, 0x00000000, 0x00000480, 0x00000033, 0x00000007, 0x00000490, 0x000000CE, 0x00000005, 0x00000007, 0x000004A0,
	0x000000D3, 0x00000001,
    0x00000003, 0x000004BC, 0x00000000, 0xFFFFFFFF, 0x00000000, 0x000004BC, 0x00000033, 0x00000007, 0x000004CC, 0x000000D4, 0x00000001, 0x00000007,
	0x000004DC, 0x000000D5,
    0x00000002, 0x00000003, 0x000004F8, 0x00000000, 0xFFFFFFFF, 0x00000000, 0x000004F8, 0x00000033, 0x00000000, 0x00000504, 0x00000013, 0x00000003,
	0x00000520, 0x00000000,
    0xFFFFFFFF, 0x00000000, 0x00000520, 0x00000033, 0x00000000, 0x0000052C, 0x00000014, 0x00000003, 0x00000548, 0x00000000, 0xFFFFFFFF, 0x00000000,
	0x00000548, 0x00000033,
    0x00000000, 0x00000554, 0x00000015, 0x00000007, 0x00000564, 0x000000D7, 0x00000002, 0x00000007, 0x00000574, 0x000000E0, 0x00000003, 0x00000000,
	0x00000580, 0x00000036,
    0x00000000, 0x0000058C, 0x00000036, 0x00000000, 0x00000598, 0x00000036, 0x00000002, 0x00000604, 0x00000007, 0x000005B0, 0x000000F9, 0x00000001,
	0x00000000, 0x000005BC,
    0x00000017, 0x00000003, 0x000005D8, 0x00000000, 0xFFFFFFFF, 0x00000000, 0x000005D8, 0x00000033, 0x00000003, 0x000005F4, 0x00000000, 0xFFFFFFFF,
	0x00000000, 0x000005F4,
    0x00000018, 0x00000007, 0x00000604, 0x000000FA, 0x00000001, 0x00000007, 0x00000614, 0x00000102, 0x00000003, 0x00000002, 0x00000718, 0x00000007,
	0x0000062C, 0x00000112,
    0x00000001, 0x00000000, 0x00000638, 0x00000019, 0x00000003, 0x00000654, 0x00000000, 0xFFFFFFFF, 0x00000000, 0x00000654, 0x00000033, 0x00000007,
	0x00000664, 0x00000113,
    0x00000001, 0x00000007, 0x00000674, 0x00000114, 0x00000003, 0x00000003, 0x00000690, 0x00000000, 0xFFFFFFFF, 0x00000000, 0x00000690, 0x00000033,
	0x00000000, 0x0000069C,
    0x0000001A, 0x00000003, 0x000006B8, 0x00000000, 0xFFFFFFFF, 0x00000000, 0x000006B8, 0x00000033, 0x00000000, 0x000006C4, 0x0000001B, 0x00000003,
	0x000006E0, 0x00000000,
    0xFFFFFFFF, 0x00000000, 0x000006E0, 0x00000033, 0x00000000, 0x000006EC, 0x0000001C, 0x00000007, 0x000006FC, 0x00000117, 0x00000002, 0x00000003,
	0x00000718, 0x00000000,
    0xFFFFFFFF, 0x00000000, 0x00000718, 0x00000033, 0x00000000, 0x00000724, 0x0000003B, 0x00000000, 0x00000730, 0x00000036, 0x00000000, 0x0000073C,
	0x00000036, 0x00000000,
    0x00000748, 0x00000036, 0x00000002, 0x000008C8, 0x00000007, 0x00000760, 0x00000145, 0x00000001, 0x00000000, 0x0000076C, 0x0000001E, 0x00000003,
	0x00000788, 0x00000000,
    0xFFFFFFFF, 0x00000000, 0x00000788, 0x00000033, 0x00000007, 0x00000798, 0x00000146, 0x00000001, 0x00000007, 0x000007A8, 0x00000147, 0x00000001,
	0x00000003, 0x000007C4,
    0x00000000, 0xFFFFFFFF, 0x00000000, 0x000007C4, 0x00000033, 0x00000000, 0x000007D0, 0x0000001F, 0x00000007, 0x000007E0, 0x00000148, 0x00000001,
	0x00000003, 0x000007FC,
    0x00000000, 0xFFFFFFFF, 0x00000000, 0x000007FC, 0x00000033, 0x00000007, 0x0000080C, 0x00000149, 0x00000001, 0x00000007, 0x0000081C, 0x0000014A,
	0x00000003, 0x00000003,
    0x00000838, 0x00000000, 0xFFFFFFFF, 0x00000000, 0x00000838, 0x00000033, 0x00000000, 0x00000844, 0x00000020, 0x00000007, 0x00000854, 0x0000014D,
	0x00000001, 0x00000003,
    0x00000870, 0x00000000, 0xFFFFFFFF, 0x00000000, 0x00000870, 0x00000033, 0x00000007, 0x00000880, 0x0000014E, 0x00000001, 0x00000007, 0x00000890,
	0x0000014F, 0x00000003,
    0x00000003, 0x000008AC, 0x00000000, 0xFFFFFFFF, 0x00000000, 0x000008AC, 0x00000033, 0x00000000, 0x000008B8, 0x00000021, 0x00000007, 0x000008C8,
	0x00000152, 0x00000002,
    0x00000007, 0x000008D8, 0x0000015A, 0x00000002, 0x00000000, 0x000008E4, 0x0000003B, 0x00000000, 0x000008F0, 0x00000036, 0x00000000, 0x000008FC,
	0x00000036, 0x00000002,
    0x00000A08, 0x00000007, 0x00000914, 0x00000178, 0x00000001, 0x00000000, 0x00000920, 0x00000023, 0x00000003, 0x0000093C, 0x00000000, 0xFFFFFFFF,
	0x00000000, 0x0000093C,
    0x00000033, 0x00000007, 0x0000094C, 0x00000179, 0x00000001, 0x00000007, 0x0000095C, 0x0000017A, 0x00000001, 0x00000003, 0x00000978, 0x00000000,
	0xFFFFFFFF, 0x00000000,
    0x00000978, 0x00000033, 0x00000000, 0x00000984, 0x00000024, 0x00000007, 0x00000994, 0x0000017B, 0x00000001, 0x00000003, 0x000009B0, 0x00000000,
	0xFFFFFFFF, 0x00000000,
    0x000009B0, 0x00000033, 0x00000007, 0x000009C0, 0x0000017C, 0x00000001, 0x00000007, 0x000009D0, 0x0000017D, 0x00000002, 0x00000003, 0x000009EC,
	0x00000000, 0xFFFFFFFF,
    0x00000000, 0x000009EC, 0x00000033, 0x00000000, 0x000009F8, 0x00000025, 0x00000007, 0x00000A08, 0x0000017F, 0x00000002, 0x00000007, 0x00000A18,
	0x00000187, 0x00000002,
    0x00000000, 0x00000A24, 0x0000003B, 0x00000000, 0x00000A30, 0x00000036, 0x00000002, 0x00000C50, 0x00000007, 0x00000A48, 0x000001A8, 0x00000001,
	0x00000000, 0x00000A54,
    0x00000027, 0x00000003, 0x00000A70, 0x00000000, 0xFFFFFFFF, 0x00000000, 0x00000A70, 0x00000033, 0x00000007, 0x00000A80, 0x000001A9, 0x00000001,
	0x00000007, 0x00000A90,
    0x000001AA, 0x00000001, 0x00000003, 0x00000AAC, 0x00000000, 0xFFFFFFFF, 0x00000000, 0x00000AAC, 0x00000033, 0x00000000, 0x00000AB8, 0x00000028,
	0x00000007, 0x00000AC8,
    0x000001AB, 0x00000001, 0x00000003, 0x00000AE4, 0x00000000, 0xFFFFFFFF, 0x00000000, 0x00000AE4, 0x00000033, 0x00000007, 0x00000AF4, 0x000001AC,
	0x00000001, 0x00000007,
    0x00000B04, 0x000001AD, 0x00000001, 0x00000003, 0x00000B20, 0x00000000, 0xFFFFFFFF, 0x00000000, 0x00000B20, 0x00000033, 0x00000000, 0x00000B2C,
	0x0000002B, 0x00000003,
    0x00000B48, 0x00000000, 0xFFFFFFFF, 0x00000000, 0x00000B48, 0x00000033, 0x00000000, 0x00000B54, 0x0000002C, 0x00000003, 0x00000B70, 0x00000000,
	0xFFFFFFFF, 0x00000000,
    0x00000B70, 0x00000033, 0x00000000, 0x00000B7C, 0x0000002D, 0x00000007, 0x00000B8C, 0x000001AE, 0x00000001, 0x00000003, 0x00000BA8, 0x00000000,
	0xFFFFFFFF, 0x00000000,
    0x00000BA8, 0x00000033, 0x00000007, 0x00000BB8, 0x000001AF, 0x00000001, 0x00000007, 0x00000BC8, 0x000001B0, 0x00000001, 0x00000003, 0x00000BE4,
	0x00000000, 0xFFFFFFFF,
    0x00000000, 0x00000BE4, 0x00000033, 0x00000000, 0x00000BF0, 0x0000002E, 0x00000003, 0x00000C0C, 0x00000000, 0xFFFFFFFF, 0x00000000, 0x00000C0C,
	0x00000033, 0x00000000,
    0x00000C18, 0x0000002F, 0x00000003, 0x00000C34, 0x00000000, 0xFFFFFFFF, 0x00000000, 0x00000C34, 0x00000033, 0x00000000, 0x00000C40, 0x00000030,
	0x00000007, 0x00000C50,
    0x000001B1, 0x00000002, 0x00000007, 0x00000C60, 0x000001BA, 0x00000003, 0x00000001, 0x00000C80, 0x00000000, 0x00000C74, 0x00000029, 0x00000000,
	0x00000C80, 0x0000002A,
    0x00000007, 0x00000C90, 0x000001CB, 0x00000002, 0x00000007, 0x00000CA0, 0x000001D5, 0x00000002, 0x00000000, 0x00000CAC, 0x00000036, 0x00000000,
	0x00000CB8, 0x00000036,
    0x00000000, 0x00000CC4, 0x00000036, 0x00000000, 0x00000CD0, 0x00000036, 0x00000000, 0x00000CDC, 0x00000036, 0x00000000, 0x00000CE8, 0x00000036,
	0x00000002, 0x00000D5C,
    0x00000007, 0x00000D00, 0x00000205, 0x00000001, 0x00000003, 0x00000D4C, 0x00000000, 0xFFFFFFFF, 0x00000001, 0x00000D4C, 0x00000000, 0x00000D24,
	0x00000039, 0x00000000,
    0x00000D30, 0x00000038, 0x00000003, 0x00000D4C, 0x00000000, 0xFFFFFFFF, 0x00000000, 0x00000D4C, 0x00000033, 0x00000007, 0x00000D5C, 0x00000206,
	0x00000001, 0x00000000,
    0x00000D68, 0x0000003A, 0x00000005, 0x00000D78, 0x00000020, 0x00000020, 0x00000001, 0x00000DA0, 0x00000005, 0x00000D90, 0x00000041, 0x0000005A,
	0x00000005, 0x00000DA0,
    0x00000061, 0x0000007A, 0x00000005, 0x00000DB0, 0x00000030, 0x00000039, 0x00000002, 0x00000E20, 0x00000003, 0x00000DD8, 0x00000000, 0x00000001,
	0x00000007, 0x00000DD8,
    0x00000225, 0x00000001, 0x00000003, 0x00000DF4, 0x00000001, 0xFFFFFFFF, 0x00000000, 0x00000DF4, 0x00000035, 0x00000007, 0x00000E04, 0x00000226,
	0x00000001, 0x00000003,
    0x00000E20, 0x00000001, 0xFFFFFFFF, 0x00000000, 0x00000E20, 0x00000035, 0x00000003, 0x00000E5C, 0x00000000, 0xFFFFFFFF, 0x00000001, 0x00000E5C,
	0x00000000, 0x00000E44,
    0x00000034, 0x00000000, 0x00000E50, 0x00000035, 0x00000000, 0x00000E5C, 0x00000033, 0x00000002, 0x00000EC0, 0x00000007, 0x00000E74, 0x00000245,
	0x00000001, 0x00000003,
    0x00000EB0, 0x00000000, 0xFFFFFFFF, 0x00000001, 0x00000EB0, 0x00000000, 0x00000E98, 0x00000037, 0x00000000, 0x00000EA4, 0x00000038, 0x00000000,
	0x00000EB0, 0x00000033,
    0x00000007, 0x00000EC0, 0x00000246, 0x00000001, 0x00000003, 0x00000F1C, 0x00000000, 0xFFFFFFFF, 0x00000001, 0x00000F1C, 0x00000000, 0x00000EE4,
	0x00000034, 0x00000000,
    0x00000EF0, 0x00000035, 0x00000000, 0x00000EFC, 0x00000033, 0x00000007, 0x00000F0C, 0x00000259, 0x00000001, 0x00000007, 0x00000F1C, 0x0000025A,
	0x00000001, 0x00000003,
    0x00000F98, 0x00000000, 0xFFFFFFFF, 0x00000001, 0x00000F98, 0x00000000, 0x00000F40, 0x00000034, 0x00000000, 0x00000F4C, 0x00000035, 0x00000000,
	0x00000F58, 0x00000033,
    0x00000007, 0x00000F68, 0x00000262, 0x00000001, 0x00000007, 0x00000F78, 0x00000263, 0x00000001, 0x00000007, 0x00000F88, 0x00000264, 0x00000001,
	0x00000007, 0x00000F98,
    0x00000265, 0x00000001, 0x00000003, 0x00000FD8, 0x00000001, 0xFFFFFFFF, 0x00000001, 0x00000FD8, 0x00000000, 0x00000FBC, 0x00000034, 0x00000000,
	0x00000FC8, 0x00000035,
    0x00000007, 0x00000FD8, 0x00000271, 0x00000001, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0xFFFFFFFF, 0x00000008, 0x00000000, 0x0000001C,
	0x00000000, 0xFFFFFFFF,
    0x0000000F, 0x00000000, 0x00000090, 0x00000000, 0xFFFFFFFF, 0x0000001E, 0x00000000, 0x00000128, 0x00000000, 0xFFFFFFFF, 0x0000002A, 0x00000000,
	0x00000138, 0x00000001,
    0x00000000, 0x0000002F, 0x00000000, 0x00000144, 0x00000000, 0xFFFFFFFF, 0x0000003E, 0x00000000, 0x000001D4, 0x00000000, 0xFFFFFFFF, 0x00000047,
	0x00000000, 0x0000020C,
    0x00000000, 0xFFFFFFFF, 0x00000054, 0x00000000, 0x0000026C, 0x00000000, 0xFFFFFFFF, 0x00000064, 0x00000000, 0x000002CC, 0x00000000, 0xFFFFFFFF,
	0x00000074, 0x00000000,
    0x0000032C, 0x00000000, 0xFFFFFFFF, 0x00000084, 0x00000000, 0x000003A0, 0x00000001, 0x00000001, 0x0000008B, 0x00000000, 0x000003BC, 0x00000001,
	0x00000002, 0x00000098,
    0x00000000, 0x000003CC, 0x00000001, 0x00000003, 0x000000A7, 0x00000000, 0x000003DC, 0x00000000, 0xFFFFFFFF, 0x000000AF, 0x00000000, 0x000003EC,
	0x00000001, 0x00000004,
    0x000000B6, 0x00000000, 0x000003F8, 0x00000001, 0x00000005, 0x000000BF, 0x00000000, 0x00000404, 0x00000000, 0xFFFFFFFF, 0x000000D9, 0x00000000,
	0x00000564, 0x00000000,
    0xFFFFFFFF, 0x000000E3, 0x00000000, 0x00000574, 0x00000001, 0x00000006, 0x000000E8, 0x00000000, 0x00000580, 0x00000001, 0x00000007, 0x000000ED,
	0x00000000, 0x0000058C,
    0x00000001, 0x00000008, 0x000000F2, 0x00000000, 0x00000598, 0x00000000, 0xFFFFFFFF, 0x000000FB, 0x00000000, 0x00000604, 0x00000000, 0xFFFFFFFF,
	0x00000105, 0x00000000,
    0x00000614, 0x00000000, 0xFFFFFFFF, 0x00000119, 0x00000000, 0x00000718, 0x00000001, 0x00000009, 0x00000124, 0x00000000, 0x00000724, 0x00000001,
	0x0000000A, 0x00000129,
    0x00000000, 0x00000730, 0x00000001, 0x0000000B, 0x0000012E, 0x00000000, 0x0000073C, 0x00000001, 0x0000000C, 0x00000133, 0x00000000, 0x00000748,
	0x00000000, 0xFFFFFFFF,
    0x00000154, 0x00000000, 0x000008C8, 0x00000000, 0xFFFFFFFF, 0x0000015C, 0x00000000, 0x000008D8, 0x00000001, 0x0000000D, 0x00000163, 0x00000000,
	0x000008E4, 0x00000001,
    0x0000000E, 0x00000169, 0x00000000, 0x000008F0, 0x00000001, 0x0000000F, 0x0000016F, 0x00000000, 0x000008FC, 0x00000000, 0xFFFFFFFF, 0x00000181,
	0x00000000, 0x00000A08,
    0x00000000, 0xFFFFFFFF, 0x00000189, 0x00000000, 0x00000A18, 0x00000001, 0x00000010, 0x00000190, 0x00000000, 0x00000A24, 0x00000001, 0x00000011,
	0x00000195, 0x00000000,
    0x00000A30, 0x00000000, 0xFFFFFFFF, 0x000001B3, 0x00000000, 0x00000C50, 0x00000000, 0xFFFFFFFF, 0x000001BD, 0x00000000, 0x00000C60, 0x00000000,
	0xFFFFFFFF, 0x000001C3,
    0x00000000, 0x00000C80, 0x00000001, 0x00000012, 0x000001CD, 0x00000000, 0x00000C90, 0x00000001, 0x00000013, 0x000001D7, 0x00000000, 0x00000CA0,
	0x00000001, 0x00000014,
    0x000001DD, 0x00000000, 0x00000CAC, 0x00000001, 0x00000015, 0x000001E3, 0x00000000, 0x00000CB8, 0x00000001, 0x00000016, 0x000001E9, 0x00000000,
	0x00000CC4, 0x00000001,
    0x00000017, 0x000001EF, 0x00000000, 0x00000CD0, 0x00000001, 0x00000018, 0x000001F5, 0x00000000, 0x00000CDC, 0x00000001, 0x00000019, 0x000001FB,
	0x00000000, 0x00000CE8,
    0x00000000, 0xFFFFFFFF, 0x00000207, 0x00000000, 0x00000D5C, 0x00000000, 0xFFFFFFFF, 0x0000020F, 0x00000000, 0x00000D68, 0x00000000, 0xFFFFFFFF,
	0x00000212, 0x00000000,
    0x00000D78, 0x00000000, 0xFFFFFFFF, 0x00000218, 0x00000000, 0x00000DA0, 0x00000000, 0xFFFFFFFF, 0x0000021E, 0x00000000, 0x00000DB0, 0x00000000,
	0xFFFFFFFF, 0x00000227,
    0x00000000, 0x00000E20, 0x00000000, 0xFFFFFFFF, 0x00000236, 0x00000000, 0x00000E5C, 0x00000000, 0xFFFFFFFF, 0x00000247, 0x00000000, 0x00000EC0,
	0x00000000, 0xFFFFFFFF,
    0x0000025B, 0x00000000, 0x00000F1C, 0x00000000, 0xFFFFFFFF, 0x00000266, 0x00000000, 0x00000F98, 0x00000000, 0xFFFFFFFF,
};


// UNIQUELY NAMED PARSER INITIALIZATION DATA
static APG_PARSER_INIT g_sParserInit = { 0, (void *) g_pfnCallbacks, (void *) g_ulaExeData
};
void *ApgServerMsgParser::m_vpParserInit = (void *) &g_sParserInit;
ApgServerMsgParser::ApgServerMsgParser()
{
    ulong ulReturn = APG_FALSE;
    ulong ulTest;
    
    // get memory and error contexts
    m_vpMemCtx = NULL;
    m_vpParserCtx = NULL;
    m_vpMemCtx = vpMemCtor();
    if (!m_vpMemCtx) {
        goto APG_EXIT;
    }
    // initialize the parser
    ulTest = ulParserInit(m_vpParserInit, m_vpMemCtx);
    if (!ulTest) {
        goto APG_EXIT;
    }
    m_vpParserCtx = vpParserCtor(m_vpParserInit, m_vpMemCtx);
    if (!m_vpParserCtx) {
        goto APG_EXIT;
    }
    // success
    ulReturn = APG_TRUE;
APG_EXIT:if (!ulReturn) {
    
	// clean up resources
	if (m_vpParserCtx) {
	    vParserDtor(m_vpParserCtx);
	    m_vpParserCtx = NULL;
	}
	if (m_vpMemCtx) {
	    vMemDtor(m_vpMemCtx);
	    m_vpMemCtx = NULL;
	}
}
}

ApgServerMsgParser::~ApgServerMsgParser()
{
    
    // clean up resources
    if (m_vpParserCtx) {
        vParserDtor(m_vpParserCtx);
    }
    if (m_vpMemCtx) {
        vMemDtor(m_vpMemCtx);
    }
}
ulong ApgServerMsgParser::ulSyntaxAnalysis(ulong ulStartRule, uchar * ucpSrc, ulong ulSrcLen, void *vpData, ulong ulTrace)
{
    ulong ulReturn = APG_FALSE;
    ulong ulTest;
    m_sSrc.ucpStr = ucpSrc;
    m_sSrc.ulLen = ulSrcLen;
    ulTest = ulParserSyntaxAnalysis(m_vpParserCtx, 0, m_sSrc.ucpStr, m_sSrc.ulLen, vpData, ulTrace);
    if (!ulTest) {
        goto APG_EXIT;
    }
    ulTest = ulParserState(m_vpParserCtx, &m_sState);
    if (!ulTest) {
        goto APG_EXIT;
    }
    ulTest = ulParserStats(m_vpParserCtx, &m_sStats);
    if (!ulTest) {
        goto APG_EXIT;
    }
    // success
    ulReturn = APG_TRUE;
APG_EXIT:return ulReturn;
}

ulong ApgServerMsgParser::ulSemanticAnalysis(void *vpData)
{
    return ulParserSemanticAnalysis(m_vpParserCtx, vpData);
}


// switch template
/*
 switch(ulState)
 {
 case SYN_PRE:
 break;
 case SYN_NOMATCH:
 break;
 case SYN_EMPTY:
 break;
 case SYN_MATCH:
 break;
 case SEM_PRE:
 break;
 case SEM_POST:
 break;
 }
 */

//{{CallBack.body
int ApgServerMsgParser::ParseData(uchar * ucpSrc, ulong ulSrcLen)
{
    
    //init psr_msg_string - used in call back functions
    psr_msg_string = (char *) ucpSrc;
    
    //init pointer to stored data
    pStoredData = &StoredData;
    int ret_val = 1;
    
    //parse data by syntax analysis
    //first parameter for ulSyntaxAnalysis is the first ID in grammar
    ret_val = ulSyntaxAnalysis(ID_MESSAGE, ucpSrc, ulSrcLen);
    
    //ret_val = ulSemanticAnalysis(NULL);
    return (ret_val);
}


//}}CallBack.body

// call back function for rule "Time"
ulong ApgServerMsgParser::pfn_Time(void *vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
    ulong ulReturn = PFN_OK;
    
    //{{CallBack.pfn_Time
    if (ulState == SYN_MATCH) {
        strncpy(parse_buffer, &psr_msg_string[ulOffset], ulLen);
        parse_buffer[ulLen] = '\0';
        pStoredData->SetSimTime(atof((char *) parse_buffer));
    }
    //}}CallBack.pfn_Time
    return ulReturn;
}


// call back function for rule "UnumNr"
ulong ApgServerMsgParser::pfn_UnumNr(void *vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
    ulong ulReturn = PFN_OK;
    
    //{{CallBack.pfn_UnumNr
    if (ulState == SYN_MATCH) {
        strncpy(parse_buffer, &psr_msg_string[ulOffset], ulLen);
        parse_buffer[ulLen] = '\0';
        pStoredData->SetPlayerNr((int) atof((char *) parse_buffer));
    }
    //}}CallBack.pfn_UnumNr
    return ulReturn;
}


// call back function for rule "TeamLeft"
ulong ApgServerMsgParser::pfn_TeamLeft(void *vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
    ulong ulReturn = PFN_OK;
    
    //{{CallBack.pfn_TeamLeft
    if (ulState == SYN_MATCH) {
        strncpy(parse_buffer, &psr_msg_string[ulOffset], ulLen);
        parse_buffer[ulLen] = '\0';
        pStoredData->SetTeamIndex(TI_LEFT);
    }
    //}}CallBack.pfn_TeamLeft
    return ulReturn;
}


// call back function for rule "TeamRight"
ulong ApgServerMsgParser::pfn_TeamRight(void *vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
    ulong ulReturn = PFN_OK;
    
    //{{CallBack.pfn_TeamRight
    if (ulState == SYN_MATCH) {
        strncpy(parse_buffer, &psr_msg_string[ulOffset], ulLen);
        parse_buffer[ulLen] = '\0';
        pStoredData->SetTeamIndex(TI_RIGHT);
    }
    //}}CallBack.pfn_TeamRight
    return ulReturn;
}


// call back function for rule "GSTime"
ulong ApgServerMsgParser::pfn_GSTime(void *vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
    ulong ulReturn = PFN_OK;
    
    //{{CallBack.pfn_GSTime
    if (ulState == SYN_MATCH) {
        strncpy(parse_buffer, &psr_msg_string[ulOffset], ulLen);
        parse_buffer[ulLen] = '\0';
        pStoredData->SetGameTime(atof((char *) parse_buffer));
    }
    //}}CallBack.pfn_GSTime
    return ulReturn;
}


// call back function for rule "PlayMode"
ulong ApgServerMsgParser::pfn_PlayMode(void *vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
    ulong ulReturn = PFN_OK;
    
    //{{CallBack.pfn_PlayMode
    if (ulState == SYN_MATCH) {
        strncpy(parse_buffer, &psr_msg_string[ulOffset], ulLen);
        parse_buffer[ulLen] = '\0';
        pStoredData->SetPlayMode(StringToPlayModeID(parse_buffer));
    }
    //}}CallBack.pfn_PlayMode
    return ulReturn;
}


// call back function for rule "GYRx"
ulong ApgServerMsgParser::pfn_GYRx(void *vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
    ulong ulReturn = PFN_OK;
    
    //{{CallBack.pfn_GYRx
    if (ulState == SYN_MATCH) {
        strncpy(parse_buffer, &psr_msg_string[ulOffset], ulLen);
        parse_buffer[ulLen] = '\0';
        pStoredData->SetGyroRate_x(atof((char *) parse_buffer));
    }
    //}}CallBack.pfn_GYRx
    return ulReturn;
}


// call back function for rule "GYRy"
ulong ApgServerMsgParser::pfn_GYRy(void *vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
    ulong ulReturn = PFN_OK;
    
    //{{CallBack.pfn_GYRy
    if (ulState == SYN_MATCH) {
        strncpy(parse_buffer, &psr_msg_string[ulOffset], ulLen);
        parse_buffer[ulLen] = '\0';
        pStoredData->SetGyroRate_y(atof((char *) parse_buffer));
    }
    //}}CallBack.pfn_GYRy
    return ulReturn;
}


// call back function for rule "GYRz"
ulong ApgServerMsgParser::pfn_GYRz(void *vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
    ulong ulReturn = PFN_OK;
    
    //{{CallBack.pfn_GYRz
    if (ulState == SYN_MATCH) {
        strncpy(parse_buffer, &psr_msg_string[ulOffset], ulLen);
        parse_buffer[ulLen] = '\0';
        pStoredData->SetGyroRate_z(atof((char *) parse_buffer));
    }
    //}}CallBack.pfn_GYRz
    return ulReturn;
}


// call back function for rule "ObjectName"
ulong ApgServerMsgParser::pfn_ObjectName(void *vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
    ulong ulReturn = PFN_OK;
    
    //{{CallBack.pfn_ObjectName
    if (ulState == SYN_MATCH) {
        strncpy(parse_buffer, &psr_msg_string[ulOffset], ulLen);
        parse_buffer[ulLen] = '\0';
        curr_obj_id = StringToObjectID(parse_buffer);
    }
    //}}CallBack.pfn_ObjectName
    return ulReturn;
}


// call back function for rule "SEEx"
ulong ApgServerMsgParser::pfn_SEEx(void *vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
    ulong ulReturn = PFN_OK;
    
    //{{CallBack.pfn_SEEx
    if (ulState == SYN_MATCH) {
        strncpy(parse_buffer, &psr_msg_string[ulOffset], ulLen);
        parse_buffer[ulLen] = '\0';
        if (curr_obj_id == OBJ_BALL)
            pStoredData->SetBallPosition_r(atof((char *) parse_buffer));
        
        else if ((curr_obj_id >= OBJ_FLAG_F1L) && (curr_obj_id <= OBJ_FLAG_G2R))
            pStoredData->SetFlagVal_x(curr_obj_id, (atof((char *) parse_buffer)));
    }
    //}}CallBack.pfn_SEEx
    return ulReturn;
}


// call back function for rule "SEEy"
ulong ApgServerMsgParser::pfn_SEEy(void *vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
    ulong ulReturn = PFN_OK;
    
    //{{CallBack.pfn_SEEy
    if (ulState == SYN_MATCH) {
        strncpy(parse_buffer, &psr_msg_string[ulOffset], ulLen);
        parse_buffer[ulLen] = '\0';
        if (curr_obj_id == OBJ_BALL)
            pStoredData->SetBallPosition_theta(atof((char *) parse_buffer));
        
        else if ((curr_obj_id >= OBJ_FLAG_F1L) && (curr_obj_id <= OBJ_FLAG_G2R))
            pStoredData->SetFlagVal_y(curr_obj_id, (atof((char *) parse_buffer)));
    }
    //}}CallBack.pfn_SEEy
    return ulReturn;
}


// call back function for rule "SEEz"
ulong ApgServerMsgParser::pfn_SEEz(void *vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
    ulong ulReturn = PFN_OK;
    
    //{{CallBack.pfn_SEEz
    if (ulState == SYN_MATCH) {
        strncpy(parse_buffer, &psr_msg_string[ulOffset], ulLen);
        parse_buffer[ulLen] = '\0';
        if (curr_obj_id == OBJ_BALL)
            pStoredData->SetBallPosition_phi(atof((char *) parse_buffer));
        
        else if ((curr_obj_id >= OBJ_FLAG_F1L) && (curr_obj_id <= OBJ_FLAG_G2R))
            pStoredData->SetFlagVal_z(curr_obj_id, (atof((char *) parse_buffer)));
    }
    //}}CallBack.pfn_SEEz
    return ulReturn;
}


// call back function for rule "UJName"
ulong ApgServerMsgParser::pfn_UJName(void *vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
    ulong ulReturn = PFN_OK;
    
    //{{CallBack.pfn_UJName
    if (ulState == SYN_MATCH) {
        strncpy(parse_buffer, &psr_msg_string[ulOffset], ulLen);
        parse_buffer[ulLen] = '\0';
        curr_joint_id = StringToUJID(parse_buffer);
    }
    //}}CallBack.pfn_UJName
    return ulReturn;
}


// call back function for rule "UJax1"
ulong ApgServerMsgParser::pfn_UJax1(void *vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
    ulong ulReturn = PFN_OK;
    
    //{{CallBack.pfn_UJax1
    if (ulState == SYN_MATCH) {
        strncpy(parse_buffer, &psr_msg_string[ulOffset], ulLen);
        parse_buffer[ulLen] = '\0';
        pStoredData->SetUJValue_ax1(curr_joint_id, (atof((char *) parse_buffer)));
    }
    //}}CallBack.pfn_UJax1
    return ulReturn;
}


// call back function for rule "UJax2"
ulong ApgServerMsgParser::pfn_UJax2(void *vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
    ulong ulReturn = PFN_OK;
    
    //{{CallBack.pfn_UJax2
    if (ulState == SYN_MATCH) {
        strncpy(parse_buffer, &psr_msg_string[ulOffset], ulLen);
        parse_buffer[ulLen] = '\0';
        pStoredData->SetUJValue_ax2(curr_joint_id, (atof((char *) parse_buffer)));
    }
    //}}CallBack.pfn_UJax2
    return ulReturn;
}


// call back function for rule "HJName"
ulong ApgServerMsgParser::pfn_HJName(void *vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
    ulong ulReturn = PFN_OK;
    
    //{{CallBack.pfn_HJName
    if (ulState == SYN_MATCH) {
        strncpy(parse_buffer, &psr_msg_string[ulOffset], ulLen);
        parse_buffer[ulLen] = '\0';
        curr_joint_id = StringToHJID(parse_buffer);
    }
    //}}CallBack.pfn_HJName
    return ulReturn;
}


// call back function for rule "HJax"
ulong ApgServerMsgParser::pfn_HJax(void *vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
    ulong ulReturn = PFN_OK;
    
    //{{CallBack.pfn_HJax
    if (ulState == SYN_MATCH) {
        strncpy(parse_buffer, &psr_msg_string[ulOffset], ulLen);
        parse_buffer[ulLen] = '\0';
        pStoredData->SetHJValue(curr_joint_id, (atof((char *) parse_buffer)));
    }
    //}}CallBack.pfn_HJax
    return ulReturn;
}


// call back function for rule "FNameLF"
ulong ApgServerMsgParser::pfn_FNameLF(void *vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
    ulong ulReturn = PFN_OK;
    
    //{{CallBack.pfn_FNameLF
    if (ulState == SYN_MATCH) {
        frp_curr_foot = 0;
    }
    //}}CallBack.pfn_FNameLF
    return ulReturn;
}


// call back function for rule "FNameRF"
ulong ApgServerMsgParser::pfn_FNameRF(void *vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
    ulong ulReturn = PFN_OK;
    
    //{{CallBack.pfn_FNameRF
    if (ulState == SYN_MATCH) {
        frp_curr_foot = 1;
    }
    //}}CallBack.pfn_FNameRF
    return ulReturn;
}


// call back function for rule "FRPCx"
ulong ApgServerMsgParser::pfn_FRPCx(void *vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
    ulong ulReturn = PFN_OK;
    
    //{{CallBack.pfn_FRPCx
    if (ulState == SYN_MATCH) {
        strncpy(parse_buffer, &psr_msg_string[ulOffset], ulLen);
        parse_buffer[ulLen] = '\0';
        if (frp_curr_foot)
            pStoredData->SetFrpLocCoordRF_x(atof((char *) parse_buffer));
        
        else
            pStoredData->SetFrpLocCoordLF_x(atof((char *) parse_buffer));
    }
    //}}CallBack.pfn_FRPCx
    return ulReturn;
}


// call back function for rule "FRPCy"
ulong ApgServerMsgParser::pfn_FRPCy(void *vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
    ulong ulReturn = PFN_OK;
    
    //{{CallBack.pfn_FRPCy
    if (ulState == SYN_MATCH) {
        strncpy(parse_buffer, &psr_msg_string[ulOffset], ulLen);
        parse_buffer[ulLen] = '\0';
        if (frp_curr_foot)
            pStoredData->SetFrpLocCoordRF_y(atof((char *) parse_buffer));
        
        else
            pStoredData->SetFrpLocCoordLF_y(atof((char *) parse_buffer));
    }
    //}}CallBack.pfn_FRPCy
    return ulReturn;
}


// call back function for rule "FRPCz"
ulong ApgServerMsgParser::pfn_FRPCz(void *vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
    ulong ulReturn = PFN_OK;
    
    //{{CallBack.pfn_FRPCz
    if (ulState == SYN_MATCH) {
        strncpy(parse_buffer, &psr_msg_string[ulOffset], ulLen);
        parse_buffer[ulLen] = '\0';
        if (frp_curr_foot)
            pStoredData->SetFrpLocCoordRF_z(atof((char *) parse_buffer));
        
        else
            pStoredData->SetFrpLocCoordLF_z(atof((char *) parse_buffer));
    }
    //}}CallBack.pfn_FRPCz
    return ulReturn;
}


// call back function for rule "FRPFx"
ulong ApgServerMsgParser::pfn_FRPFx(void *vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
    ulong ulReturn = PFN_OK;
    
    //{{CallBack.pfn_FRPFx
    if (ulState == SYN_MATCH) {
        strncpy(parse_buffer, &psr_msg_string[ulOffset], ulLen);
        parse_buffer[ulLen] = '\0';
        if (frp_curr_foot)
            pStoredData->SetFrpForceVectorRF_x(atof((char *) parse_buffer));
        
        else
            pStoredData->SetFrpForceVectorLF_x(atof((char *) parse_buffer));
    }
    //}}CallBack.pfn_FRPFx
    return ulReturn;
}


// call back function for rule "FRPFy"
ulong ApgServerMsgParser::pfn_FRPFy(void *vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
    ulong ulReturn = PFN_OK;
    
    //{{CallBack.pfn_FRPFy
    if (ulState == SYN_MATCH) {
        strncpy(parse_buffer, &psr_msg_string[ulOffset], ulLen);
        parse_buffer[ulLen] = '\0';
        if (frp_curr_foot)
            pStoredData->SetFrpForceVectorRF_y(atof((char *) parse_buffer));
        
        else
            pStoredData->SetFrpForceVectorLF_y(atof((char *) parse_buffer));
    }
    //}}CallBack.pfn_FRPFy
    return ulReturn;
}


// call back function for rule "FRPFz"
ulong ApgServerMsgParser::pfn_FRPFz(void *vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
    ulong ulReturn = PFN_OK;
    
    //{{CallBack.pfn_FRPFz
    if (ulState == SYN_MATCH) {
        strncpy(parse_buffer, &psr_msg_string[ulOffset], ulLen);
        parse_buffer[ulLen] = '\0';
        if (frp_curr_foot)
            pStoredData->SetFFrpForceVectorRF_z(atof((char *) parse_buffer));
        
        else
            pStoredData->SetFFrpForceVectorLF_z(atof((char *) parse_buffer));
    }
    //}}CallBack.pfn_FRPFz
    return ulReturn;
}
