/*! \file Thread.cpp
 <pre>
 <b>File:</b> Thread.cpp
 <b>Project:</b> Robocup 3D
 <b>Authors:</b> Hviezdna Jedenastka
 <b>Created:</b> 03/10/2008
 <b>Contents:</b> POSIX Threads wrapper class
 </pre>
 */

#include "Thread.h"
#include "Logger.h"

#include <iostream>

using namespace std;

/*!
 * Body of the thread.
 * ThreadMain executes run method of the Thread object.
 * @param _params thread parameters
 */
void *threadMain(void *_params) {
    
    int rc;
    void *ret;
    ThreadParams *params = (ThreadParams *) _params;
    
    rc = params->thread->run();
    ret = (void *) &rc;
    
    pthread_exit(ret);
    return ret;
}

int Thread::lastid = 0;

Thread::Thread() {
    
    id = ++lastid;
    signal = THREADSIG_STOP;
    status = 0;
    
    params.thread = this;
    
    pthread_attr_init(&attr);
    pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_JOINABLE);
}

Thread::~Thread() {
}

int Thread::start() {
    
    signal = THREADSIG_RUN;
    
#ifdef _LOGGER
    Logger::logoutput(Logger::LOG_THREADS, log_thread_creating, id);
#endif
    
    int rc = pthread_create(&thread, &attr, threadMain, (void *) &params);
    
    if (rc) {
        
#ifdef _LOGGER
        Logger::logoutput(Logger::LOG_THREADS, log_thread_creating_error, id, rc);
#endif
        
        cerr << "ERROR: Failed to create thread [" << id << "], pthread_create() returned " << rc << endl;
        
    } else {
        
#ifdef _LOGGER
        Logger::logoutput(Logger::LOG_THREADS, log_thread_created, id);
#endif
        
    }
    
    pthread_attr_destroy(&attr);
    return rc;
}

int Thread::stop() {
    
    signal = THREADSIG_STOP;
    status = 0;
    
#ifdef _LOGGER
    Logger::logoutput(Logger::LOG_THREADS, log_thread_joining, id);
#endif
    
    int rc = pthread_join(thread, (void **) &status);
    
    if (rc) {
        
#ifdef _LOGGER
        Logger::logoutput(Logger::LOG_THREADS, log_thread_joining_error, id, rc);
#endif
        cerr << "Failed to join thread [" << id << "], pthread_join() returned " << rc << endl;
        
    } else {
        
#ifdef _LOGGER
        Logger::logoutput(Logger::LOG_THREADS, log_thread_joined, id, status);
#endif
        
    }
    
    return rc;
}
