/*! \file Vector3D.h
<pre>
<b>File:</b> Vector3D.h
<b>Project:</b> Robocup 3D
<b>Authors:</b> Hviezdna Jedenastka
<b>Created:</b> 02/27/2008
<b>Contents: This class represents x,y,z coordinations of any object, and provides
			 useful converting methods between polar and planar system of coords. </b> <TEXT>
<hr size=2>
<h2><b>Changes</b></h2>
<b>Date</b>             <b>Author</b>          <b>Comment</b>
02/27/2008       Hviezdna Jedenastka           Prva verzia
</pre>
*/

#ifndef VECTOR3D_H_
#define VECTOR3D_H_

#include <iostream>
#include <math.h>

const double PI		= 3.141592653589793;	/*!< Pi constant */
const double DEG2PI = 1.745329251994e-02;	/*!< Pin in radians */

using namespace std;

/* Vector 3D class*/
class Vector3D {

public:

	double x;		/*!< x coord */
	double y;		/*!< y coord */
	double z;		/*!< z coord */
	
	Vector3D() {

		x = 0;
		y = 0;
		z = 0;
	}
	
	Vector3D(double _x, double _y, double _z) {

		x = _x;
		y = _y;
		z = _z;
	}

	/*!< overloaded operators =,+=,-=, ==, +, -, and * */
	
	inline Vector3D& operator = (const Vector3D& v3D) {
	
		x = v3D.x;
		y = v3D.y;
		z = v3D.z;
		
		return *this;
	}

	inline Vector3D& operator += (const double s) {
	
		x += s;
		y += s;
		z += s;
		
		return *this;
	}

	inline Vector3D& operator += (const Vector3D& v3D) {
	
		x += v3D.x;
		y += v3D.y;
		z += v3D.z;
		
		return *this;
	}

	inline Vector3D& operator -= (const double s) {
	
		x -= s;
		y -= s;
		z -= s;
		
		return *this;
	}

	inline Vector3D& operator -= (const Vector3D& v3D) {
	
		x -= v3D.x;
		y -= v3D.y;
		z -= v3D.z;
		
		return *this;
	}
	
	inline bool operator == (const Vector3D& v3D) const {
	
		return (x == v3D.x && y == v3D.y && z == v3D.z);
	}
	
	inline Vector3D operator + (const Vector3D& v3D) const {
	
		return Vector3D(x + v3D.x, y + v3D.y, z + v3D.z);
	}

	inline Vector3D operator - () const {
	
		return Vector3D(-x, -y, -z);
	}

	inline Vector3D operator - (const Vector3D& v3D) const {
	
		return Vector3D(x - v3D.x, y - v3D.y, z - v3D.z);
	}	
	
	inline Vector3D operator * (const Vector3D& v3D) const {
	
		return Vector3D(y * v3D.z - z * v3D.y, 
						z * v3D.x - x * v3D.z,
						x * v3D.y - y * v3D.x);
	}
	
	inline Vector3D operator * (double s) {
	
		return Vector3D(s * x, s * y, s * z); 
	}

	/*!< Conversion method to get planar coordinations from polar (easier for human to understand) */
	
	inline Vector3D& polarToPlanar() {
	
		double r = x;
		double theta = y;
		double phi = z;

		x = r * cos(theta * DEG2PI);
		y = r * sin(theta * DEG2PI);
		z = r * sin(phi * DEG2PI);
		
		return *this;
	}
	
	friend ostream& operator << (ostream& outStream, const Vector3D& v3D);
};

#endif /*VECTOR3D_H_*/
